/*
 * При обновлении не перезаписываются файлы, рядом с которыми лежит файл
 * с таким же именем плюс суффиксом '.orig'.
 * Также не перезаписываются классы в /WEB-INF/classes/, которые не находятся в поддиректории ru/bitel/.
 * /WEB-INF/classes/ru/bitel/ перезаписывается всегда.
 */
import java.net.*;
import java.nio.file.*;
import java.util.jar.*;
import java.util.zip.*;

import ru.bitel.common.io.*;

System.out.println( "Starting update MyBGBilling" );

File deploymentsDir = new File( "../../../../deployments" );

File configurationDir = new File( deploymentsDir.getParentFile(), "configuration" );
File binDir = new File( deploymentsDir.getParentFile().getParentFile(), "bin" );
File warDir = new File( deploymentsDir, "MyBGBilling.war" );

System.out.println( "deploymentsDir=" + warDir );

String version = null;

try {
    version = new Manifest( new FileInputStream( new File( warDir, "META-INF/MANIFEST.MF" ) ) )
    	.getMainAttributes()
    	.getValue( "Specification-Version", version );
} catch(Exception ex) {
    version = "beta";
}

URL url = new URL("https://bgbilling.ru/download/${version}/mybgbilling/MyBGBilling.last");

println ("Downloading MyBGBilling.war from " + url);

File distribution = File.createTempFile( "mybgbilling-distr", ".zip" );
println "distribution = " + distribution;
url.withInputStream{ i -> distribution.withOutputStream{ it << i }};
distribution.deleteOnExit();

println "Download OK"

// check
ZipInputStream zis = new ZipInputStream( new FileInputStream( distribution ) );
for( ZipEntry ze; (ze = zis.getNextEntry()) != null; ) {}
zis.close();

println "Check ZIP OK"

File classesDir = new File( warDir, "WEB-INF/classes/ru/bitel" );
classesDir.deleteDir();
classesDir.mkdirs();

Path tmpPath = Files.createTempFile( "zip-entry", ".tmp" );
zis = new ZipInputStream( new FileInputStream( distribution ) );
for( ZipEntry ze; (ze = zis.getNextEntry()) != null; ) {

	String name = ze.getName();

	// если папка создаем и переходим к следующему элементу архива
	if ( ze.isDirectory() ) {
		new File( deploymentsDir, name ).mkdirs();
		continue;
	}

	// копируем классы в папку MyBGBilling.war/WEB-INF/classes/ru/bitel
	if ( name.startsWith("MyBGBilling.war/WEB-INF/classes/ru/bitel") ) {
		Files.copy( zis, Paths.get( deploymentsDir.toPath().toString(), name ) );
		continue;
	}

	// текущая конфигурация, не перезаписываем
	if( name.startsWith( "MyBGBilling.war/WEB-INF/mybgbilling-" ) && name.endsWith( ".groovy" ) ) {
		continue;
	}

	// копируем файл из архива во временный файл (нужно, так как с файлам будем несколько операций, а это не удобно делать с ZipInputStream)
	Files.copy( zis, tmpPath, java.nio.file.StandardCopyOption.REPLACE_EXISTING );
	
	File oldFile;
	// копирование mybgbilling-menu.groovy
	if( name.startsWith( "MyBGBilling.war/WEB-INF/defaults" ) && name.endsWith( "mybgbilling-menu.groovy" ) ) {
		oldFile = new File( warDir, "WEB-INF/mybgbilling-menu.groovy.orig" );
		if( oldFile.exists() ) {
			if( !Arrays.equals( Files.readAllBytes( oldFile.toPath() ), Files.readAllBytes( tmpPath ) ) ) {
				println "Replace $oldFile"
				Files.copy( tmpPath, oldFile.toPath(), java.nio.file.StandardCopyOption.REPLACE_EXISTING );
			}
		} else {
			oldFile = new File( warDir, "WEB-INF/mybgbilling-menu.groovy" );
			File oldBakFile = new File( warDir, "WEB-INF/mybgbilling-menu.groovy.0bak" );
			if ( oldFile.exists() ) {
				if ( !Arrays.equals( Files.readAllBytes( oldFile.toPath() ), Files.readAllBytes( tmpPath ) ) ) {
					Files.copy( oldFile.toPath(), oldBakFile.toPath(), java.nio.file.StandardCopyOption.REPLACE_EXISTING );
					println "Replace $oldFile"
					Files.copy( tmpPath, oldFile.toPath(), java.nio.file.StandardCopyOption.REPLACE_EXISTING );
				}
			}
		}
	}

	oldFile = new File( deploymentsDir, name + ".orig" );
	if( oldFile.exists() ) {
		if( !Array.equals( Files.readAllBytes( oldFile.toPath ), Files.readAllBytes( tmpPath ) ) ) {
			println "Replace $oldFile"
			Files.copy( tmpPath, oldFile.toPath() );
		}
	} else {
		if( name.endsWith( ".orig" ) ) { 
			continue; 
		}

		oldFile = new File( deploymentsDir, name );
		if( oldFile.exists() ) {
			if( !Arrays.equals( Files.readAllBytes( oldFile.toPath() ), Files.readAllBytes( tmpPath ) ) ) {
				println "Replace $oldFile"
				Files.copy( tmpPath, oldFile.toPath(), java.nio.file.StandardCopyOption.REPLACE_EXISTING );
			} else {
				continue;
			}
		} else {
			println "Add $oldFile"
			Files.copy( tmpPath, oldFile.toPath() );
		}

//		if( name.startsWith( "MyBGBilling.war/WEB-INF/script/files" ) && name.endsWith( ".sh" ) && !name.endsWith( "setenv.sh" ) ) {
//			def newPath = Paths.get( binDir.getPath(), oldFile.getName() );
//			println "Replace $newPath"
//			Files.copy( Paths.get( oldFile.getPath() ), newPath, java.nio.file.StandardCopyOption.REPLACE_EXISTING );
//		}
	}
}

Files.deleteIfExists( tmpPath );