/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.common;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.GraphicsEnvironment;
import java.awt.RenderingHints;
import java.awt.font.FontRenderContext;
import java.awt.font.GlyphVector;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.OutputStream;
import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import javax.imageio.ImageIO;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import com.jhlabs.image.MarbleFilter;
import com.jhlabs.image.RippleFilter;

import ru.bitel.bgbilling.common.BGException;

public class CaptchaRenderer
{
	private static final Random random = new Random();
	private static final SecureRandom secureRandom = new SecureRandom();
	private static final Logger logger = LogManager.getLogger();
	private static final Font[] fonts;

	static
	{
		Font[] _fonts;
		try
		{
			List<Font> fontList = new ArrayList<Font>();

			GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
			for( String s : ge.getAvailableFontFamilyNames() )
			{
				if( "Verdana".equalsIgnoreCase( s ) )
				{
					fontList.add( new Font( "Verdana", Font.PLAIN, 30 ) );
					fontList.add( new Font( "Verdana", Font.PLAIN, 26 ) );
				}
				else if( "Courier".equalsIgnoreCase( s ) )
				{
					fontList.add( new Font( "Courier", Font.BOLD, 30 ) );
				}
				else if( "Arial".equalsIgnoreCase( s ) )
				{
					fontList.add( new Font( "Arial", Font.BOLD, 30 ) );
				}
				else if( "Lucida Sans Typewriter".equalsIgnoreCase( s ) )
				{
					new Font( "Lucida Sans Typewriter", Font.PLAIN, 30 );
				}
				else if( "Lucida Bright".equalsIgnoreCase( s ) )
				{
					new Font( "Lucida Bright", Font.ITALIC, 30 );
				}
				else if( "SansSerif".equalsIgnoreCase( s ) )
				{
					new Font( "SansSerif", Font.PLAIN, 30 );
				}
				else if( "Times New Roman".equalsIgnoreCase( s ) )
				{
					new Font( "Times New Roman", Font.PLAIN, 30 );
				}
				else if( "DejaVu LGC Sans".equalsIgnoreCase( s ) )
				{
					new Font( "DejaVu LGC Sans", Font.PLAIN, 30 );
				}
				else if( "DejaVu LGC Serif".equalsIgnoreCase( s ) )
				{
					new Font( "DejaVu LGC Serif", Font.PLAIN, 30 );
				}

				fontList.add( new Font( "Dialog", Font.PLAIN, 30 ) );
				fontList.add( new Font( "Dialog", Font.PLAIN, 26 ) );
			}

			_fonts = fontList.toArray( new Font[fontList.size()] );
		}
		catch( Throwable e )
		{
			_fonts = new Font[] { new Font( "Arial", Font.BOLD, 30 ), new Font( "Courier", Font.BOLD, 30 ),
								  new Font( "LucidaTypewriterBold", Font.PLAIN, 30 ),
								  new Font( "LucidaBrightItalic", Font.ITALIC, 30 ) };
			e.printStackTrace();
		}

		fonts = _fonts;
		if( logger.isDebugEnabled())
		{
		    logger.debug( "Load font for captcha.." );
		    for(Font f: fonts)
		    {
		       logger.debug( f.getName() +"["+f.getFamily()+"]");
		    }
		}
	}

//	private static final char[] symbols = new char[] { '0', '1', '2', '3', '4', '6', '7', '8', '9', 'A', 'B', 'C', 'E',
//													   'F', 'K', 'L', 'M', 'N', 'P', 'R', 'T', 'U', 'X', 'H', 'W', 'Y', 'V' };
	
	private static final char[] symbols = new char[] { '1', '2', '3', '4', '6', '7', '8', '9', 'A', 'B', 'C', 'E',
													   'F', 'K', 'L', 'M', 'N', 'P', 'R', 'T', 'U', 'X', 'H', 'W', 'Y', 'V',
													   'a', 'b', 'c', 'd', 'e', 'f', 'g', 'm', 'n', 'p', 's', 'u', 'x', 'h', 'w', 'y', 'v' };

	private static final char[] digits = new char[] { '0', '1', '2', '3', '4', '6', '7', '8', '9' };

	private static final Color[] colors = { Color.red, new Color( 0, 200, 0 ), Color.magenta, Color.orange };

	public static class CaptchaConfig
	{
		public boolean marble;
		public float marbleAmout;
		public float marbleTurbulence;

		public boolean ripple;
		public float rippleXAmplitude;
		public float rippleYAmplitude;
		public float rippleXWavelength;
		public float rippleYWavelength;

		public boolean rotate;

		public int lines;

		public CaptchaConfig()
		{
			marble = true;
			ripple = true;
			rotate = true;

			marbleAmout = 1.0f;
			marbleTurbulence = 0.09f;

			rippleXAmplitude = 16.0f;
			rippleYAmplitude = 1.3f;
			rippleXWavelength = 32.0f;
			rippleYWavelength = 32.0f;

			lines = 6;
		}
	}

	private CaptchaConfig captchaConfig;

	public CaptchaRenderer( CaptchaConfig captchaConfig )
	{
		this.captchaConfig = captchaConfig != null ? captchaConfig : new CaptchaConfig();
	}

	public CaptchaRenderer()
	{
		this.captchaConfig = new CaptchaConfig();
	}

	/*
	//старый
	captcha.marble=1
	captcha.marble.amount=2.0
	captcha.marble.turbulence=0.1
	captcha.ripple=1
	captcha.ripple.xamplitude=3.0
	captcha.ripple.yamplitude=2.3
	captcha.rotate=0
	
	//новый
	captcha.marble=1
	captcha.marble.amount=1.0
	captcha.marble.turbulence=0.01
	captcha.ripple=1
	captcha.ripple.xamplitude=6.0
	captcha.ripple.yamplitude=1.3
	captcha.ripple.xwavelength=32.0
	captcha.ripple.ywavelength=32.0
	captcha.rotate=1
	
	//вариация
	captcha.marble=0
	captcha.ripple.xamplitude=12.0
	captcha.ripple.yamplitude=6.0
	captcha.ripple.xwavelength=38.0
	captcha.ripple.ywavelength=32.0
	 */

	private static BufferedImage render( CaptchaConfig conf, String text )
	{
		BufferedImage result = renderText( text, 0, 0, conf.lines, conf.rotate );

		if( conf.marble )
		{
			MarbleFilter marble = new MarbleFilter();
			marble.setAmount( conf.marbleAmout );
			marble.setTurbulence(conf.marbleTurbulence );

			result = marble.filter( result, null );
		}
		
		if( conf.ripple )
		{
			RippleFilter ripple = new RippleFilter();
			ripple.setWaveType( RippleFilter.NOISE );
			ripple.setXAmplitude( conf.rippleXAmplitude-10 );
			ripple.setYAmplitude( conf.rippleYAmplitude );
			ripple.setXWavelength( conf.rippleXWavelength + random.nextInt( 5 ) - 2 );
			ripple.setYWavelength( conf.rippleYWavelength+ random.nextInt( 3 ) - 1 );

			result = ripple.filter( result, null );
		}
		
		/*CrystallizeFilter filter = new CrystallizeFilter();
		 filter.setAmount( 1 );
		filter.filter( result, null );
		
		WaterFilter water = new WaterFilter();
		water.setCentreX( 75 );
		water.setCentreY( 23 );
		water.setAmplitude( 10 );
		water.setRadius( 20 );
		water.setWavelength( 20 );
		result = water.filter( result, null );
		
		TwirlFilter twirl = new TwirlFilter();
		twirl.setCentreX( 20 );
		twirl.setCentreY( 20 );
		twirl.setRadius( 50 );
		twirl.setAngle( (float)0.9 );
		
		result = twirl.filter( result, null );*/

		BufferedImage result2 = new BufferedImage( 150, 45, BufferedImage.TYPE_4BYTE_ABGR );
		Graphics2D gfx = result2.createGraphics();
		gfx.drawImage( result, 0, 0, 150, 45, 35, 10, 155 + 40, 45 + 10, null );
		result = result2;

		return result;
	}

	private static BufferedImage renderText( String text, int width, int height, int lines, boolean rotate )
	{
		width = 200;
		height = 60;
		BufferedImage result = new BufferedImage( width, height, BufferedImage.TYPE_4BYTE_ABGR );

		Graphics2D gfx = result.createGraphics();
		RenderingHints hints = new RenderingHints( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
		hints.add( new RenderingHints( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY ) );
		gfx.setRenderingHints( hints );

		gfx.setBackground( Color.WHITE );
		gfx.clearRect( 0, 0, width, height );

		Color clr = colors[random.nextInt( colors.length )];
		gfx.setColor( new Color( Math.min( 255, clr.getRed() + 5 ), Math.min( 255, clr.getGreen() + 5 ), Math.min( 255, clr.getBlue() + 5 ) ) );
		
		//gfx.setStroke( new BasicStroke( (float)0.3 ) );
		
		for( int i = random.nextInt( lines - 1 ) + 1; i >= 0; i-- )
		{
			switch( random.nextInt( 2 ) )
			{
				case 1:
					gfx.drawLine( random.nextInt( width / 4 ) + 10, random.nextInt( height - 20 ) + 10, random.nextInt( width / 4 )
																										+ width - width / 4 - 10, random.nextInt( height - 20 ) + 10 );
					break;

				default:
					gfx.drawLine( random.nextInt( width - 20 ) + 10, random.nextInt( height - 20 ) + 10, random.nextInt( width - 20 ) + 10, random.nextInt( height - 20 ) + 10 );
					break;
			}
		}
		gfx.setColor( clr );
		

		char[] wc = text.toCharArray();
		FontRenderContext frc = gfx.getFontRenderContext();
		int startPosX = 40;

		AffineTransform oldTransform = gfx.getTransform();

		for( int i = 0; i < wc.length; i++ )
		{
			char[] itchar = new char[] { wc[i] };
			int choiceFont = random.nextInt( fonts.length );
			Font itFont = fonts[choiceFont];
			logger.debug( "Font is choosed for captcha.."+itFont.getName() +"["+itFont.getFamily()+"]" );
			gfx.setFont( itFont );
			GlyphVector gv = itFont.createGlyphVector( frc, itchar );

			if( rotate )
			{
				AffineTransform tx = new AffineTransform();
				tx.rotate( ((double)(random.nextInt( 9 ) - 4)) / 10, startPosX + gv.getVisualBounds().getCenterX(), 45 + gv.getVisualBounds().getCenterY() );
				gfx.setTransform( tx );
			}

			gfx.drawChars( itchar, 0, itchar.length, startPosX, 45 );
			startPosX = startPosX + (int)gv.getVisualBounds().getWidth() + 6;
		}

		gfx.setTransform( oldTransform );

		return result;
	}

	public String render( OutputStream out, boolean onlyDigits )
		throws BGException
	{
		final char[] key;
		if( onlyDigits )
		{
			key = digits;
		}
		else
		{
			key = symbols;
		}

		try
		{
			StringBuilder sb = new StringBuilder( 6 );
			for( int i = 0; i < 6; i++ )
				sb.append( key[secureRandom.nextInt( key.length )] );
			String result = sb.toString();

			ImageIO.write( render( this.captchaConfig, result ), "png", out );

			return sb.toString();
		}
		catch( IOException e )
		{
			throw new BGException( e );
		}
	}
}
