/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.common;

import java.io.UnsupportedEncodingException;
import java.util.Date;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.StringTokenizer;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.common.ParameterMap;
import ru.bitel.common.Utils;

public class MailMsg
{
	public final static String KEY_MAIL_SMTP_HOST = "mail.smtp.host";
	public final static String KEY_MAIL_FROM_EMAIL = "mail.from.email";
	public final static String KEY_MAIL_FROM_NAME = "mail.from.name";
	public final static String KEY_MAIL_TO_EMAIL = "mail.to.email";
	public final static String KEY_MAIL_TO_NAME = "mail.to.name";
	public final static String KEY_MAIL_ENCODING = "mail.encoding";

	private String result = "";
	private String encoding;
	private MimeMessage msg;

	private boolean _correctInit = false;

	public MailMsg( ParameterMap setup )
	{
		String fromAddress = setup.get( KEY_MAIL_FROM_EMAIL, null );
		String fromName = setup.get( KEY_MAIL_FROM_NAME, "BGBilling server" );
		encoding = setup.get( KEY_MAIL_ENCODING, "UTF-8" );

		String user = setup.get( "mail.smtp.user", null );
		String pswd = setup.get( "mail.smtp.pswd", null );

		if( fromAddress != null )
		{
			Session session = null;
			//
			Properties props = new Properties();
			props.setProperty( "mail.smtp.host", setup.get( KEY_MAIL_SMTP_HOST, null ) );
			props.setProperty( "mail.smtp.port", setup.get( "mail.smtp.port", "25" ) );
			props.setProperty( "mail.smtp.localhost", setup.get( "mail.smtp.localhost", "" ) );
			props.setProperty( "mail.debug", String.valueOf( setup.getInt( "mail.debug", 0 ) > 0 || "true".equals( setup.get( "mail.debug", null ) ) ) );

			// параметры mail, для ssl
			for( Entry<String, String> entry : setup.sub( "mail.properties." ).entrySet() )
			{
				props.put( entry.getKey(), entry.getValue() );
			}
			//
			Authenticator authenticator = null;
			if( Utils.notBlankString( user ) && Utils.notBlankString( pswd ) )
			{
				authenticator = new Authenticator( user, pswd );
				props.setProperty( "mail.smtp.auth", "true" );
				props.setProperty( "mail.smtp.submitter", authenticator.getPasswordAuthentication().getUserName() );
			}

			session = Session.getInstance( props, authenticator );

			msg = new MimeMessage( session );
			try
			{
				msg.setFrom( new InternetAddress( fromAddress, fromName, encoding ) );
				msg.setSentDate( new Date() );

				_correctInit = true;
			}
			catch( Exception ex )
			{
				ex.printStackTrace();
			}
		}
	}

	/**
	 * Проверяет, была ли корректной инициализация
	 * @return true - если корректна
	 */
	public boolean wasInitCorrectly()
	{
		return _correctInit;
	}

	/**
	 * Отправляет простое текствое письмо адресату
	 * @param recipients адресат
	 * @param subject тема
	 * @param text сообщение
	 * @throws MessagingException 
	 */
	public void sendMessage( String recipients, String subject, String text )
		throws BGException
	{
		Multipart multipart = new MimeMultipart();
		try
		{
			MimeBodyPart mimeBodyPart = new MimeBodyPart();
			mimeBodyPart.setText( text, encoding );
			multipart.addBodyPart( mimeBodyPart );
		}
		catch( MessagingException e )
		{
			throw new BGException( e );
		}
		sendMessage( recipients, subject, multipart );
	}

	/**
	 * Отправляет письмо адресату
	 * @param recipients адресат
	 * @param subject тема
	 * @param multipart тело сообщения
	 */
	public void sendMessage( String recipients, String subject, Multipart mp )
	{
		StringTokenizer st = new StringTokenizer( recipients, ";,\n\r" );
		while( st.hasMoreTokens() )
		{
			try
			{
				String mail = st.nextToken().trim();
				result += "Message send to " + mail;
				msg.setSubject( subject, encoding );
				msg.setContent( mp );
				InternetAddress[] addresses = InternetAddress.parse( mail );
				result += encodingPersonal( addresses );
				msg.setRecipients( Message.RecipientType.TO, addresses );
				Transport.send( msg );
				result += " => Ok\n";
			}
			catch( Exception e )
			{
				e.printStackTrace();
				result += " => Error: (" + e.toString() + ")\n";
			}
		}

	}

	public String getResult()
	{
		return result;
	}

	private static class Authenticator
		extends javax.mail.Authenticator
	{
		private PasswordAuthentication authentication;

		public Authenticator( String user, String password )
		{
			authentication = new PasswordAuthentication( user, password );
		}

		@Override
		protected PasswordAuthentication getPasswordAuthentication()
		{
			return authentication;
		}
	}

	private String encodingPersonal( InternetAddress[] addresses )
	{
		StringBuilder result = new StringBuilder();
		if( addresses != null )
		{
			for( InternetAddress address : addresses )
			{
				try
				{
					address.setPersonal( address.getPersonal(), "utf-8" );
				}
				catch( UnsupportedEncodingException uee )
				{
					result.append( "Error: " + uee.toString() + "\n" );
				}
			}
		}
		return result.toString();
	}
}
