/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.common;

import java.util.Iterator;
import java.util.ResourceBundle;

import javax.el.ELException;
import javax.enterprise.context.NonexistentConversationException;
import javax.enterprise.event.ObserverException;
import javax.faces.FacesException;
import javax.faces.application.FacesMessage;
import javax.faces.application.ViewExpiredException;
import javax.faces.context.ExceptionHandler;
import javax.faces.context.ExceptionHandlerWrapper;
import javax.faces.context.FacesContext;
import javax.faces.el.EvaluationException;
import javax.faces.event.ExceptionQueuedEvent;
import javax.faces.event.ExceptionQueuedEventContext;
import javax.faces.view.facelets.TagAttributeException;
import javax.xml.ws.WebServiceException;
import javax.xml.ws.soap.SOAPFaultException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.common.BGRuntimeException;
import ru.bitel.mybgbilling.kernel.navigation.NavigationBean;

public class MyExceptionHandler
	extends ExceptionHandlerWrapper
{
	private static final Logger logger = LogManager.getLogger();

	private ExceptionHandler exceptionHandler;

	public MyExceptionHandler( ExceptionHandler exceptionHandler )
	{
		this.exceptionHandler = exceptionHandler;
	}

	@Override
	public ExceptionHandler getWrapped()
	{
		return this.exceptionHandler;
	}

	@Override
	public void handle()
		throws FacesException
	{
		for( Iterator<ExceptionQueuedEvent> i = getUnhandledExceptionQueuedEvents().iterator(); i.hasNext(); )
		{
			ExceptionQueuedEvent exceptionQueuedEvent = i.next();

			ExceptionQueuedEventContext exceptionQueuedEventContext = (ExceptionQueuedEventContext)exceptionQueuedEvent.getSource();

			Throwable throwable = exceptionQueuedEventContext.getException();

			logger.info( "ERROR " + throwable.getClass() );

			if( throwable instanceof Throwable )
			{
				Throwable t = (Throwable)throwable;

				if( t.getClass().getName().equals( "org.jboss.weld.context.NonexistentConversationException" ) )
				{
					logger.error( t.getMessage() );
				}

				if( t instanceof TagAttributeException )
				{
					String message = t.getMessage();
					if( message != null && message.contains( "navigationBean.page" ) && message.contains( "Invalid path" ) )
					{
						final FacesContext context = FacesContext.getCurrentInstance();

						NavigationBean navigationBean = (NavigationBean)context.getELContext().getELResolver().getValue( context.getELContext(), null, "navigationBean" );
						if( navigationBean != null )
						{
							navigationBean.setPage( null );
						}
					}
				}

				if( t instanceof FacesException && t.getCause() != null )
				{
					t = t.getCause();
				}

				if( t instanceof FacesException && t.getCause() != null )
				{
					t = t.getCause();
				}

                if( t instanceof EvaluationException && t.getCause() != null )
				{
					t = t.getCause();
				}

				if( t instanceof ELException && t.getCause() != null )
				{
					t = t.getCause();
				}

				if( t instanceof ELException && t.getCause() != null )
				{
					t = t.getCause();
				}
				
				if( t instanceof ObserverException )
				{
					t = t.getCause();
				}

				if( t instanceof BGRuntimeException && t.getCause() != null )
				{
					t = t.getCause();
				}

				if( t instanceof BGException || t instanceof SOAPFaultException )
				{
					String tag;

					if( t instanceof BGException )
					{
						tag = ((BGException)t).getTag();
					}
					else
					{
						tag = null;
					}

					if( t.getCause() instanceof SOAPFaultException )
					{
						t = t.getCause();
					}

					//				FacesContext facesContext = FacesContext.getCurrentInstance();

					//				Map<String, Object> requestMap = facesContext.getExternalContext().getRequestMap();
					//				NavigationHandler navigationHandler = facesContext.getApplication().getNavigationHandler();

					try
					{
						FacesContext context = FacesContext.getCurrentInstance();

						String message = null;

						if( tag != null && tag.length() > 0 )
						{
							try
							{
								ResourceBundle bundle = context.getApplication().getResourceBundle( context, "msg" );
								message = bundle.getString( tag );
							}
							catch( Exception e )
							{
								logger.error( e.getMessage() );
							}
						}

						logger.info( String.format( "Error message: %s %s", message, t.getMessage() ) );

						if( t instanceof WebServiceException )
						{
							NavigationBean navigationBean = (NavigationBean)context.getELContext().getELResolver().getValue( context.getELContext(), null, "navigationBean" );
							if( navigationBean != null )
							{
								logger.error( "WebServiceException." );
								//logger.error( "WebServiceException. Set page to default." );
								//navigationBean.setPage( null );
							}
						}

						context.addMessage( null, new FacesMessage( FacesMessage.SEVERITY_ERROR, "Error Message", message != null ? message : t.getMessage() ) );

						//					requestMap.put( "currentView", t.getMessage() );
						//					facesContext.getExternalContext().getFlash().put( "exceptioniNFO", t.getCause() );
						//					navigationHandler.handleNavigation( facesContext, null, "/error?faces-redirect=true" );
						//					facesContext.renderResponse();

						i.remove();
					}
					finally
					{

					}
				}
                else if( t instanceof NonexistentConversationException
                         || (t.getClass() != null && t.getClass().getName().equals( "org.jboss.weld.context.NonexistentConversationException" )) )
				{
					FacesContext context = FacesContext.getCurrentInstance();
					
					logger.info( t.getMessage() );

					context.addMessage( null, new FacesMessage( FacesMessage.SEVERITY_ERROR, "Error Message", "Форма устарела, попробуйте снова." ) );
					
					if( context.getPartialViewContext().isAjaxRequest() )
					{
						//String errorPageLocation = "/index.xhtml";
						//context.setViewRoot( context.getApplication().getViewHandler().createView( context, errorPageLocation ) );

						FacesContext.getCurrentInstance().getPartialViewContext().getRenderIds().add( "page-wrapper" );
						//context.getPartialViewContext().ren
						//context.getPartialViewContext().setPartialRequest( false );
						//context.getPartialViewContext().setRenderAll( true );
						context.renderResponse();

						i.remove();
					}
				}
				else if( t instanceof ViewExpiredException )
				{
					FacesContext context = FacesContext.getCurrentInstance();

					logger.info( "EXPIRED" );

					context.addMessage( null, new FacesMessage( FacesMessage.SEVERITY_ERROR, "Error Message", "Форма устарела, попробуйте снова." ) );

					if( context.getPartialViewContext().isAjaxRequest() )
					{
						//String errorPageLocation = "/index.xhtml";
						//context.setViewRoot( context.getApplication().getViewHandler().createView( context, errorPageLocation ) );

						FacesContext.getCurrentInstance().getPartialViewContext().getRenderIds().add( "page-wrapper" );
						//context.getPartialViewContext().ren
						//context.getPartialViewContext().setPartialRequest( false );
						//context.getPartialViewContext().setRenderAll( true );
						context.renderResponse();

						i.remove();
					}
				}
			}
		}

		getWrapped().handle();
	}
}
