/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.common;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.enterprise.context.SessionScoped;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;
import javax.servlet.http.HttpServletRequest;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.container.ws.common.WSLinkedClient;
import ru.bitel.bgbilling.kernel.directory.api.client.ClientDirectoryFactory;
import ru.bitel.bgbilling.kernel.directory.api.common.bean.Directory;
import ru.bitel.common.model.Id;
import ru.bitel.mybgbilling.kernel.ws.Pool;
import ru.bitel.mybgbilling.kernel.ws.WSClient;

@Named
@SessionScoped
public class MySessionBean
	implements Serializable
{
	private static final long serialVersionUID = -1734745787438582284L;

	private WSClient wsClient;

	@Inject
	private MyApplicationBean application;

	public MySessionBean()
	{}

	@PostConstruct
	void init()
	{
		this.wsClient = application.getWsClient();

		final FacesContext context = FacesContext.getCurrentInstance();
		final HttpServletRequest request = (HttpServletRequest)context.getExternalContext().getRequest();

		this.needCaptcha = application.needCaptcha( application.getRemoteAddr( request ), null );
	}

	final static class Key
	{
		final Class<?> clazz;
		final int moduleId;

		public Key( Class<?> clazz, int moduleId )
		{
			this.clazz = clazz;
			this.moduleId = moduleId;
		}

		@Override
		public int hashCode()
		{
			return 31 * (31 + clazz.hashCode()) + moduleId;
		}

		@Override
		public boolean equals( final Object obj )
		{
			if( this == obj )
			{
				return true;
			}

			if( obj == null )
			{
				return false;
			}

			final Key other = (Key)obj;

			return clazz.equals( other.clazz ) && moduleId == other.moduleId;
		}
	}

	private Map<Key, Object> serviceMap = null;
	private Map<Key, Object> directoryMap = null;

	//	public <T> T getPort( Class<T> iface, int moduleId )
	//	{
	//		return wsClient.getPort( iface, mid );
	//	}

	// нельзя потому что CDI typesafe
	// или можно но через wrapper - @Inject Wrapper<ContractService> contractService;
	//	@Produces
	//	// @SessionScoped нельзя, потому что используем InjectionPoint - потому кэшируем сами
	//	@BGWebService
	//	public NewsService getPort( InjectionPoint injectionPoint, NavigationBean navigationBean )
	//	{
	//		return (NewsService)getPort( (Class<?>)injectionPoint.getType(), navigationBean.getModuleId() );
	//	}

	private static final ConcurrentHashMap<Key, Pool<Object>> PORT_POOL_MAP = new ConcurrentHashMap<Key, Pool<Object>>();

	@SuppressWarnings("unchecked")
	public <T> T getPort( final Class<T> iface, final int moduleId )
	{
		if( serviceMap == null )
		{
			serviceMap = new HashMap<Key, Object>();
		}

		final Key key = new Key( iface, moduleId );

		//T result = (T)serviceMap.get( key );
		
		final WSClient wsClient = this.wsClient;

        return (T)serviceMap.computeIfAbsent( key,
            ( key1 ) -> PORT_POOL_MAP.computeIfAbsent( key1, ( key2 ) -> new Pool<Object>( () -> wsClient.getPort( iface, moduleId ), 10 ) )
                .take() );

		/*
		final Map<String, Object> requestContext = ((BindingProvider)result).getRequestContext();
		Map<String, List<String>> requestHeaders = (Map<String, List<String>>)requestContext
																							.computeIfAbsent( MessageContext.HTTP_REQUEST_HEADERS,
																											  ( k ) -> new HashMap<String, List<String>>() );
		
		requestHeaders.put( "X-Real-Ip", Arrays.asList( "BILL_GATES" ) );
		 */

		/*if( result == null )
		{
			logger.info( "Create service " + iface.getName() + " for moduleId: " + moduleId );
		
			result = wsClient.getPort( iface, moduleId );
		
			serviceMap.put( key, result );
		}
		
		return result;*/
	}

	@PreDestroy
	void destroy()
	{
		for( Map.Entry<Key, Object> e : serviceMap.entrySet() )
		{
			//	logger.info( "Return " + iface + " " + moduleId );

			final WSClient wsClient = this.wsClient;

			PORT_POOL_MAP.computeIfAbsent( e.getKey(), ( key1 ) -> new Pool<Object>( () -> wsClient.getPort( key1.clazz, key1.moduleId ), 10 ) ).recycle( e.getValue() );
		}
		
		serviceMap.clear();
	}

	public void returnPort( final Class<?> iface, final int moduleId, final Object port )
	{
		//		final Key key = new Key( iface, moduleId );
		//		logger.info( "Return " + iface + " " + moduleId );
		//		pool.computeIfAbsent( key,
		//							  ( key1 ) -> new Pool<Object>(
		//															( ) -> wsClient.getPort( iface, moduleId ), 10 ) ).recycle( port );
	}

	private final WSLinkedClient cachedWsClient = new WSLinkedClient()
	{
		@Override
		public <T> T getPort( Class<T> iface, int moduleId )
		{
			return MySessionBean.this.getPort( iface, moduleId );
		}
	};

	public <K extends Id> Directory<K> getDirectory( Class<K> clazz, int moduleId )
		throws BGException
	{
		return ClientDirectoryFactory.newDirectory( cachedWsClient, clazz, moduleId, true );
	}

	@SuppressWarnings("unchecked")
	public <K extends Id> Directory<K> getDirectory( final Class<K> clazz, final int moduleId, final boolean cached )
		throws BGException
	{
		if( cached )
		{
			if( directoryMap == null )
			{
				directoryMap = new HashMap<Key, Object>( 8 );
			}

			final Key key = new Key( clazz, moduleId );
			Directory<K> result = (Directory<K>)directoryMap.get( key );

			if( result == null )
			{
				result = ClientDirectoryFactory.newDirectory( cachedWsClient, clazz, moduleId, true );

				directoryMap.put( key, result );
			}

			return result;
		}
		else
		{
			return ClientDirectoryFactory.newDirectory( cachedWsClient, clazz, moduleId, true );
		}
	}

	private boolean needCaptcha;

	public boolean isNeedCaptcha()
	{
		return needCaptcha;
	}

	public void setNeedCaptcha( boolean needCaptcha )
	{
		this.needCaptcha = needCaptcha;
	}

	public static MySessionBean get( FacesContext context )
	{
		return context.getApplication().evaluateExpressionGet( context, "#{mySessionBean}", MySessionBean.class );
	}
}
