/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.common;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.activation.DataSource;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;

import ru.bitel.bgbilling.common.bean.Writable;
import ru.bitel.bgbilling.common.bean.Request;
import ru.bitel.common.XMLUtils;
import ru.bitel.common.io.Base64;

/**
 * Обработка запроса - получение ответа. 
 */
public class TransferData
{
	private static final String ERROR_MSG = "Unexpected IO exception occured when posting the request!";
	private static final String ENCODING = "UTF-8";

	private URL url = null;
	private byte[] inBytes;
	private boolean debug = false;
	private String userName = null;
	private String userPswd = null;
	private Map<String, String> cookies = new HashMap<String, String>();
	private String str = "";
	private String encoding = ENCODING; // последняя кодировка которую удалось разобрать
	private String contentType = null; // последняя contentType которую удалось разобрать
	private String error = null;
	private String message = null;

	public TransferData( URL url )
	{
		this.url = url;
		this.debug = System.getProperty( "bgbilling.transfer.debug", "false" ).equals( "true" );
	}

	public void setUser( String name, String pswd )
	{
		userName = name;
		userPswd = pswd;
	}

	private static String encode( String inValue )
	{
		String outValue = "";
		try
		{
			outValue = URLEncoder.encode( inValue, ENCODING );
		}
		catch( UnsupportedEncodingException e )
		{}
		return outValue;
	}

	public void postData( Request request )
		throws IOException
	{
		postData( request, debug );
	}

	private static final Pattern charsetPattern = Pattern.compile( "charset=([\\w\\d\\-]+)[\\s;]*.*$" );

	/**
	 * Делаем запрос.
	 */
	private HttpURLConnection doConnection( Request request )
		throws IOException
	{
		String userAndPswd = userName + ":" + userPswd;
		final HttpURLConnection con = (HttpURLConnection)url.openConnection();
		con.setRequestMethod( "POST" );
		con.setRequestProperty( "Content-type", "application/x-www-form-urlencoded" );
		con.setRequestProperty( "Authorization", "Basic " + Base64.encode( userAndPswd ) );
		for( String name : cookies.keySet() )
		{
			con.setRequestProperty( "cookie", name + "=" + cookies.get( name ) );
		}
		con.setDoOutput( true );
		con.setDoInput( true );
		PrintStream ps = new PrintStream( con.getOutputStream(), true );
		// параметры установленные в запросе
		for( String key : request.keys() )
		{
			ps.print( key );
			ps.print( '=' );
			Object o = request.getValue( key );
			if( o instanceof Writable )
			{
				Writable writable = (Writable)o;

				if( writable.getWriterClass() == OutputStream.class )
				{
					writable.write( ps );
				}
				else if( writable.getWriterClass() == Writer.class )
				{
					writable.write( new URLWriter( ps ) );
				}
			}
			else
			{
				ps.print( encode( request.getValue( key ).toString() ) );
			}
			ps.print( '&' );
		}
		// добавление адреса сервера
		if( request.isSendServerURL() )
		{
			ps.print( "serverURL" );
			ps.print( "=" );
			ps.print( encode( url.toString() ) );
		}
		//
		if( ps.checkError() )
		{
			throw new IOException( ERROR_MSG );
		}
		//
		if( debug )
		{
			StringBuilder buf = new StringBuilder();
			for( String key : request.keys() )
			{
				buf.append( key );
				buf.append( '=' );
				Object o = request.getValue( key );
				if( o instanceof Writable )
				{
					buf.append( "..." );
				}
				else
				{
					buf.append( encode( request.getValue( key ).toString() ) );
				}
				buf.append( '&' );
			}
			// добавление адреса сервера
			if( request.isSendServerURL() )
			{
				buf.append( "serverURL" );
				buf.append( "=" );
				buf.append( encode( url.toString() ) );
			}
			System.err.println( url + "?" + (buf.length() < 20000 ? buf.toString() : buf.substring( 0, 20000 )) );
			buf = null;
		}
		request.clear();
		return con;
	}

	/**
	 * Разбираем куки и прочие параметры (кодировку, contentType) из запроса.
	 */
	private void parseCookieEtc( final HttpURLConnection con )
	{
		contentType = null;
		encoding = ENCODING;
		String key;
		for( int n = 1; (key = con.getHeaderFieldKey( n )) != null; n++ )
		{
			if( key.equalsIgnoreCase( "Set-Cookie" ) )
			{
				int i;
				String cookie = con.getHeaderField( n );
				if( cookie != null && (i = cookie.indexOf( ';' )) > -1 )
				{
					cookie = cookie.substring( 0, i );
				}
				if( cookie != null && (i = cookie.indexOf( '=' )) > -1 )
				{
					String name = cookie.substring( 0, i );
					String value = cookie.substring( i + 1 );
					cookies.put( name, value );
				}
			}
			else if( key.equalsIgnoreCase( "Content-Type" ) )
			{
				contentType = con.getHeaderField( n );
				Matcher m = charsetPattern.matcher( contentType );
				if( m.find() )
				{
					encoding = m.group( 1 );
				}
			}
		}

		// checkValue
		error = con.getHeaderField( "bgbilling-error" );
		message = con.getHeaderField( "bgbilling-message" );
	}

	public void postData( Request request, boolean debug )
		throws IOException
	{
		final HttpURLConnection con = doConnection( request );

		str = "";

		if( con.getResponseCode() == HttpURLConnection.HTTP_OK )
		{
			parseCookieEtc( con );

			int b = 0;

			Vector<byte[]> blocks = new Vector<byte[]>();
			int length = 0;
			byte[] in = new byte[1024];
			BufferedInputStream bis = new BufferedInputStream( con.getInputStream() );
			while( (b = bis.read( in, 0, 1024 )) != -1 )
			{
				length += b;
				byte[] block = new byte[b];
				System.arraycopy( in, 0, block, 0, b );
				blocks.add( block );
			}
			bis.close();
			con.disconnect();
			inBytes = new byte[length];
			int pos = 0;
			for( int i = 0; i < blocks.size(); i++ )
			{
				byte[] block = blocks.get( i );
				System.arraycopy( block, 0, inBytes, pos, block.length );
				pos += block.length;
			}
			str = new String( inBytes, encoding );
		}
	}

	public InputStream getInputStream( Request request )
		throws Exception
	{
		final HttpURLConnection con = doConnection( request );

		if( con.getResponseCode() == HttpURLConnection.HTTP_OK )
		{
			parseCookieEtc( con );

			str = "";
			BufferedInputStream bis = new BufferedInputStream( con.getInputStream() )
			{
				@Override
				public void close()
					throws IOException
				{
					super.close();
					con.disconnect();
				}
			};

			return bis;
		}

		return null;
	}

	public HTTPDataSource getDataSource( Request request )
		throws Exception
	{
		final HttpURLConnection con = doConnection( request );

		if( con.getResponseCode() == HttpURLConnection.HTTP_OK )
		{
			parseCookieEtc( con );
			str = "";
			return new HTTPDataSource( con, contentType, error == null ? false : true );
		}

		return null;
	}

	public Document getDocument( boolean debug )
	{
		Document doc = null;
		try
		{
			int len = str.length();
			if( debug )
			{
				String message = "[ length = " + len + " ] xml = ";
				//
				int debugMessageMaxLength = ru.bitel.common.Utils.parseInt( System.getProperty( "debug.message.max.length", "1500" ), 1500 );
				if( len > debugMessageMaxLength )
				{
					message += str.substring( 0, debugMessageMaxLength );
					message += "...";
				}
				else
				{
					message += str;
				}
				System.err.println( message );
			}
			// костыль: защита от того, что приходят кривые символы, чтобы просто парсер тупо не валился (нулевой символ может прийти в странных ситуациях HD#5692)
			str = replaceCharacterEntity( str );
			doc = XMLUtils.parseDocument( new InputSource( new StringReader( str.toString() ) ) );
		}
		catch( Exception ex )
		{
			ex.printStackTrace();
		}
		return doc;
	}

	private final static Pattern CHARACTER_ENTITY_INVALID_REGEXP =
		Pattern.compile( "&#0;|&#1;|&#2;|&#3;|&#4;|&#5;|&#6;|&#7;|&#8;|&#11;|&#12;|&#14;|&#15;|&#16;|&#17;|&#18;|&#19;|&#20;|&#21;|&#22;|&#23;|&#24;|&#25;|&#26;|&#27;|&#28;|&#29;|&#30;|&#31;" );

	/**
	 * Заменяет character entity вида &#XX; на вопросики, т.к. парсер не есть их. Хотя на сервере может такие символы сделать.
	 * Допустимые такие:
	 * Char       ::=      #x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD] | [#x10000-#x10FFFF]  / any Unicode character, excluding the surrogate blocks, FFFE, and FFFF. /
	 * Т.е. нельзя в десятичном: 0-8, 11, 12, 14-31
	 * Почему сервер генерит недопустимые - тут уже вопрос, пока костыль. TODO по-хорошему надо на сервере с этим как-то бороться, только пока недоразобрался в каком именно месте. 
	 */
	private static String replaceCharacterEntity( String xml )
	{
		return CHARACTER_ENTITY_INVALID_REGEXP.matcher( xml ).replaceAll( "?" );
	}

	public Document getDocument()
	{
		return getDocument( debug );
	}

	public byte[] getBytes()
	{
		return inBytes;
	}

	public String getString()
	{
		return str;
	}

	public class HTTPDataSource
		implements DataSource
	{
		private String contentType;
		private InputStream is;
		private boolean error = false;

		public boolean isError()
		{
			return error;
		}

		private HTTPDataSource( final HttpURLConnection con, final String contentType, boolean error )
		{
			this.contentType = contentType;
			this.error = error;
			try
			{
				is = new BufferedInputStream( con.getInputStream() )
				{
					@Override
					public void close()
						throws IOException
					{
						super.close();
						con.disconnect();
					}
				};
			}
			catch( IOException e )
			{
				e.printStackTrace();
			}
		}

		@Override
		public String getContentType()
		{
			return contentType;
		}

		@Override
		public InputStream getInputStream()
			throws IOException
		{
			return is;
		}

		@Override
		public String getName()
		{
			return null;
		}

		@Override
		public OutputStream getOutputStream()
			throws IOException
		{
			throw new UnsupportedOperationException();
		}

		public boolean isContentType( String contentType )
		{
			return contentType == null && this.contentType == null || (this.contentType != null && this.contentType.contains( contentType ));
		}

		public boolean isContentTypeTextXml()
		{
			return isContentType( "text/xml" );
		}
	}

	private class URLWriter
		extends Writer
	{
		private int i = 0;
		protected StringBuilder buf = new StringBuilder( 2042 );
		private PrintStream out;

		public URLWriter( PrintStream o )
			throws UnsupportedEncodingException
		{
			this.out = o;
		}

		@Override
		public void flush()
		{
			if( buf.length() > 0 )
			{
				out.print( encode( buf.toString() ) );
				buf.setLength( 0 );
				i = 0;
			}
		}

		@Override
		public void close()
			throws IOException
		{
			flush();
		}

		public void fl()
		{
			if( i >= 1024 )
			{
				out.print( encode( buf.toString() ) );
				buf.setLength( 0 );
				i = 0;
			}
		}

		@Override
		public void write( int c )
		{
			buf.append( (char)c );
			i++;
			fl();
		}

		@Override
		public void write( char cbuf[], int off, int len )
		{
			if( (off < 0) || (off > cbuf.length) || (len < 0) || ((off + len) > cbuf.length) || ((off + len) < 0) )
			{
				throw new IndexOutOfBoundsException();
			}
			else if( len == 0 )
			{
				return;
			}
			buf.append( cbuf, off, len );
			i += len;
			fl();
		}

		@Override
		public void write( String str )
		{
			buf.append( str );
			i += str.length();
			fl();
		}

		@Override
		public void write( String str, int off, int len )
		{
			buf.append( str.substring( off, off + len ) );
			i += len;
			fl();
		}
	}

	public String getMessage()
	{
		return message;
	}

	public String getError()
	{
		return error;
	}
}
