package ru.bitel.mybgbilling.kernel.common;

import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;
import java.util.jar.Attributes;
import java.util.jar.Manifest;

import javax.annotation.PostConstruct;
import javax.enterprise.context.ApplicationScoped;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

@Named
@ApplicationScoped
public class VersionBean
{
    private static final Logger logger = LogManager.getLogger();

    private String specificationVersion = "beta";
    private String implementationVersion = "beta";
    private String version = "beta";

    private String buildNumber;
    private String buildTimestamp;

    private String bgbillingImplementationVersion;

    @PostConstruct
    private void init()
    {
        try
        {
            Enumeration<URL> resources = getClass().getClassLoader().getResources( "META-INF/MANIFEST.MF" );

            while( resources.hasMoreElements() )
            {
                try
                {
                    URL url = resources.nextElement();
                    Manifest manifest = new Manifest( url.openStream() );
                    Attributes attrs = manifest.getMainAttributes();

                    if( !"MyBGBilling".equals( attrs.getValue( "Specification-Title" ) ) )
                    {
                        continue;
                    }

                    specificationVersion = attrs.getValue( "Specification-Version" );
                    implementationVersion = attrs.getValue( "Implementation-Version" );
                    version = specificationVersion;//.substring( 0, 1 ) + ".0";

                    buildNumber = attrs.getValue( "Build-Number" );
                    buildTimestamp = attrs.getValue( "Build-Timestamp" );

                    bgbillingImplementationVersion = attrs.getValue( "BGBilling-Implementation-Version" );

                    logger.info( "Implementation-Version: " + implementationVersion );
                    logger.info( "Build-Timestamp: " + buildTimestamp );
                }
                catch( IOException ex )
                {
                    logger.error( ex.getMessage(), ex );
                }
            }
        }
        catch( Exception ex )
        {
            logger.error( ex.getMessage(), ex );
        }
    }

    public String getSpecificationVersion()
    {
        return specificationVersion;
    }

    public String getImplementationVersion()
    {
        return implementationVersion;
    }

    public String getBuildNumber()
    {
        return buildNumber;
    }

    public String getBuildTimestamp()
    {
        return buildTimestamp;
    }

    public String getBGBillingImplementationVersion()
    {
        return bgbillingImplementationVersion;
    }

    public String getVersion()
    {
        return version;
    }
}
