/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.common.inject;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import javax.interceptor.InvocationContext;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.mybgbilling.kernel.common.MySessionBean;
import ru.bitel.mybgbilling.kernel.navigation.NavigationBean;

public abstract class InjectManager
{
	private static final Logger logger = LogManager.getLogger();

	private final ConcurrentMap<Class<?>, InjectPlan> map = new ConcurrentHashMap<Class<?>, InjectPlan>();

	public void process( final MySessionBean wsClientBean, final NavigationBean navigationBean, final InvocationContext ctx, final Object o )
	{
		InjectPlan plan = map.get( o.getClass() );
		if( plan != null )
		{
			if( plan != NULL )
			{
				try
				{
					plan.process( wsClientBean, navigationBean, ctx, o );
				}
				catch( Exception e )
				{
					logger.error( e.getMessage(), e );
				}

				return;
			}
			else
			{
				return;
			}
		}

		plan = buildPlan( o.getClass(), o );
		map.putIfAbsent( o.getClass(), plan );

		if( plan != NULL )
		{
			try
			{
				plan.process( wsClientBean, navigationBean, ctx, o );
			}
			catch( Exception e )
			{
				logger.error( e.getMessage(), e );
			}
		}
	}

	public void cleanup( final MySessionBean wsClientBean, final NavigationBean navigationBean, final InvocationContext ctx, final Object o )
	{
		InjectPlan plan = map.get( o.getClass() );
		if( plan != null )
		{
			if( plan != NULL )
			{
				try
				{
					plan.cleanup( wsClientBean, navigationBean, ctx, o );
				}
				catch( Exception e )
				{
					logger.error( e.getMessage(), e );
				}

				return;
			}
			else
			{
				return;
			}
		}
	}

	private static final InjectPlan NULL = new InjectPlan();

	protected InjectPlan buildPlan( Class<?> clazz, final Object o )
	{
		final List<InjectPlanField> fieldInjectionList = new ArrayList<InjectPlanField>();

		while( clazz != null && !clazz.equals( Object.class ) )
		{
			for( final Field f : clazz.getDeclaredFields() )
			{
				final InjectPlanField rule = buildInjection( o, f );
				if( rule != null )
				{
					fieldInjectionList.add( rule );
				}
			}

			clazz = clazz.getSuperclass();
		}

		if( fieldInjectionList.size() > 0 )
		{
			return new InjectPlan( fieldInjectionList );
		}

		return NULL;
	}

	protected abstract InjectPlanField buildInjection( Object o, Field f );
}
