/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.common.inject;

import java.io.File;
import java.io.IOException;
import java.lang.invoke.MethodHandles;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.PathMatcher;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Stream;

import javax.enterprise.event.Observes;
import javax.enterprise.inject.spi.AfterTypeDiscovery;
import javax.enterprise.inject.spi.BeanManager;
import javax.enterprise.inject.spi.BeforeBeanDiscovery;
import javax.enterprise.inject.spi.Extension;
import javax.enterprise.inject.spi.ProcessAnnotatedType;

import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.dynamic.common.bean.CompilationResult;
import ru.bitel.common.model.Pair;
import ru.bitel.mybgbilling.kernel.common.inject.CompilerBeanWrapper.CompiledUnit;

public class ProcessAnnotationsExtension
    extends ClassLoader
    implements Extension
{
	private static final Logger logger = LogManager.getLogger();

	private static Pattern modulePattern = Pattern.compile( "mybgbilling\\.modules\\.(\\w+)\\." );
	private static Pattern pluginPattern = Pattern.compile( "mybgbilling\\.plugins\\.(\\w+)\\." );

	private static Pattern paymentModulePattern = Pattern.compile( "mybgbilling\\.modules\\.payment\\.(\\w+)PaymentBean" );
	
    void beforeBeanDiscovery( @Observes BeforeBeanDiscovery bbd, BeanManager manager )
    {
        try
        {
            loadDynamicBeans( bbd, manager );
        }
        catch( Throwable e )
        {
            logger.log( Level.ERROR, e.getMessage(), e );
        }
    }

    private void loadDynamicBeans( BeforeBeanDiscovery bbd, BeanManager manager )
        throws URISyntaxException, IOException, BGException
    {
        URL url = this.getClass().getResource( this.getClass().getSimpleName() + ".class" );
        String path = url.toURI().getSchemeSpecificPart();

        path = path.substring( 0, path.lastIndexOf( "/WEB-INF/" ) + 9 );
        path = path + "dyn/";

        File beans = new File( path );

        final PathMatcher matcher = FileSystems.getDefault().getPathMatcher( "regex:.+\\.(java)\\z" );

        List<String> srcFiles = new ArrayList<String>();
        try( Stream<Path> paths = Files.walk( beans.toPath() ) )
        {
            paths.filter( Files::isRegularFile )
                 .filter( matcher::matches )
                 .forEach( a -> srcFiles.add( a.toString() ) );
        }
        
        CompilerBeanWrapper javac = new CompilerBeanWrapper( beans, new File( System.getProperty( "java.io.tmpdir" ) ) );      
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        Pair<CompilationResult, List<CompiledUnit>> result = javac.compile( srcFiles, loader );
        List<CompiledUnit> units = result.getSecond();
        
        for( CompiledUnit unit : units )
        {
            try
            {
                logger.info( "Found named bean " + unit.className );
                
                byte[] classBytes = Files.readAllBytes( Paths.get( unit.classFile.toURI() ) );
                MethodHandles.lookup().defineClass( classBytes );
                
                Class<?> clazz = loader.loadClass( unit.className );
                bbd.addAnnotatedType( manager.createAnnotatedType( clazz ), clazz.getName() );
            }
            catch( Exception e )
            {
                logger.log( Level.ERROR, unit.className + ": " + e.getMessage(), e );
            }
        }
    }
    
    public <T> void decorateAnnotatedType( @Observes ProcessAnnotatedType<T> pat )
    {
		final String className = pat.getAnnotatedType().getJavaClass().getName();

		logger.info( "Check : " + className );

		if( className.contains( "$" ) )
		{
			pat.veto();
			return;
		}

		String module = null;
		String plugin = null;

		Matcher m = paymentModulePattern.matcher( className );
		if( m.find() )
		{
			module = m.group( 1 ).toLowerCase();
		}
		else
		{
			m = modulePattern.matcher( className );
			if( m.find() )
			{
				module = m.group( 1 );
			}
			else
			{
				m = pluginPattern.matcher( className );
				if( m.find() )
				{
					plugin = m.group( 1 );
				}
			}
		}
		
		if( module != null && "voiceip|phone".contains( module ) )
		{}
		else if( module != null )
		{
			URL url = getClass().getClassLoader().getResource( "ru/bitel/bgbilling/modules/" + module );
			if( url == null )
			{
				logger.info( "Skip process (module not found): " + className );
				pat.veto();
			}
		}
		else if( plugin != null )
		{
			URL url = getClass().getClassLoader().getResource( "ru/bitel/bgbilling/plugins/" + plugin );
			if( url == null )
			{
				logger.info( "Skip process (plugin not found): " + className );
				pat.veto();
			}
		}

		//    pat.setAnnotatedType(pat.getAnnotatedType());
	}
    
    void afterTypeDiscovery( @Observes AfterTypeDiscovery atd )
    {

    }

	public void processAnnotatedType( @Observes ProcessAnnotatedType pat )
	{
		/*AnnotatedTypeBuilder<T> builder = new AnnotatedTypeBuilder<T>().readFromType(pat.getAnnotatedType());
		
		pat.getAnnotatedType().getMethods().forEach(method -> {
		    builder.addToMethod(method, interceptors);
		});*/

		//AnnotatedType<X>
/*
		if( pat.getAnnotatedType().isAnnotationPresent( Named.class ) && !pat.getAnnotatedType().isAnnotationPresent( BGInjection.class ) )
		{
			WrappedAnnotatedType wrapped = new WrappedAnnotatedType<>( pat.getAnnotatedType() );
			wrapped.getAnnotations().add( new BGInjection()
			{

				@Override
				public Class<? extends Annotation> annotationType()
				{
					return BGInjection.class;
				}
			} );

			pat.setAnnotatedType( wrapped );
		}
		
		*/
	}
	
}
