/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.common.inject;

import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.util.HashSet;
import java.util.Set;

import javax.enterprise.inject.spi.AnnotatedType;

public class WrappedAnnotatedType<X>
	implements AnnotatedType<X>
{
	private AnnotatedType wrappedAnnotatedType;
	private Set<Annotation> annotations;

	public WrappedAnnotatedType( AnnotatedType wrappedAnnotatedType )
	{
		this.wrappedAnnotatedType = wrappedAnnotatedType;
		this.annotations = new HashSet<Annotation>( wrappedAnnotatedType.getAnnotations().size() );
		this.annotations.addAll( wrappedAnnotatedType.getAnnotations() );
	}

	public Class getJavaClass()
	{
		return wrappedAnnotatedType.getJavaClass();
	}

	public Set getConstructors()
	{
		return wrappedAnnotatedType.getConstructors();
	}

	public Set getMethods()
	{
		return wrappedAnnotatedType.getMethods();
	}

	public Set getFields()
	{
		return wrappedAnnotatedType.getFields();
	}

	public Type getBaseType()
	{
		return wrappedAnnotatedType.getBaseType();
	}

	public Set<Type> getTypeClosure()
	{
		return wrappedAnnotatedType.getTypeClosure();
	}

	public <T extends Annotation> T getAnnotation( Class<T> annotationType )
	{
		for( Annotation ann : annotations )
		{
			if( ann.annotationType().equals( annotationType ) )
			{
				return (T)ann;
			}
		}

		return null;
	}

	public Set<Annotation> getAnnotations()
	{
		return annotations;
	}

	public boolean isAnnotationPresent( Class<? extends Annotation> annotationType )
	{
		if( wrappedAnnotatedType.isAnnotationPresent( annotationType ) )
		{
			return true;
		}

		for( Annotation ann : annotations )
		{
			if( ann.annotationType().equals( annotationType ) )
			{
				return true;
			}
		}

		return false;
	}
}
