/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.common.scope;

import java.lang.annotation.Annotation;

import javax.enterprise.context.spi.Context;
import javax.enterprise.context.spi.Contextual;
import javax.enterprise.context.spi.CreationalContext;
import javax.faces.context.FacesContext;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.common.ref.ConcurrentReferenceHashMap;
import ru.bitel.common.ref.ConcurrentReferenceHashMap.ReferenceType;
import ru.bitel.mybgbilling.kernel.navigation.NavigationBean;

public class SessionModuleScopeContext
	implements Context
{
	private static final Logger logger = LogManager.getLogger();

	@Override
	public boolean isActive()
	{
		//logger.info( "isActive" );

		return true;
	}

	private ConcurrentReferenceHashMap<String, SessionModuleScopeMap> map = new ConcurrentReferenceHashMap<String, SessionModuleScopeMap>( ReferenceType.STRONG, ReferenceType.WEAK );
	private ConcurrentReferenceHashMap<String, NavigationBean> navigationMap = new ConcurrentReferenceHashMap<String, NavigationBean>( ReferenceType.STRONG, ReferenceType.WEAK );

	@Override
	public <T> T get( final Contextual<T> contextual )
	{
		final FacesContext context = FacesContext.getCurrentInstance();

		if( context == null )
		{
			logger.info( "FacesContext is null! Are you try use CDI in async thread?" );
		}

		final String sessionId = context.getExternalContext().getSessionId( true );

		SessionModuleScopeMap map = this.map.get( sessionId );
		if( map == null )
		{
			return null;
		}

		NavigationBean navigationBean = navigationMap.get( sessionId );
		if( navigationBean == null )
		{
			navigationBean = (NavigationBean)context.getELContext().getELResolver().getValue( context.getELContext(), null, "navigationBean" );
			navigationMap.put( sessionId, navigationBean );
		}

		return map.get( contextual, navigationBean.getModuleId() );
	}

	@Override
	public <T> T get( final Contextual<T> contextual, final CreationalContext<T> creationalContext )
	{
		final FacesContext context = FacesContext.getCurrentInstance();

		final String sessionId = context.getExternalContext().getSessionId( true );

		SessionModuleScopeMap map = this.map.get( sessionId );
		if( map == null )
		{
			map = (SessionModuleScopeMap)context.getELContext().getELResolver().getValue( context.getELContext(), null, "sessionModuleScopeMap" );
			this.map.put( sessionId, map );
		}

		NavigationBean navigationBean = navigationMap.get( sessionId );
		if( navigationBean == null )
		{
			navigationBean = (NavigationBean)context.getELContext().getELResolver().getValue( context.getELContext(), null, "navigationBean" );
			navigationMap.put( sessionId, navigationBean );
		}

		T result = map.get( contextual, navigationBean.getModuleId() );
		if( result != null )
		{
			return result;
		}
		else
		{
			return map.create( contextual, creationalContext, navigationBean.getModuleId() );
		}
	}

	@Override
	public Class<? extends Annotation> getScope()
	{
		return SessionModuleScoped.class;
	}
}
