/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
 */
package ru.bitel.mybgbilling.kernel.common.scope;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.annotation.PreDestroy;
import javax.enterprise.context.spi.Contextual;
import javax.enterprise.context.spi.CreationalContext;
import javax.faces.view.ViewScoped;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

@Named
@ViewScoped
public class ViewModuleScopeMap
    implements Serializable
{
    private static final Logger logger = LogManager.getLogger();

    private static class InstanceInfo<T>
    {
        public CreationalContext<T> ctx;
        public T instance;
    }

    private Map<Integer, Map<Contextual, InstanceInfo>> moduleMap = new HashMap<Integer, Map<Contextual, InstanceInfo>>();

    void deactivate( Integer moduleId )
    {
        Map<Contextual, InstanceInfo> map = moduleMap.get( moduleId );
        if( map == null )
        {
            throw new RuntimeException( "WsScope with id =" + moduleId + " doesn't exist" );
        }
        Set<Contextual> keySet = map.keySet();
        for( Contextual contextual : keySet )
        {
            InstanceInfo instanceInfo = map.get( contextual );
            contextual.destroy( instanceInfo.instance, instanceInfo.ctx );
        }

        moduleMap.remove( moduleId );
    }

    @PreDestroy
    void destroy()
    {
        logger.info( "destroy" );

        try
        {
            for( Map<Contextual, InstanceInfo> map : moduleMap.values() )
            {
                Set<Contextual> keySet = map.keySet();
                for( Contextual contextual : keySet )
                {
                    InstanceInfo instanceInfo = map.get( contextual );
                    contextual.destroy( instanceInfo.instance, instanceInfo.ctx );
                }
            }

            moduleMap.clear();
        }
        catch( Throwable t )
        {
            logger.error( t.getMessage(), t );
        }
    }

    <T> T create( Contextual<T> contextual, CreationalContext<T> creationalContext, Integer moduleId )
    {
        T instance = contextual.create( creationalContext );
        InstanceInfo<T> info = new InstanceInfo<T>();
        info.ctx = creationalContext;
        info.instance = instance;

        Map<Contextual, InstanceInfo> map = moduleMap.get( moduleId );
        if( map == null )
        {
            map = new HashMap<Contextual, InstanceInfo>();
            moduleMap.put( moduleId, map );
        }

        map.put( contextual, info );

        return instance;
    }

    <T> T get( Contextual<T> contextual, Integer moduleId )
    {
        Map<Contextual, InstanceInfo> map = moduleMap.get( moduleId );
        if( map == null )
        {
            return null;
        }

        InstanceInfo<T> info = map.get( contextual );
        if( info == null )
        {
            return null;
        }

        return info.instance;
    }

    public boolean containsKey( Integer moduleId )
    {
        return moduleMap.containsKey( moduleId );
    }
}
