/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.common.utils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.DoubleSummaryStatistics;
import java.util.IntSummaryStatistics;
import java.util.LinkedList;
import java.util.List;
import java.util.LongSummaryStatistics;
import java.util.Map;

import javax.el.LambdaExpression;
import javax.enterprise.context.ApplicationScoped;
import javax.inject.Named;

@Named
@ApplicationScoped
public class Collectors
{
	public <E> Map<?, ?> toMap( Collection<E> c, LambdaExpression keyMapper, LambdaExpression valueMapper )
	{
		return (c != null ? c : Collections.<E> emptyList()).stream().collect( java.util.stream.Collectors.toMap( k -> keyMapper.invoke( k ), v -> valueMapper.invoke( v ) ) );
	}

	public <E> Map<?, ?> toMap( Collection<E> c, LambdaExpression keyMapper, LambdaExpression valueMapper, LambdaExpression mergeFunction )
	{
		return (c != null ? c : Collections.<E> emptyList()).stream().collect( java.util.stream.Collectors.toMap( k -> keyMapper.invoke( k ), v -> valueMapper.invoke( v ), ( a, b ) -> mergeFunction.invoke( a, b ) ) );
	}

	public <E> Map<?, List<E>> groupingBy( Collection<E> c, LambdaExpression classifier )
	{
		return (c != null ? c : Collections.<E> emptyList()).stream().collect( java.util.stream.Collectors.groupingBy( k -> classifier.invoke( k ) ) );
	}

	public <E> DoubleSummaryStatistics summarizingDouble( Collection<E> c, LambdaExpression mapper )
	{
		return (c != null ? c : Collections.<E> emptyList()).stream().collect( java.util.stream.Collectors.summarizingDouble( k -> (Double)mapper.invoke( k ) ) );
	}

	public <E> Double summingDouble( Collection<E> c, LambdaExpression mapper )
	{
		return (c != null ? c : Collections.<E> emptyList()).stream().collect( java.util.stream.Collectors.summingDouble( k -> (Double)mapper.invoke( k ) ) );
	}

	public <E> LongSummaryStatistics summarizingLong( Collection<E> c, LambdaExpression mapper )
	{
		return (c != null ? c : Collections.<E> emptyList()).stream().collect( java.util.stream.Collectors.summarizingLong( k -> (Long)mapper.invoke( k ) ) );
	}

	public <E> Long summingLong( Collection<E> c, LambdaExpression mapper )
	{
		return (c != null ? c : Collections.<E> emptyList()).stream().collect( java.util.stream.Collectors.summingLong( k -> (Long)mapper.invoke( k ) ) );
	}

	public <E> IntSummaryStatistics summarizingInt( Collection<E> c, LambdaExpression mapper )
	{
		return (c != null ? c : Collections.<E> emptyList()).stream().collect( java.util.stream.Collectors.summarizingInt( k -> (Integer)mapper.invoke( k ) ) );
	}

	public <E> Integer summingInt( Collection<E> c, LambdaExpression mapper )
	{
		return (c != null ? c : Collections.<E> emptyList()).stream().collect( java.util.stream.Collectors.summingInt( k -> (Integer)mapper.invoke( k ) ) );
	}

	public <E> List<E> toList( Collection<E> c )
	{
		if( c != null )
		{
			return new ArrayList<E>( c );
		}
		else
		{
			return new ArrayList<E>();
		}
	}

	public <E> ArrayList<E> toArrayList( Collection<E> c )
	{
		if( c != null )
		{
			return new ArrayList<E>( c );
		}
		else
		{
			return new ArrayList<E>();
		}
	}

	public <E> LinkedList<E> toLinkedList( Collection<E> c )
	{
		if( c != null )
		{
			return new LinkedList<E>( c );
		}
		else
		{
			return new LinkedList<E>();
		}
	}
}
