/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.common.utils;

import java.io.Serializable;
import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.TimeZone;

import javax.annotation.PostConstruct;
import javax.enterprise.context.SessionScoped;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.DateTimeConverter;
import javax.inject.Inject;
import javax.inject.Named;

import ru.bitel.mybgbilling.kernel.navigation.LocaleBean;

@Named
@SessionScoped
//@FacesConverter(value="localDateConverter")
public class LocalDateConverter
	extends DateTimeConverter
	implements Serializable
{
	private static final long serialVersionUID = 9202484096601485761L;

	@Inject
	private LocaleBean localeBean;

	@PostConstruct
	private void init()
	{
		this.setTimeZone( TimeZone.getDefault() );
	}

	@Override
	public Object getAsObject( FacesContext facesContext, UIComponent uiComponent, String value )
	{
		this.setLocale( localeBean.getLocale() );

		LocalDate ldate = null;
		Date date = null;
		Object o = super.getAsObject( facesContext, uiComponent, value );
		if( o == null )
		{
			return null;
		}

		if( o instanceof Date )
		{
			date = (Date)o;
			Instant instant = Instant.ofEpochMilli( date.getTime() );
			ldate = LocalDateTime.ofInstant( instant, ZoneId.systemDefault() ).toLocalDate();
			return ldate;
		}
		else
		{
			throw new IllegalArgumentException( String.format( "value=%s could not be converted to a LocalDate, result super.getAsObject=%s", value, o ) );
		}
	}

	@Override
	public String getAsString( FacesContext facesContext, UIComponent uiComponent, Object value )
	{
		this.setLocale( localeBean.getLocale() );

		if( value == null )
		{
			return super.getAsString( facesContext, uiComponent, value );
		}

		if( value instanceof LocalDate )
		{
			LocalDate lDate = (LocalDate)value;
			Instant instant = lDate.atStartOfDay().atZone( ZoneId.systemDefault() ).toInstant();
			Date date = Date.from( instant );
			return super.getAsString( facesContext, uiComponent, date );
		}
		else
		{
			throw new IllegalArgumentException( String.format( "value=%s is not a instanceof LocalDate", value ) );
		}
	}
}
