/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
 */
package ru.bitel.mybgbilling.kernel.common.utils;

import java.io.Serializable;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.Month;
import java.time.YearMonth;
import java.time.ZoneId;
import java.time.temporal.TemporalField;
import java.time.temporal.WeekFields;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import javax.enterprise.context.SessionScoped;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;

import ru.bitel.common.model.IdTitle;
import ru.bitel.mybgbilling.kernel.navigation.LocaleBean;

@Named
@SessionScoped
public class TimeUtils
    implements Serializable
{
    public boolean dateInRange( Date checking, Date dateFrom, Date dateTo )
    {
        return ru.bitel.common.TimeUtils.dateInRange( checking, dateFrom, dateTo );
    }

    @Inject
    private LocaleBean localeBean;

    public boolean dateBefore( Date date1, Date date2 )
    {
        return ru.bitel.common.TimeUtils.dateBefore( date1, date2 );
    }

    public boolean dateEquals( Date date1, Date date2 )
    {
        return ru.bitel.common.TimeUtils.dateEqual( date1, date2 );
    }

    public List<List<Integer>> daysTable( LocalDate date )
    {
        return daysTable( Date.from( date.atStartOfDay().atZone( ZoneId.systemDefault() ).toInstant() ) );
    }

    public List<List<Integer>> daysTable( Date date )
    {
        Calendar calendar = Calendar.getInstance();

        if( date != null )
        {
            calendar.setTime( date );
        }

        List<List<Integer>> result = new ArrayList<List<Integer>>( 5 );
        List<Integer> row = new ArrayList<Integer>( 10 );

        int min = calendar.getActualMinimum( Calendar.DATE );
        int max = calendar.getActualMaximum( Calendar.DATE );

        for( int i = min; i <= max; i++ )
        {
            row.add( i );

            if( i % 10 == 0 )
            {
                if( row.size() != 0 )
                {
                    result.add( row );
                }

                row = new ArrayList<Integer>( 10 );
            }
        }

        if( row.size() != 0 )
        {
            result.add( row );
        }

        return result;
    }

    private static final List<List<Integer>> hoursTable = hoursTable0();

    public List<List<Integer>> hoursTable()
    {
        return hoursTable;
    }

    private static List<List<Integer>> hoursTable0()
    {
        List<List<Integer>> result = new ArrayList<List<Integer>>( 5 );
        List<Integer> row = new ArrayList<Integer>( 10 );

        int min = 1;
        int max = 24;

        for( int i = min; i <= max; i++ )
        {
            row.add( i - 1 );

            if( i % 8 == 0 )
            {
                if( row.size() != 0 )
                {
                    result.add( Collections.unmodifiableList( row ) );
                }

                row = new ArrayList<Integer>( 10 );
            }
        }

        if( row.size() != 0 )
        {
            result.add( row );
        }

        return Collections.unmodifiableList( result );
    }

    public String format( Date date )
    {
        return format( date, null );
    }

    public String format( Date date, String pattern )
    {
        if( date == null )
        {
            return null;
        }

        final Locale locale = FacesContext.getCurrentInstance().getViewRoot().getLocale();

        final DateFormat format;

        if( pattern == null || "both".equals( pattern ) )
        {
            format = DateFormat.getDateTimeInstance( DateFormat.DEFAULT, DateFormat.DEFAULT, locale );
        }
        else if( "date".equals( pattern ) )
        {
            format = DateFormat.getDateInstance( DateFormat.DEFAULT, locale );
        }
        else if( "time".equals( pattern ) )
        {
            format = DateFormat.getTimeInstance( DateFormat.DEFAULT, locale );
        }
        else
        {
            format = new SimpleDateFormat( pattern, locale );
        }

        return format.format( date );
    }

    public boolean inSameMonth( LocalDate dateFrom, LocalDate dateTo, LocalDate date )
    {
        YearMonth d = YearMonth.from( date );

        if( dateFrom != null )
        {
            YearMonth from = YearMonth.from( dateFrom );
            if( from.isAfter( d ) )
            {
                return false;
            }
        }

        if( dateTo != null )
        {
            YearMonth to = YearMonth.from( dateTo );
            if( to.isBefore( d ) )
            {
                return false;
            }
        }

        return true;
    }

    public List<DayOfWeek> getDaysOfWeek()
    {
        final Locale locale = FacesContext.getCurrentInstance().getViewRoot().getLocale();
        final TemporalField fieldISO = WeekFields.of( locale ).dayOfWeek();

        LocalDate first = LocalDate.now().with( fieldISO, 1 );

        List<DayOfWeek> result = new ArrayList<>( 7 );

        DayOfWeek dayOfWeek = first.getDayOfWeek();

        result.add( dayOfWeek );

        for( int i = 0; i < 6; i++ )
        {
            dayOfWeek = DayOfWeek.of( (dayOfWeek.getValue() % 7) + 1 );
            result.add( dayOfWeek );
        }

        return result;
    }

    public List<IdTitle> getWeek()
    {
        return localeBean.getWeek();
    }

    public List<Month> getMonths()
    {
        return Arrays.asList( Month.values() );
    }
}
