/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.common.utils;

import java.security.SecureRandom;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.enterprise.context.ApplicationScoped;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.inject.Named;
import javax.security.auth.login.AccountNotFoundException;
import javax.security.auth.login.CredentialException;
import javax.security.auth.login.FailedLoginException;

//import org.jboss.security.SecurityContextAssociation;

@Named
@ApplicationScoped
public class Utils
	extends CommonUtils
{
	private final static SecureRandom random = new SecureRandom();

	public static String escapeXmlNotValid( StringBuilder buf, String value )
	{
		char ch;

		boolean was = false;

		buf.setLength( 0 );

		for( int j = 0; j < value.length(); j++ )
		{
			ch = value.charAt( j );
			int ich = ch;

			// То что считается правильным xml-символом по стандарту
			if( (ich >= 0x20 && ich <= 0xD7FF) || ich == 0x9 || ich == 0xA || ich == 0xD
				|| (ich >= 0xE000 && ich <= 0xFFFD) || (ich >= 0x10000 && ich <= 0x10FFFF) )
			{
				if( was )
				{
					buf.append( ch );
				}
			}
			else
			{
				if( !was )
				{
					was = true;

					buf.append( value, 0, j );
				}

				buf.append( "\\u" );
				buf.append( ich );
			}
		}

		if( !was )
		{
			return value;
		}

		return buf.toString();
	}

	public static void main( String[] args )
	{
		System.out.println( escapeXmlNotValid( new StringBuilder(), "действительно символ \"\"." ) );
		System.out.println( escapeXmlNotValid( new StringBuilder(), "действително символ \"." ) );
	}

	public static String randomString( int length )
	{
		String chars = "1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
		int charsLength = chars.length();

		StringBuilder sb = new StringBuilder();

		for( int i = 0; i < length; i++ )
		{
			sb.appendCodePoint( chars.codePointAt( random.nextInt( charsLength ) ) );
		}

		return sb.toString();
	}

	public String escapeClientId( String id )
	{
		return id != null ? id.replaceAll( ":", "\\\\:" ) : null;
	}

	public String escapeClientId2( String id )
	{
		return id != null ? id.replaceAll( ":", "\\\\\\\\:" ) : null;
	}

	public List<Integer> integerRange( int min, int max )
	{
		final List<Integer> result = new ArrayList<Integer>( max - min );

		for( int i = min; i <= max; i++ )
		{
			result.add( i );
		}

		return result;
	}

	public String format( String pattern, Object arg1 )
	{
		return MessageFormat.format( pattern, arg1 );
	}

	public String format( String pattern, Object arg1, Object arg2 )
	{
		return MessageFormat.format( pattern, arg1, arg2 );
	}

	public String format( String pattern, Object arg1, Object arg2, Object arg3 )
	{
		return MessageFormat.format( pattern, arg1, arg2, arg3 );
	}

	public String getLoginFailureMsg()
	{
		String result = getLoginFailureMsg0();
		if( result != null )
		{
			FacesContext.getCurrentInstance().addMessage( "username", new FacesMessage( FacesMessage.SEVERITY_ERROR, "Error Message", result ) );
		}

		return result;
	}

	private String getLoginFailureMsg0()
	{
		Exception e = null;//(Exception)SecurityContextAssociation.getContextInfo( "org.jboss.security.exception" );

		if( e == null )
		{
			return null;
		}

		if( e.getCause() instanceof AccountNotFoundException )
		{
			return e.getCause().getLocalizedMessage();
		}

		if( e instanceof AccountNotFoundException )
		{
			return e.getLocalizedMessage();
		}

		if( e.getCause() instanceof CredentialException )
		{
			return e.getCause().getLocalizedMessage();
		}

		if( e instanceof CredentialException )
		{
			return e.getLocalizedMessage();
		}

		if( e.getCause() instanceof FailedLoginException )
		{
			return e.getCause().getLocalizedMessage();
		}

		if( e instanceof FailedLoginException )
		{
			return e.getLocalizedMessage();
		}

		return e.getLocalizedMessage();
	}
	
	public static void addTagErrorMessage( final FacesContext context, final String clientId, String tag, String message )
	{
		ResourceBundle rb = context.getApplication().getResourceBundle( context, "msg" );

		if( tag != null && tag.length() > 0 )
		{
			try
			{
				message = rb.getString( tag );
			}
			catch( MissingResourceException e )
			{
				try
				{
					message = rb.getString( message );
				}
				catch( MissingResourceException e2 )
				{}
			}
		}
		else
		{
			try
			{
				message = rb.getString( message );
			}
			catch( MissingResourceException e )
			{}
		}

		FacesMessage faceMessage = new FacesMessage( FacesMessage.SEVERITY_ERROR, "Error Message", message );
		context.addMessage( clientId, faceMessage );
	}

	public static void addErrorMessage( final FacesContext context, final String clientId, String message )
	{
		try
		{
			message = context.getApplication().getResourceBundle( context, "msg" ).getString( message );
		}
		catch( MissingResourceException e )
		{}

		FacesMessage faceMessage = new FacesMessage( FacesMessage.SEVERITY_ERROR, "Error Message", message );
		context.addMessage( clientId, faceMessage );
	}

	public static void addErrorMessage( final FacesContext context, final String clientId, String message, Object... arguments )
	{
		String pattern;
		try
		{
			pattern = context.getApplication().getResourceBundle( context, "msg" ).getString( message );
		}
		catch( MissingResourceException e )
		{
			pattern = message;
		}

		message = MessageFormat.format( pattern, arguments );

		FacesMessage faceMessage = new FacesMessage( FacesMessage.SEVERITY_ERROR, "Error Message", message );
		context.addMessage( clientId, faceMessage );
	}

	public static void addInfoMessage( final FacesContext context, final String clientId, String message )
	{
		try
		{
			message = context.getApplication().getResourceBundle( context, "msg" ).getString( message );
		}
		catch( MissingResourceException e )
		{}

		FacesMessage faceMessage = new FacesMessage( FacesMessage.SEVERITY_INFO, "Info Message", message );
		context.addMessage( clientId, faceMessage );
	}

	public static void addInfoMessage( final FacesContext context, final String clientId, String message, Object... arguments )
	{
		String pattern;
		try
		{
			pattern = context.getApplication().getResourceBundle( context, "msg" ).getString( message );
		}
		catch( MissingResourceException e )
		{
			pattern = message;
		}

		message = MessageFormat.format( pattern, arguments );

		FacesMessage faceMessage = new FacesMessage( FacesMessage.SEVERITY_INFO, "Info Message", message );
		context.addMessage( clientId, faceMessage );
	}

	public UIComponent findComponent( final UIComponent c, final String id )
	{
		if( id.equals( c.getId() ) )
		{
			return c;
		}

		Iterator<UIComponent> kids = c.getFacetsAndChildren();
		while( kids.hasNext() )
		{
			UIComponent found = findComponent( kids.next(), id );
			if( found != null )
			{
				return found;
			}
		}

		return null;
	}

	public Object anyNotNull( Object arg1, Object arg2 )
	{
		return anyNotNull0( arg1, arg2 );
	}

	public Object anyNotNull( Object arg1, Object arg2, Object arg3 )
	{
		return anyNotNull0( arg1, arg2, arg3 );
	}

	public Object anyNotNull( Object arg1, Object arg2, Object arg3, Object arg4 )
	{
		return anyNotNull0( arg1, arg2, arg3, arg4 );
	}

	public Object anyNotNull( Object arg1, Object arg2, Object arg3, Object arg4, Object arg5 )
	{
		return anyNotNull0( arg1, arg2, arg3, arg4, arg5 );
	}

	private Object anyNotNull0( Object... args )
	{
		for( Object o : args )
		{
			if( o != null )
			{
				return o;
			}
		}

		return null;
	}
}
