/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.common.utils;

import java.io.Serializable;
import java.text.ParseException;
import java.time.LocalDate;
import java.time.YearMonth;
import java.time.ZoneId;
import java.util.Calendar;
import java.util.Date;

import javax.enterprise.context.SessionScoped;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.inject.Inject;
import javax.inject.Named;

import ru.bitel.mybgbilling.kernel.navigation.LocaleBean;

//@FacesConverter(value = "monthConverter")
@Named
@SessionScoped
public class YearMonthConverter
	implements Converter, Serializable
{
	private static final long serialVersionUID = 666760418862653057L;

	@Inject
	private LocaleBean localeBean;

	@Override
	public Object getAsObject( FacesContext context, UIComponent component, String value )
	{
		Date date = parse( context, component, value );
		if( date == null )
		{
			date = null;
		}

		return YearMonth.from( date.toInstant().atZone( ZoneId.systemDefault() ) );
	}

	public Date parse( FacesContext context, UIComponent component, String value )
	{
		try
		{
			return localeBean.getMonthFormat().parse( value );
		}
		catch( ParseException ex )
		{
			try
			{
				return localeBean.getMonthFormat2().parse( value );
			}
			catch( ParseException ex1 )
			{
				try
				{
					return localeBean.getMonthFormat3().parse( value );
				}
				catch( ParseException ex2 )
				{
					try
					{
						return localeBean.getDateFormat().parse( value );
					}
					catch( ParseException ex3 )
					{}
				}
			}
		}

		return null;
	}

	@Override
	public String getAsString( FacesContext context, UIComponent component, Object value )
	{
		if( value instanceof LocalDate )
		{
			value = Date.from( ((LocalDate)value).atStartOfDay().atZone( ZoneId.systemDefault() ).toInstant() );
		}
		else if( value instanceof YearMonth )
		{
			value = Date.from( ((YearMonth)value).atDay( 1 ).atStartOfDay().atZone( ZoneId.systemDefault() ).toInstant() );
		}

		if( value instanceof Date )
		{
			return localeBean.getMonthFormat().format( value );
		}
		else if( value instanceof Calendar )
		{
			Calendar calendar = (Calendar)value;

			return localeBean.getMonthFormat().format( calendar.getTime() );
		}

		return null;
	}
}
