package ru.bitel.mybgbilling.kernel.contract;

import java.util.List;
import java.util.stream.Collectors;

import javax.enterprise.context.RequestScoped;
import javax.inject.Inject;
import javax.inject.Named;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.common.BGMessageException;
import ru.bitel.bgbilling.kernel.contract.script.common.bean.AdditionalContractAction;
import ru.bitel.bgbilling.kernel.contract.script.common.bean.AdditionalContractActionResult;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.Configuration;
import ru.bitel.mybgbilling.kernel.menu.MenuBean;
import ru.bitel.mybgbilling.kernel.menu.MenuItem;

/**
 * Активация модуля на договоре с помощью доп. действия.
 * 
 * @author amir
 */
@Named
@RequestScoped
public class ActivateModuleBean
    extends AbstractBean
{
    @Inject
    private Configuration configuration;

    @Inject
    private MenuBean menuBean;

    @Inject
    private AdditionalActionBean additionalActionBean;

    private List<AdditionalContractAction> activateActions;

    @Override
    protected void init()
        throws BGException
    {
        List<Integer> activateModuleActions = configuration.getActivateModuleActions()
            .get( this.getModuleId() );

        this.activateActions = additionalActionBean.getAllAdditionalContractActionList()
            .stream()
            .filter( action -> activateModuleActions.contains( action.getId() ) )
            .collect( Collectors.toList() );
    }

    public List<AdditionalContractAction> getActivateActions()
    {
        return activateActions;
    }

    public void invoke( int actionId )
        throws BGMessageException, BGException
    {
        additionalActionBean.invoke( actionId );
    }

    public AdditionalContractActionResult getResult()
    {
        return additionalActionBean.getResult();
    }

    public void finish()
    {
        additionalActionBean.setResult( null );

        // переключаемся на страницу модуля
        MenuItem menuItem = menuBean.getMenuItemForModule( getModuleId() );
        if( menuItem != null )
        {
            navigationBean.setPage( menuItem.getPage(), menuItem.getModuleId(), menuItem.getSubPage() );
        }
    }
}
