/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
 */
package ru.bitel.mybgbilling.kernel.contract;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import javax.enterprise.context.SessionScoped;
import javax.enterprise.inject.Instance;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;
import javax.servlet.http.HttpServletRequest;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.common.BGMessageException;
import ru.bitel.bgbilling.kernel.contract.script.common.service.ContractScriptService;
import ru.bitel.bgbilling.kernel.contract.script.common.bean.AdditionalContractAction;
import ru.bitel.bgbilling.kernel.contract.script.common.bean.AdditionalContractActionResult;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Lazy;
import ru.bitel.common.model.MapHolder;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.Configuration;
import ru.bitel.mybgbilling.kernel.common.MyApplicationBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.menu.MenuBean;

/**
 * Доп. действия.
 * 
 * @see 
 */
@Named
@SessionScoped
@BGInjection
public class AdditionalActionBean
    extends AbstractBean
{
    @Inject
    private MyApplicationBean application;

    @Inject
    private Configuration configuration;

    @BGInject( module = false )
    private ContractScriptService contractScriptService;

    @Inject
    private Instance<MenuBean> menuBean;

    @Inject
    private BalanceBean balanceBean;

    private Supplier<List<AdditionalContractAction>> allAdditionalContractActionList;
    
    private Supplier<List<AdditionalContractAction>> additionalContractActionList;

    private AdditionalContractActionResult additionalActionResult;

    @Override
    protected void init()
        throws BGException
    {
        super.init();
        
        populate();
    }

	public void populate()
	{
        final HttpServletRequest request = (HttpServletRequest)FacesContext.getCurrentInstance().getExternalContext().getRequest();

        final String remoteAddr = application.getRemoteAddr( request );

        Map<String, String> params =
            request.getParameterMap().entrySet().stream().filter( a -> a.getKey().startsWith( "javax." ) || a.getValue() == null || a.getValue().length == 0 )
                   //.map( a -> new AbstractMap.SimpleEntry<>( a.getKey(), a.getValue()[0] ) )
                   .collect( Collectors.toMap( a -> a.getKey(), a -> a.getValue()[0] ) );

        params.put( "remoteAddr", remoteAddr );
	    
        // действия активации модуля
        final Set<Integer> activateActions = configuration.getActivateModuleActions()
            .values().stream()
            .flatMap( list -> list.stream() )
            .collect( Collectors.toSet() );
        
        allAdditionalContractActionList = Async.of( () -> Utils.maskNull( contractScriptService.additionalContractActionList( getContractId(), true, new MapHolder<>( params ) ) ) );
        additionalContractActionList = Lazy.of( () -> allAdditionalContractActionList.get()
            .stream()
            .filter( action -> !activateActions.contains( action.getId() ) ) // убираем из списка действий активации модуля - они отображаются отдельно
            .collect( Collectors.toList() ) );
    }

    public List<AdditionalContractAction> getAdditionalContractActionList()
        throws BGException
    {
        return additionalContractActionList.get();
    }

    public List<AdditionalContractAction> getAllAdditionalContractActionList()
        throws BGException
    {
        return allAdditionalContractActionList.get();
    }

    public AdditionalContractActionResult additionalContractActionInvoke( int actionId )
        throws BGMessageException, BGException
    {
        try
        {
            final HttpServletRequest request = (HttpServletRequest)FacesContext.getCurrentInstance().getExternalContext().getRequest();

            final String remoteAddr = application.getRemoteAddr( request );

            Map<String, String> params = request.getParameterMap().entrySet().stream()
                .filter( a -> a.getKey().startsWith( "javax." ) || a.getValue() == null || a.getValue().length == 0 )
                // .map( a -> new AbstractMap.SimpleEntry<>( a.getKey(), a.getValue()[0] ) )
                .collect( Collectors.toMap( a -> a.getKey(), a -> a.getValue()[0] ) );

            params.put( "remoteAddr", remoteAddr );

            return contractScriptService.additionalContractActionInvoke( getContractId(), actionId, true, new MapHolder<>( params ) );
        }
        finally
        {
            try
            {
                customerBean.populate();

                balanceBean.populateCurrentContractBalance();

                menuBean.get().reloadMenu();
            }
            finally
            {
                populate();
            }
        }
    }

    public void invoke( int actionId )
        throws BGMessageException, BGException
    {
        additionalActionResult = additionalContractActionInvoke( actionId );
    }

    public AdditionalContractActionResult getResult()
    {
        return additionalActionResult;
    }

    public void setResult( AdditionalContractActionResult additionalActionResult )
    {
        this.additionalActionResult = additionalActionResult;
    }
}
