/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.contract;

import static java.time.temporal.TemporalAdjusters.firstDayOfMonth;
import static java.time.temporal.TemporalAdjusters.lastDayOfMonth;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.YearMonth;
import java.time.ZoneId;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.function.Supplier;

import javax.enterprise.context.SessionScoped;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.AjaxBehaviorEvent;
import javax.inject.Inject;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.contract.balance.common.bean.Charge;
import ru.bitel.bgbilling.kernel.contract.balance.common.bean.ChargeType;
import ru.bitel.bgbilling.kernel.contract.balance.common.bean.ContractAccount;
import ru.bitel.bgbilling.kernel.contract.balance.common.bean.ContractBalance;
import ru.bitel.bgbilling.kernel.contract.balance.common.bean.Payment;
import ru.bitel.bgbilling.kernel.contract.balance.common.bean.PaymentType;
import ru.bitel.bgbilling.kernel.contract.balance.common.service.BalanceService;
import ru.bitel.bgbilling.kernel.contract.balance.common.service.ChargeService;
import ru.bitel.bgbilling.kernel.contract.balance.common.service.PaymentService;
import ru.bitel.bgbilling.kernel.directory.api.common.bean.Directory;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Expirable;
import ru.bitel.common.function.ThrowingSupplier;
import ru.bitel.common.model.Period;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.ws.CustomerBean;

@Named
@SessionScoped
@BGInjection
public class BalanceBean
	extends AbstractBean
	implements Serializable
{
	private static final Logger logger = LogManager.getLogger();

	@Inject
	private CustomerBean customerBean;

	@Inject
	private LimitBean limitBean;

	@BGInject(module = false)
	private BalanceService balanceService;

	@BGInject(module = false)
	private PaymentService paymentService;

	@BGInject(module = false)
	private ChargeService chargeService;

	private Date month;

	private Supplier<ContractBalance> currentContractBalance;

	private Async<ContractBalance> contractBalance;

	private Async<List<Payment>> paymentList;

	private Async<List<Charge>> chargeList;

	private Async<List<ContractAccount>> contractAccountList;

	@BGInject(type = PaymentType.class, module = false)
	private Directory<PaymentType> paymentTypeDirectory;

	@BGInject(type = ChargeType.class, module = false)
	private Directory<ChargeType> chargeTypeDirectory;

	@Override
	protected void init()
		throws BGException
	{
		super.init();

		month = new Date();

		populate();

//		logger.info( System.getProperty( "jboss.server.base.dir" ) );
//		logger.info( System.getProperty( "jboss.home.dir" ) );
//		logger.info( System.getProperty( "jboss.server.base.dir" ) );
//		logger.info( System.getProperty( "jboss.server.config.dir" ) );
	}

	public Date getMonth()
	{
		return month;
	}

	public void setMonth( Date month )
	{
		logger.info( "setMonth" );

		if( month == null )
		{
			month = new Date();
		}

		this.month = month;
	}

	public void ajaxListener( AjaxBehaviorEvent event )
	{
		logger.info( "ajaxListener" );
	}

	public void actionListener( ActionEvent event )
	{
		// Or in some ActionEvent method (e.g. <h:commandXxx actionListener>).
		logger.info( "ajaxListener" );
	}

	public void populate()
		throws BGException
	{
		logger.info( "populate" );
		
		final Calendar calendar = new GregorianCalendar();
		calendar.setTime( month );

		final YearMonth nowYearMonth = YearMonth.now();

		contractBalance = Async.of( () -> {

			final int year = calendar.get( Calendar.YEAR );
			final int month = calendar.get( Calendar.MONTH ) + 1;

			ContractBalance result = balanceService.contractBalanceGet( getContractId(), year, month );
			
			if( year != nowYearMonth.getYear() || month != nowYearMonth.getMonthValue() )
			{
				result.setReserve( BigDecimal.ZERO );
			}

			if( result.getYear() != year || result.getMonth() != month )
			{
				ContractBalance newResult = new ContractBalance();
				newResult.setYear( year );
				newResult.setMonth( month );
				newResult.setIncomingSaldo( result.toBalanceWithoutReserves() );
				newResult.setPayments( BigDecimal.ZERO );
				newResult.setCharges( BigDecimal.ZERO );
				newResult.setAccounts( BigDecimal.ZERO );
				newResult.setReserve( result.getReserve() );

				return newResult;
			}

			return result;
		} );

		//		LocalDate date = month.toInstant().atZone( ZoneId.systemDefault() ).toLocalDate();
		//
		//		YearMonth yearMonth = YearMonth.from( date );
		//		final YearMonth currentYearMonth = YearMonth.now();
		//
		//		if( currentYearMonth.equals( yearMonth ) )
		//		{
		//			currentContractBalance = contractBalance;
		//		}
		//		else
		//		{
		//			currentContractBalance = Async.of( () -> {
		//
		//				final int year = currentYearMonth.getYear();
		//				final int month = currentYearMonth.getMonthValue();
		//
		//				ContractBalance result = balanceService.contractBalanceGet( getContractId(), year, month );
		//
		//				if( result.getYear() != year || result.getMonth() != month )
		//				{
		//					ContractBalance newResult = new ContractBalance();
		//					newResult.setYear( year );
		//					newResult.setMonth( month );
		//					newResult.setIncomingSaldo( result.toBalance() );
		//					newResult.setPayments( BigDecimal.ZERO );
		//					newResult.setCharges( BigDecimal.ZERO );
		//					newResult.setAccounts( BigDecimal.ZERO );
		//					newResult.setReserve( BigDecimal.ZERO );
		//
		//					return newResult;
		//				}
		//
		//				return result;
		//			} );
		//		}

		LocalDate dateFrom = month.toInstant().atZone( ZoneId.systemDefault() ).toLocalDate().with( firstDayOfMonth() );
		LocalDate dateTo = dateFrom.with( lastDayOfMonth() );

		Period period = new Period( Date.from( dateFrom.atStartOfDay( ZoneId.systemDefault() ).toInstant() ),
									Date.from( dateTo.atStartOfDay( ZoneId.systemDefault() ).toInstant() ) );

		paymentList = Async.of( () -> paymentService.paymentList( getContractId(), period, 1, null ) );

		chargeList = Async.of( () -> chargeService.chargeList( getContractId(), -1, period, 1, null ) );

		contractAccountList = Async.of( () -> balanceService.contractAccountList( getContractId(), period, 1, null ) );

		LocalDate date = month.toInstant().atZone( ZoneId.systemDefault() ).toLocalDate();

		YearMonth yearMonth = YearMonth.from( date );
		final YearMonth currentYearMonth = YearMonth.now();

		if( currentYearMonth.equals( yearMonth ) )
		{
			currentContractBalance = Expirable.of( contractBalance, currentContractBalance(), 120, TimeUnit.SECONDS );
		}

		limitBean.populate();
	}

	private ThrowingSupplier<ContractBalance> currentContractBalance()
	{
		return () -> {

			final YearMonth currentYearMonth = YearMonth.now();

			final int year = currentYearMonth.getYear();
			final int month = currentYearMonth.getMonthValue();

			ContractBalance result = balanceService.contractBalanceGet( getContractId(), year, month );

			if( result.getYear() != year || result.getMonth() != month )
			{
				ContractBalance newResult = new ContractBalance();
				newResult.setYear( year );
				newResult.setMonth( month );
				newResult.setIncomingSaldo( result.toBalance() );
				newResult.setPayments( BigDecimal.ZERO );
				newResult.setCharges( BigDecimal.ZERO );
				newResult.setAccounts( BigDecimal.ZERO );
				newResult.setReserve( BigDecimal.ZERO );

				return newResult;
			}

			return result;

		};
	}

	public void populateCurrentContractBalance()
	{
		currentContractBalance = Expirable.asyncOf( currentContractBalance(), 120, TimeUnit.SECONDS );
	}

	public ContractBalance getCurrentContractBalance()
		throws BGException
	{
		return currentContractBalance.get();
	}

	public ContractBalance getContractBalance()
		throws BGException
	{
		if( true ) return contractBalance.get();

		if( !FacesContext.getCurrentInstance().getRenderResponse() )
		{
			return null;
		}

		final Calendar calendar = new GregorianCalendar();
		calendar.setTime( month );

		final int year = calendar.get( Calendar.YEAR );
		final int month = calendar.get( Calendar.MONTH ) + 1;
		logger.info( "asdasd" );
		ContractBalance result = balanceService.contractBalanceGet( customerBean.getContractId(), year, month );

		if( result.getYear() != year || result.getMonth() != month )
		{
			ContractBalance newResult = new ContractBalance();
			newResult.setYear( year );
			newResult.setMonth( month );
			newResult.setIncomingSaldo( result.toBalance() );
			newResult.setPayments( BigDecimal.ZERO );
			newResult.setCharges( BigDecimal.ZERO );
			newResult.setAccounts( BigDecimal.ZERO );
			newResult.setReserve( BigDecimal.ZERO );

			result = newResult;
		}

		return result;
	}

	public List<Payment> getPaymentList()
		throws BGException
	{
		return paymentList.get();
	}

	public List<Charge> getChargeList()
		throws BGException
	{
		return chargeList.get();
	}

	public List<ContractAccount> getContractAccountList()
		throws BGException
	{
		return contractAccountList.get();
	}

	public ChargeType getChargeType( int id )
		throws BGException
	{
		return chargeTypeDirectory.get( id );
	}

	public PaymentType getPaymentType( int id )
		throws BGException
	{
		return paymentTypeDirectory.get( id );
	}
}
