package ru.bitel.mybgbilling.kernel.contract;

import static ru.bitel.bgbilling.kernel.contract.api.common.bean.Contract.*;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.YearMonth;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import javax.enterprise.context.ConversationScoped;
import javax.inject.Inject;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.contract.api.common.bean.Contract;
import ru.bitel.bgbilling.kernel.contract.balance.common.bean.BalanceDistributeData;
import ru.bitel.bgbilling.kernel.contract.balance.common.bean.ContractBalance;
import ru.bitel.bgbilling.kernel.contract.balance.common.service.BalanceService;
import ru.bitel.common.function.Async;
import ru.bitel.mybgbilling.kernel.common.AbstractConversationBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.ws.CustomerBean;

@Named
@ConversationScoped
@BGInjection
public class BalanceDistributeBean
	extends AbstractConversationBean
	implements Serializable
{
	private static final Logger logger = LogManager.getLogger();

	@Inject
	private CustomerBean customerBean;

	@Inject
	private BalanceBean balanceBean;

    @BGInject( module = false )
	private BalanceService balanceService;

	private Map<Integer, BalanceDistributeData> balanceDistributeDataMap = new HashMap<>();

	private Map<Integer, BigDecimal> balanceMap = new HashMap<>();

	@Override
	protected void init()
		throws BGException
	{
		logger.info( "BalanceDistributeBean.init" );

		super.init();

		populateBalance();
	}

	public void populateBalance()
		throws BGException
	{
		this.balanceDistributeDataMap.clear();

		Map<Integer, Supplier<BigDecimal>> balanceMap = new HashMap<>();

		for( Contract subContract : customerBean.getSubContractList() )
		{
			if( subContract.getBalanceSubMode() != SUB_MODE_INDEPENDENT )
			{
				continue;
			}

			BalanceDistributeData data = new BalanceDistributeData();
			data.setContractId( subContract.getId() );
			data.setAmout( BigDecimal.ZERO );

			this.balanceDistributeDataMap.put( subContract.getId(), data );

			final YearMonth currentYearMonth = YearMonth.now();
			final int year = currentYearMonth.getYear();
			final int month = currentYearMonth.getMonthValue();

			balanceMap.put( subContract.getId(), Async.of( () -> {

				ContractBalance result = balanceService.contractBalanceGet( subContract.getId(), year, month );

				if( result.getYear() != year || result.getMonth() != month )
				{
					ContractBalance newResult = new ContractBalance();
					newResult.setYear( year );
					newResult.setMonth( month );
					newResult.setIncomingSaldo( result.toBalance() );
					newResult.setPayments( BigDecimal.ZERO );
					newResult.setCharges( BigDecimal.ZERO );
					newResult.setAccounts( BigDecimal.ZERO );
					newResult.setReserve( BigDecimal.ZERO );

					return subContract.getBalanceMode() == Contract.CREDIT_BALANCE_MODE ? newResult.toSaldo() : newResult.toBalance();
				}

				return subContract.getBalanceMode() == Contract.CREDIT_BALANCE_MODE ? result.toSaldo() : result.toBalance();
			} ) );
		}

		this.balanceMap.clear();

		for( Map.Entry<Integer, Supplier<BigDecimal>> e : balanceMap.entrySet() )
		{
			this.balanceMap.put( e.getKey(), e.getValue().get() );
		}
	}

	public Map<Integer, BalanceDistributeData> getBalanceDistributeDataMap()
	{
		return balanceDistributeDataMap;
	}

	public Map<Integer, BigDecimal> getBalanceMap()
	{
		return balanceMap;
	}

	public void beginDistribute()
	{
		navigationBean.setPage( "kernel/balanceDistribute", 0 );
	}

	public void distribute()
		throws BGException
	{
		List<BalanceDistributeData> balanceDistributeDataList =
			balanceDistributeDataMap.values().stream().filter( a -> a.getAmout() != null && BigDecimal.ZERO.compareTo( a.getAmout() ) < 0 )
									.collect( Collectors.toList() );

		if( balanceDistributeDataList.size() == 0 )
		{
			addErrorMessage( null, "Укажите суммы для распределения." );
			return;
		}

		balanceService.contractBalanceTransfer( getContractId(), new Date(), balanceDistributeDataList, true );

		balanceDistributeDataMap.values().forEach( a -> a.setAmout( BigDecimal.ZERO ) );

		balanceBean.populateCurrentContractBalance();
		populateBalance();
	}
}
