/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.contract;

import java.io.Serializable;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.function.Supplier;

import javax.activation.DataHandler;
import javax.enterprise.context.SessionScoped;
import javax.inject.Named;
import javax.xml.ws.Holder;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.common.BGIllegalAccessException;
import ru.bitel.bgbilling.kernel.contract.api.common.bean.ContractCard;
import ru.bitel.bgbilling.kernel.contract.api.common.service.ContractCardService;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Expirable;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;

@Named
@SessionScoped
@BGInjection
public class ContractCardBean
	extends AbstractBean
	implements Serializable
{
	private static final Logger logger = LogManager.getLogger();

	@BGInject(module = false)
	private ContractCardService contractCardService;

	private Supplier<List<ContractCard>> contractCardList;

	@Override
	protected void init()
		throws BGException
	{
		super.init();

		contractCardList = Expirable.asyncOf( () -> contractCardService.contractCardList( getContractId() ), 120, TimeUnit.SECONDS );
	}

	public List<ContractCard> getContractCardList()
		throws BGException
	{
		return contractCardList.get();
	}

	public void populate()
		throws BGException
	{
		logger.info( "populate -> nothing" );
	}

	public void exportContractCard( String contractCardId )
		throws BGException
	{
		if( Utils.isBlankString( contractCardId ) )
		{
			throw new BGIllegalAccessException();
		}

		populate();

		ContractCard contractCard = contractCardList.get().stream().filter( ( a ) -> a.getId().equals( contractCardId ) ).findFirst().orElse( null );

		if( contractCard == null )
		{
			throw new BGIllegalAccessException();
		}

		final Holder<DataHandler> holder = new Holder<DataHandler>();

		contractCardService.contractCardExport( getContractId(), contractCardId, holder );

		//String titleCard = Base64.decode( Base64.decodeWeb( ge ), "utf-8" );

		//имя PDF файла
		StringBuilder fileName = new StringBuilder();
		fileName.append( contractCard.getTitle() );
		fileName.append( ".pdf" );

		writeToOutput( holder.value, fileName.toString(), false );
	}
}
