/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.contract;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import java.util.function.Supplier;

import javax.enterprise.context.SessionScoped;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.contract.api.common.bean.Contract;
import ru.bitel.bgbilling.kernel.contract.api.common.bean.parameter.ContractParameterGroup;
import ru.bitel.bgbilling.kernel.contract.api.common.bean.parameter.ContractParameterGroupAttr;
import ru.bitel.bgbilling.kernel.contract.api.common.bean.parameter.ContractParameterListItem;
import ru.bitel.bgbilling.kernel.contract.api.common.service.ContractParameterService;
import ru.bitel.bgbilling.kernel.contract.api.common.service.ContractService;
import ru.bitel.bgbilling.kernel.directory.api.common.bean.Directory;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Expirable;
import ru.bitel.common.function.Lazy;
import ru.bitel.common.function.ThrowingSupplier;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.oss.kernel.entity.common.bean.EntityAttr;
import ru.bitel.oss.kernel.entity.common.bean.EntityAttrEmail;
import ru.bitel.oss.kernel.entity.common.bean.EntityAttrPhone;

@Named
@SessionScoped
@BGInjection
public class ParameterBean
	extends AbstractBean
	implements Serializable
{
    private static final long serialVersionUID = 4565132112940433567L;

    private static final Logger logger = LogManager.getLogger();

    @BGInject(type = ContractParameterGroup.class, module = false)
	private Directory<ContractParameterGroup> contractParameterGroupDirectory;
	
    private Map<Integer, Async<List<ContractParameterListItem>>> сontractParameterListItemListMap = new HashMap<>();
    
    @BGInject(module = false)
    private ContractParameterService contractParameterService;

	@BGInject(module = false)
	private ContractService contractService;

	private Supplier<ContractParameterGroup> contractParameterGroup;

	/**
	 * Все параметры договора.
	 */
	private Supplier<Map<Integer, EntityAttr>> contractParameterMap;

	private Supplier<Map<Integer, Supplier<Map<Integer, EntityAttr>>>> subContractsParameterMap;

	/**
	 * Параметры договора, которые могут быть видны абоненту.
	 */
	private Supplier<Map<Integer, EntityAttr>> customerContractParameterMap;

	private int contractParameterId;

	private EntityAttr contractParameter;

	@Override
	protected void init()
		throws BGException
	{
		super.init();

		final int paramGroupId = customerBean.getContract().getParamGroupId();
		logger.info( "paramGroupId=" + paramGroupId );

		contractParameterGroup = Expirable.asyncOf( () -> contractParameterGroupDirectory.get( paramGroupId ), 3, TimeUnit.MINUTES );

		populate();
	}

	public void populate()
		throws BGException
	{
		final int paramGroupId = customerBean.getContract().getParamGroupId();

        contractParameterMap =
            Lazy.of( () -> {
                try
                {
                    return Optional.ofNullable( contractService.contractParameterMap( getContractId(), paramGroupId ) ).map( ( a ) -> a.getMap() ).orElse( null );
                }
                catch( Exception ex )
                {
                    logger.error( ex.getMessage(), ex );
                }

                return new HashMap<>( 4 );
            } );

		customerContractParameterMap = Lazy.of( () -> {

			Map<Integer, EntityAttr> contractParameterMap = getFullContractParameterMap();
			Map<Integer, EntityAttr> map = new HashMap<>();

			if( getContractParameterGroup() != null && getContractParameterGroup().getAttributeList() != null )
			{
				getContractParameterGroup().getAttributeList().stream().map( ( a ) -> (ContractParameterGroupAttr)a ).filter( a -> a.isCustomerRead() || a.isCustomerWrite() ).forEach( a -> {

					EntityAttr attr = contractParameterMap.get( a.getId() );
					if( a != null )
					{
						map.put( a.getId(), attr );
					}
				} );
			}

			/*for( EntitySpecAttr specAttr : getContractParameterGroup().getAttributeList() )
			{
				ContractParameterGroupAttr attr = (ContractParameterGroupAttr)specAttr;
			
				if( attr.isCustomerRead() )
				{
					EntityAttr a = getContractParameterMap().get( attr.getId() );
					if( a != null )
					{
						map.put( attr.getId(), a );
					}
				}
			
			}*/

			return map;
		} );

		Async.of( () -> {
			contractParameterMap.get();
			customerContractParameterMap.get();
		} );

		final List<Contract> subContractList = customerBean.getSubContractList();

		subContractsParameterMap = Lazy.of( () -> {

			Map<Integer, Supplier<Map<Integer, EntityAttr>>> result = new HashMap<>();

			int i = 0;

			for( Contract contract : subContractList )
			{
                final ThrowingSupplier<Map<Integer, EntityAttr>> supplier = () -> {
                    try
                    {
                        return Optional.ofNullable( contractService.contractParameterMap( contract.getId(), contract.getParamGroupId() ) )
                                       .map( ( a ) -> a.getMap() ).orElse( null );
                    }
                    catch( Exception ex )
                    {
                        logger.error( ex.getMessage(), ex );
                    }

                    return new HashMap<>( 4 );
                };

				if( i++ < 5 )
				{
					result.put( contract.getId(), Async.of( supplier ) );
				}
				else
				{
					result.put( contract.getId(), Lazy.of( supplier ) );
				}
			}

			return result;
		} );
	}
	
    public List<ContractParameterListItem> getContractParameterListItemList( int parameterTypeId )
        throws BGException
    {
        return сontractParameterListItemListMap.computeIfAbsent( parameterTypeId, key -> Async.of( () -> {
            List<ContractParameterListItem> result = Utils.maskNull( contractParameterService.contractParameterListItemList( parameterTypeId ) );
            Collections.sort( result, ( a, b ) -> Utils.maskNull( a.getTitle() ).compareTo( Utils.maskNull( b.getTitle() ) ) );
            return result;
        } ) ).get();
    }

	public ContractParameterGroup getContractParameterGroup()
	{
		return contractParameterGroup.get();
	}

	/**
	 * Получение всех параметров договора.
	 * @return
	 */
	public Map<Integer, EntityAttr> getFullContractParameterMap()
	{
		return contractParameterMap.get();
	}

	/**
	 * @deprecated use {@link #getFullContractParameterMap()}
	 * @return
	 */
	@Deprecated
	public Map<Integer, EntityAttr> getContractParameterMap()
	{
		return contractParameterMap.get();
	}

	/**
	 * Получение параметров договора, которые может видеть или редактировать абонент.
	 * @return
	 */
	public Map<Integer, EntityAttr> getCustomerContractParameterMap()
	{
		return customerContractParameterMap.get();
	}

	public Map<Integer, EntityAttr> getSubContractParameterMap( final int contractId )
	{
		final Supplier<Map<Integer, EntityAttr>> supplier = subContractsParameterMap.get().get( contractId );

		return supplier != null ? supplier.get() : null;
	}

	public int getContractParameterId()
	{
		return contractParameterId;
	}

	public EntityAttr getContractParameter()
	{
		return contractParameter;
	}

	/**
	 * Редактирование параметра.
	 * @param id
	 * @throws BGException
	 */
	public void editParam( int id )
		throws BGException
	{
		ContractParameterGroupAttr specAttr =
			(ContractParameterGroupAttr)getContractParameterGroup().getAttributeList().stream().filter( a -> a.getId() == id ).findFirst().orElse( null );
		if( specAttr == null || !specAttr.isCustomerWrite() )
		{
			addErrorMessage( null, "Данный параметр редактировать запрещено." );
			return;
		}

		EntityAttr contractParameter = null;

		// если параметр читать абоненту нельзя - просто создаем новый параметр - он может перезаписать значение
        if( specAttr.isCustomerRead() )
        {
            try
            {
                Map<Integer, EntityAttr> contractParameterMap =
                    Optional.ofNullable( contractService.contractParameterMap( getContractId(), customerBean.getContract().getParamGroupId() ) ).map( ( a ) -> a.getMap() ).orElse( null );

                contractParameter = contractParameterMap.get( id );
            }
            catch( Exception ex )
            {
                logger.error( ex.getMessage(), ex );
            }
        }
		
		if( contractParameter == null )
		{
			try
			{
				contractParameter = EntityAttr.newInstance( specAttr.getType(), getContractId(), specAttr.getId() );
			}
			catch( Exception e )
			{
				logger.error( e.getMessage(), e );

				addErrorMessage( null, "Данный параметр невозможно редактировать." );
				return;
			}
		}

		this.contractParameterId = id;
		this.contractParameter = contractParameter;

		if( this.contractParameter instanceof EntityAttrEmail )
		{
			((EntityAttrEmail)this.contractParameter).addContact( "", "" );
		}
		else if( this.contractParameter instanceof EntityAttrPhone )
		{
			((EntityAttrPhone)this.contractParameter).addContact( "", "" );
		}

		navigationBean.setSubPage( "edit" );
	}

	/**
	 * Сохранение параметра.
	 * @throws BGException
	 */
	public void saveParam()
		throws BGException
	{
		logger.info( "saveParam" );

		ContractParameterGroupAttr specAttr =
			(ContractParameterGroupAttr)getContractParameterGroup().getAttributeList().stream().filter( a -> a.getId() == contractParameterId ).findFirst().orElse( null );
		if( specAttr == null || !specAttr.isCustomerWrite() )
		{
			addErrorMessage( null, "Данный параметр редактировать запрещено." );
			return;
		}

		contractService.contractParameterUpdate( getContractId(), contractParameter );

		navigationBean.setSubPage( "" );

		populate();
	}
}
