/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.contract;

import javax.annotation.PostConstruct;
import javax.enterprise.context.RequestScoped;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.validator.ValidatorException;
import javax.inject.Named;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import javax.validation.executable.ValidateOnExecution;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.contract.api.common.bean.Contract;
import ru.bitel.bgbilling.kernel.contract.api.common.service.ContractService;
import ru.bitel.common.Utils;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;

@Named
@RequestScoped
@BGInjection
public class PasswordBean
	extends AbstractBean
{
	private static final Logger logger = LogManager.getLogger();

	@BGInject(module = false)
	private ContractService contractService;

	@NotNull
	@Size(min = 1)
	private String oldPassword;

	@NotNull
	@Size(min = 1)
	private String newPassword;

	@NotNull
	@Size(min = 1)
	private String newPassword2;

	@PostConstruct
	protected void init()
	{
	}

	public String getOldPassword()
	{
		return oldPassword;
	}

	public void setOldPassword( String oldPassword )
	{
		this.oldPassword = oldPassword;
	}

	public String getNewPassword()
	{
		return newPassword;
	}

	public void setNewPassword( String newPassword )
	{
		logger.info( "setNewPassword " + newPassword );
		this.newPassword = newPassword;
	}

	public String getNewPassword2()
	{
		return newPassword2;
	}

	public void setNewPassword2( String newPassword2 )
	{
		logger.info( "setNewPassword2 " + newPassword2 );
		this.newPassword2 = newPassword2;
	}

	@ValidateOnExecution
	public void changePassword()
		throws BGException
	{
		if( (Utils.isBlankString( oldPassword ) && !customerBean.isPasswordOnce()) || Utils.isBlankString( newPassword ) || Utils.isBlankString( newPassword2 ) )
		{
			return;
		}

		logger.info( "changePassword " + newPassword2 );
		if( !newPassword.equals( newPassword2 ) )
		{
			// throw new ValidatorException( new FacesMessage( FacesMessage.SEVERITY_ERROR, "AAAA", "AAAA" ) );
			addErrorMessage( "passwordForm:newPassword", "Пароли не совпадают!" );
			addErrorMessage( "passwordForm:newPassword2", "Пароли не совпадают!" );

			return;
		}

		final Contract contract = contractService.contractGet( getContractId() );
		if( !customerBean.isPasswordOnce() && !contract.getPassword().equals( oldPassword ) )
		{
			addErrorMessage( "passwordForm:oldPassword", "Неверно введен старый пароль!" );

			return;
		}

		// как в самолете
		if( customerBean.isPasswordOnce() || contract.getPassword().equals( oldPassword ) )
		{
			contractService.contractPasswordUpdate( getContractId(), newPassword, false );

			addInfoMessage( null, "Пароль успешно изменен!" );
			
			customerBean.setPasswordOnce( false );

			customerBean.populate();
		}
	}

	public void validateOldPassword( FacesContext context, UIComponent component, Object value )
		throws ValidatorException
	{
		String oldPassword = (String)value;

		{
			FacesMessage message = new FacesMessage( value + " " + context.getApplication().getResourceBundle( context, "msg" ).getObject( "input.required" ) );
			//throw new ValidatorException( message );

			//throw new ValidatorException( new FacesMessage( FacesMessage.SEVERITY_ERROR, msg, msg ) );
		}
	}
}
