/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.contract;

import java.io.Serializable;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Date;
import java.util.List;

import javax.enterprise.context.SessionScoped;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.contract.api.common.bean.ContractStatus;
import ru.bitel.bgbilling.kernel.contract.api.common.service.ContractStatusService;
import ru.bitel.bgbilling.kernel.contract.api.common.service.ContractStatusService.ContractStatusChangeParameters;
import ru.bitel.bgbilling.kernel.contract.status.common.bean.Status;
import ru.bitel.bgbilling.kernel.directory.api.common.bean.Directory;
import ru.bitel.common.TimeUtils;
import ru.bitel.common.function.Lazy;
import ru.bitel.common.model.MapHolder;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;

@SuppressWarnings("serial")
@Named
@SessionScoped
@BGInjection
public class StatusBean
	extends AbstractBean
	implements Serializable
{
	private static final Logger logger = LogManager.getLogger();

    @BGInject( module = false )
	private ContractStatusService contractStatusService;

    @BGInject( type = Status.class, module = false )
	private Directory<Status> statusDirectory;

	private Lazy<List<ContractStatus>> contractStatusList;

	private Lazy<ContractStatusChangeParameters> contractStatusChangeParameters;

	private Date changeDate;
	
	private String changeStatusMessage = "";
	private boolean changeStatusNeedConfirm = false ;

	@Override
	protected void init()
		throws BGException
	{
        logger.info( "init()" );

        super.init();
		populate();
	}

	public String getStatusTitle( int id )
		throws BGException
	{
		Status status = statusDirectory.get( id );
		if( status != null )
		{
			return status.getTitle();
		}

		return null;
	}

	public Date getChangeDate()
	{
        logger.info( "getChangeDate()" );

        return changeDate;
	}

	public void setChangeDate( Date changeDate )
		throws BGException
	{
	    logger.info( "setChangeDate( " + changeDate + " )" );
	    
		if( changeDate != null )
		{
			final ContractStatusChangeParameters params = getContractStatusChangeParameters();

			if( params.getDate() != null )
			{
				if( TimeUtils.dateBefore( changeDate, params.getDate() ) )
				{
					changeDate = params.getDate();
				}
			}
			else if( params.getDateList() != null && params.getDateList().size() > 0 )
			{
                final Date changeDateTest = changeDate;
                if( params.getDateList().stream()
                    .noneMatch( date -> TimeUtils.dateEqual( date, changeDateTest ) ) )
                {
                    changeDate = (Date)params.getDateList().get( 0 );
                }
			}
			else if( changeDate.toInstant().atZone( ZoneId.systemDefault() ).toLocalDate().compareTo( LocalDate.now() ) > 0 )
			{
				changeDate = null;
			}
		}

		this.changeDate = changeDate;
		//чистим сообщения и прочее
		changeStatusNeedConfirm = false;
        changeStatusMessage = null;
	}

	public List<ContractStatus> getContractStatusList()
		throws BGException
	{
		return contractStatusList.get();
	}

	public ContractStatusChangeParameters getContractStatusChangeParameters()
		throws BGException
	{
		return contractStatusChangeParameters.get();
	}

	public void populate()
		throws BGException
	{
	    logger.info( "populate()" );

	    customerBean.populate();

		contractStatusList = Lazy.of( ( ) -> reverse( contractStatusService.contractStatusList( getContractId(), null ) ) );

		contractStatusChangeParameters = Lazy.of( supEx( () -> contractStatusService.contractStatusChangeParametersGet( getContractId() ) ) );
	}

	public void changeStatus( int status, Date date, String comment )
		throws BGException
	{
		logger.debug( "changeStatus(" + status + ", " + date + ", " + comment + ")" );

		ContractStatus contractStatus = new ContractStatus();
		contractStatus.setContractId( getContractId() );
		contractStatus.setStatus( status );
		contractStatus.setDateFrom( date );
		contractStatus.setDateTo( null );
		contractStatus.setComment( comment != null ? comment : "Изменено абонентом" );
        changeStatusNeedConfirm = false;
        changeStatusMessage="";
        MapHolder<String, String> answer = contractStatusService.contractStatusChange( getContractId(), contractStatus, true, changeStatusNeedConfirm );
        changeStatusMessage = answer.getMap().get( "message" );
        String result = answer.getMap().get( "result" );
        if ( result != null )
        {
            if ( result.equals( "confirm" ) )// нужно подтверждение
            {
                changeStatusNeedConfirm = true;
                populate();
                return; // выводим запрос
            }

        }
        changeDate = null;
        populate();
	}
	
    public void confirmChangeStatus( int status, Date date, String comment )
        throws BGException
    {
        logger.info( "confirmChangeStatus( ... )" );

        ContractStatus contractStatus = new ContractStatus();
        contractStatus.setContractId( getContractId() );
        contractStatus.setStatus( status );
        contractStatus.setDateFrom( date );
        contractStatus.setDateTo( null );
        contractStatus.setComment( comment != null ? comment : "Изменено абонентом" );
        changeStatusNeedConfirm = false;
        changeStatusMessage="";
        MapHolder<String, String> answer = contractStatusService.contractStatusChange( getContractId(), contractStatus, true,true );
        changeStatusMessage = answer.getMap().get( "message" );
        changeDate = null;
        populate();
    }
    
    public void noConfirmChangeStatus( int status, Date date, String comment )
        throws BGException
    {
        logger.info( "noConfirmChangeStatus( ... )" );
        
        changeStatusNeedConfirm = false;
        changeStatusMessage = null;
        changeDate = null;
        populate();
    }

	public String getChangeStatusMessage()
    {
        return changeStatusMessage;
    }

    public void setChangeStatusMessage( String changeStatusMessage )
    {
        this.changeStatusMessage = changeStatusMessage;
    }

    public boolean isChangeStatusNeedConfirm()
    {
        return changeStatusNeedConfirm;
    }

    public void setChangeStatusNeedConfirm( boolean changeStatusNeedConfirm )
    {
        this.changeStatusNeedConfirm = changeStatusNeedConfirm;
    }
}