/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.contract;

import java.util.Date;
import java.util.List;

import javax.enterprise.context.SessionScoped;
import javax.inject.Inject;
import javax.inject.Named;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.contract.api.common.bean.ContractTariff;
import ru.bitel.bgbilling.kernel.contract.api.common.bean.ContractTariffGroup;
import ru.bitel.bgbilling.kernel.contract.api.common.service.ContractTariffService;
import ru.bitel.bgbilling.kernel.tariff.common.bean.TariffGroup;
import ru.bitel.bgbilling.kernel.tariff.common.bean.TariffGroupTariff;
import ru.bitel.bgbilling.kernel.tariff.common.bean.TariffPlan;
import ru.bitel.bgbilling.kernel.tariff.common.service.TariffService;
import ru.bitel.common.TimeUtils;
import ru.bitel.common.Utils;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;

/**
 * Смена тарифа.<br>
 * @author amir
 */
@Named
@SessionScoped
@BGInjection
public class TariffChangeBean
	extends AbstractBean
{
    private static final long serialVersionUID = -4148494394138294418L;

	@Inject
	private TariffBean tariffBean;

	@BGInject(module = false)
	private ContractTariffService contractTariffService;

    @BGInject(module = false)
    private TariffService tariffService;
	
	/**
	 * Тариф, с которого переходим.
	 */
	private ContractTariff fromContractTariff;

	/**
	 * ID тарифного плана (tariffPlanId), на который переходим.
	 */
	private int toTariffPlanId;

	/**
	 * Список дат, с которых можно перейти на выбранный {@link #toTariffPlanId} тариф.
	 */
    private List<String> availableDateList; // не помню, почему String; скорее всего можно исправить на Date с изменением xhtml.

    /**
     * Выбранная дата, с которой будет осуществлен переход на новый тариф.
     */
	private Date fromDate;

	/**
	 * Получение тарифа, с которого переходим.
	 * @return
	 */
	public ContractTariff getFromContractTariff()
	{
		return fromContractTariff;
	}

	/**
	 * Получение ID тарифа (tariffPlanId), на который переходим.
	 * @return
	 */
	public int getToTariffPlanId()
	{
		return toTariffPlanId;
	}

	/**
	 * Установка тарифа, на который переходим.
	 * @param toTariffPlanId ID тарифного плана, на который будет переход.
	 * @throws BGException
	 */
	public void setToTariffPlanId( int toTariffPlanId )
		throws BGException
	{
		this.toTariffPlanId = toTariffPlanId;
		
		TariffGroup tariffGroup = tariffService.tariffGroupGet(  getTariffGroupId( fromContractTariff ) );
		if( tariffGroup == null ) // не даем выбрать тариф, если у него нет тарифной группы
		{
			this.toTariffPlanId = 0;
			return;
		}
		
		boolean ok = false;
		Date now = new Date();
		for ( TariffGroupTariff tariffGroupTariff : tariffGroup.getPlanList() )
		{
		    if ( tariffGroupTariff.getId() == toTariffPlanId
		        && ( tariffGroupTariff.getDate1() == null || !tariffGroupTariff.getDate1().after( now ) )
		        && ( tariffGroupTariff.getDate2() == null || !tariffGroupTariff.getDate2().before( now ) ) )
		    {
		        ok = true;
		    }
		}
		
		if ( !ok )
		{
            this.toTariffPlanId = 0;
            return;
		}
		
		if( toTariffPlanId > 0 )
		{
		    // получаем возможные даты, с которых можно осуществить переход (в биллинге может выполняться скрипт обработки списка этих дат)
            List<Date> availableDateList = Utils.maskNull( contractTariffService.contractTariffChangeAvailableDateList( getContractId(), tariffGroup.getId(), fromContractTariff.getTariffPlanId(), toTariffPlanId ) );
			this.availableDateList = dateListToStringList( availableDateList );
		}
	}

	/**
	 * Получение списка дат, с которых можно перейти на выбранный тариф.
	 * @return
	 */
	public List<String> getAvailableDateList()
	{
		return availableDateList;
	}

	/**
	 * Получение установленной даты, с которой будет переход на новый тариф.
	 * @return
	 */
	public Date getFromDate()
	{
		return fromDate;
	}

	/**
	 * Установка даты, с которой будет переход на новый тариф.
	 * @param date
	 */
	public void setFromDate( Date date )
	{
		this.fromDate = date;
	}

	/**
	 * Начало смены тарифа.
	 * @param contractTariffId тарифный план, с которого переходим.
	 * @throws BGException
	 */
	public void change( int contractTariffId )
		throws BGException
	{
		fromContractTariff = contractTariffService.contractTariffGet( contractTariffId ); // запоминаем тариф, с которого будет переход

		if( fromContractTariff == null || fromContractTariff.getContractId() != getContractId() )
		{
			addErrorMessage( null, "Данный тарифный план не найден на договоре." );
			return;
		}

		toTariffPlanId = 0; 
		fromDate = null;
		
        step = 1;
	}

	/**
	 * Сохраняем переход на новый тарифный план (после того как выбрали тариф и дату).
	 * @throws BGException
	 */
	public void apply()
		throws BGException
	{
        if( fromContractTariff == null || toTariffPlanId <= 0 || fromDate == null )
        {
            step = 0;
            return;
        }

		tariffBean.populate();

        // проверяем, что на данный тариф можно переходить
        List<TariffPlan> tariffPlanList = tariffBean.getAvailableTariffPlanList( fromContractTariff );
        if( Utils.maskNull( tariffPlanList ).stream().noneMatch( tariffPlan -> tariffPlan.getId() == toTariffPlanId ) )
        {
            addErrorMessage( null, "Извините, но Вы не можете перейти с текущего тарифа на выбранный." );
            return;
        }

		TariffGroup tariffGroup = tariffService.tariffGroupGet( getTariffGroupId( fromContractTariff ) );
		if( tariffGroup == null )
		{
			addErrorMessage( null, "Извините, но вы не можете сменить тариф. Тарифная группа не найдена." );
			return;
		}

        // проверяем, что дата есть в списке дат, с которых можно перейти
        List<Date> dateList = contractTariffService.contractTariffChangeAvailableDateList( getContractId(), tariffGroup.getId(), fromContractTariff.getTariffPlanId(), toTariffPlanId );
        if( dateList.stream().noneMatch( date -> TimeUtils.dateEqual( date, fromDate ) ) )
        {
            addErrorMessage( null, "Извините, но Вы не можете перейти на новый тариф с выбранной даты." );
            return;
        }

		contractTariffService.contractTariffChangeByCustomer( getContractId(), tariffGroup.getId(), fromContractTariff.getId(), toTariffPlanId, fromDate );

		tariffBean.populate(); // обновляем данные (в т.ч. список текущих тарифов) TariffBean
		
		this.fromDate = null;
		this.toTariffPlanId = 0;
		
        step = 0;
	}
	
	private int getTariffGroupId( ContractTariff contractTariff )
	    throws BGException
	{
        int tariffGroupId = contractTariff.getTariffGroupId();
        if ( tariffGroupId < 1 )
        {
            ContractTariffGroup contractTariffGroup = contractTariffService.contractTariffGroupGet( contractTariff.getContractId() );
            if ( contractTariffGroup != null )
            {
                tariffGroupId = contractTariffGroup.getTariffGroupId();
            }
        }
        return tariffGroupId;
	}
}
