/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.directory;

import java.io.Serializable;
import java.lang.reflect.Field;

import javax.enterprise.context.ApplicationScoped;
import javax.interceptor.InvocationContext;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.directory.api.common.bean.Directory;
import ru.bitel.common.model.Id;
import ru.bitel.mybgbilling.kernel.common.MySessionBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.InjectManager;
import ru.bitel.mybgbilling.kernel.common.inject.InjectPlanField;
import ru.bitel.mybgbilling.kernel.navigation.NavigationBean;

@ApplicationScoped
public class DirectoryInjectManager
	extends InjectManager
	implements Serializable
{
	private static final Logger logger = LogManager.getLogger();

	@Override
	protected InjectPlanField buildInjection( final Object o, final Field f )
	{
		try
		{
			final boolean accessible = f.isAccessible();

			if( !accessible )
			{
				f.setAccessible( true );
			}

			if( f.get( o ) != null )
			{
				if( !accessible )
				{
					f.setAccessible( accessible );
				}

				return null;
			}

			final BGInject bgInject = f.getAnnotation( BGInject.class );

			if( bgInject == null )
			{
				if( !accessible )
				{
					f.setAccessible( accessible );
				}

				return null;
			}

			if( !Directory.class.isAssignableFrom( f.getType() ) )
			{
				if( !accessible )
				{
					f.setAccessible( accessible );
				}

				return null;
			}

			final Class<? extends Id> type = bgInject.type().asSubclass( Id.class );

			logger.info( f.getName() );

			return new InjectPlanField( f )
			{
				@Override
				public void process( final MySessionBean wsClientBean, final NavigationBean navigationBean, final InvocationContext ctx, final Object o )
					throws IllegalArgumentException, IllegalAccessException
				{
					// final Integer moduleId = (Integer)ctx.getContextData().get( "moduleId" );

					final int moduleId;

					if( bgInject.module() )
					{
						final Integer moduleIdO = navigationBean.getModuleId();
						moduleId = moduleIdO != null ? moduleIdO : 0;
					}
					else
					{
						moduleId = 0;
					}

					try
					{
						Object directory = wsClientBean.getDirectory( type, moduleId, true );
						field.set( o, directory );
					}
					catch( BGException e )
					{
						logger.error( e.getMessage(), e );
					}
				}

				@Override
				public void cleanup( MySessionBean wsClientBean, NavigationBean navigationBean, InvocationContext ctx, Object o )
					throws IllegalArgumentException, IllegalAccessException
				{
					// TODO Auto-generated method stub
					
				}
			};
		}
		catch( Exception e )
		{
			logger.error( e.getMessage(), e );
		}

		return null;
	}
}
