/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.menu;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.annotation.PostConstruct;
import javax.el.ELContext;
import javax.el.ExpressionFactory;
import javax.el.ValueExpression;
import javax.enterprise.context.SessionScoped;
import javax.enterprise.inject.Instance;
import javax.faces.component.UIComponent;
import javax.faces.component.visit.VisitCallback;
import javax.faces.component.visit.VisitContext;
import javax.faces.component.visit.VisitResult;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseId;
import javax.inject.Inject;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.kernel.module.common.bean.BGModule;
import ru.bitel.common.Utils;
import ru.bitel.mybgbilling.kernel.common.Configuration;
import ru.bitel.mybgbilling.kernel.contract.AdditionalActionBean;
import ru.bitel.mybgbilling.kernel.navigation.LocaleBean;
import ru.bitel.mybgbilling.kernel.ws.CustomerBean;

/**
 * Генерирует меню для сессии.
 * 
 * @author amir
 * @see #reloadMenu()
 * @see #process(List, Set, List)
 */
@Named
@SessionScoped
public class MenuBean
	implements Serializable
{
	private static final long serialVersionUID = 6542737132993388781L;

	private static final Logger logger = LogManager.getLogger();

	private MenuItem menu;

	@Inject
	private Configuration configuration;

	@Inject
	private CustomerBean customerBean;

	@Inject
	private LocaleBean localeBean;
	
	@Inject
	private Instance<AdditionalActionBean> additionalActionBean;

	private UIComponent menuComponent;

	public UIComponent getMenuComponent()
	{
		return menuComponent;
	}

	public void setMenuComponent( UIComponent menuComponent )
	{
		this.menuComponent = menuComponent;
	}

	@PostConstruct
	void init()
	{
		/*try
		{
			ExternalContext externalContext = FacesContext.getCurrentInstance().getExternalContext();
		
			menu = JAXB.unmarshal( externalContext.getResourceAsStream( "/WEB-INF/menu.xml" ), MenuItem.class );
		}
		catch( Exception e )
		{
			logger.error( e.getMessage(), e );
		}*/

		reloadMenu();
	}
	
	public void reloadMenu()
	{
		try
		{
		    // получаем меню из mybgbilling-menu.groovy
			menu = configuration.getMenuFactory().apply( customerBean.getContract() );
			
			Set<Integer> activationModuleIds = new HashSet<>(); 

			// дополнительно обрабатываем дерево и получаем новое дерево
            menu.setChildren( process( menu.getChildren(), activationModuleIds, customerBean.getContractModuleIds(), configuration.getModuleList() ) );
		}
		catch( Exception e )
		{
			logger.error( e.getMessage(), e );
		}
	}

    private List<MenuItem> process( List<MenuItem> menuItemList, Set<Integer> activationModuleIds, Set<Integer> contractModuleIds, List<BGModule> moduleList )
	{
		List<MenuItem> result = new ArrayList<>();

		for( MenuItem menuItem : menuItemList )
		{
			if( !menuItem.isShow() )
			{
				continue;
			}
			
			if( Utils.notBlankString( menuItem.getBadge() ) )
			{
				menuItem.badgeExpression = parseExpression( menuItem.getBadge(), Integer.class );
			}
			else
			{
				menuItem.setBadge( null );
			}

			if( Utils.notBlankString( menuItem.getBadgeUpdate() ) )
			{
				menuItem.badgeUpdateExpression = parseExpression(  menuItem.getBadgeUpdate(), Object.class );
			}
			else
			{
				menuItem.setBadgeUpdate( null );
			}

			// пропускаем пункт меню, если указан код модуля и если такого модуля у абонента нет
			if( menuItem.getModuleId() > 0 && !contractModuleIds.contains( menuItem.getModuleId() ) )
			{
				continue;
			}

			if( menuItem.getModule() != null && !"kernel".equals( menuItem.getModule() ) && menuItem.getModuleId() <= 0 )
			{
			    // экземпляров модуля может быть несколько, поэтому пункт возможно придется продублировать
				for( BGModule module : moduleList )
				{
                    if( !menuItem.getModule().equals( module.getName() ) )
                    {
                        continue;
                    }

                    if( !contractModuleIds.contains( module.getId() ) )
                    {
                        // возможно надо добавить пункт активации модуля
                        checkModuleActivation( activationModuleIds, menuItem, module, result );
                        continue;
                    }

					MenuItem menuItem2 = menuItem.clone( module.getId() );
					if( Utils.isBlankString( menuItem2.getTitle() ) )
					{
		                menuItem2.setTitle( localeBean.msg( "menu." + module.getName() + "." + module.getId(), localeBean.msg( "menu." + module.getName(), module.getTitle() ) ) );
					}

					List<MenuItem> children = menuItem2.getChildren();
					if( children != null && children.size() > 0 )
					{
						menuItem2.setChildren( process( children, activationModuleIds, contractModuleIds, moduleList ) );
					}

					result.add( menuItem2 );
				}
			}
			else
			{
				List<MenuItem> children = menuItem.getChildren();
				if( children != null && children.size() > 0 )
				{
					menuItem.setChildren( process( children, activationModuleIds, contractModuleIds, moduleList ) );
				}

				result.add( menuItem );
			}
		}

		return result;
	}

    private void checkModuleActivation( Set<Integer> activationModuleIds, MenuItem menuItem, BGModule module, List<MenuItem> result )
    {
        try
        {
            // если впервые встретили модуль, и еще не пробовали добавить пункт активации
            if( !activationModuleIds.add( module.getId() ) )
            {
                return;
            }
            
            List<Integer> actions = configuration.getActivateModuleActions().get( module.getId() );
            if( actions == null || actions.isEmpty() )
            {
                return;
            }

            // смотрим, доступны ли такие действия
            if( additionalActionBean.get().getAllAdditionalContractActionList()
                .stream()
                .noneMatch( a -> actions.contains( a.getId() ) ) )
            {
                return;
            }

            MenuItem menuItem2 = menuItem.clone( module.getId() );
            if( Utils.isBlankString( menuItem2.getTitle() ) )
            {
                menuItem2.setTitle( localeBean.msg( "menu." + module.getName() + "." + module.getId(), localeBean.msg( "menu." + module.getName(), module.getTitle() ) ) );
            }

            menuItem2.setModuleId( module.getId() );
            menuItem2.setPage( "kernel/activateModule" );
            menuItem2.setChildren( Collections.emptyList() );

            result.add( menuItem2 );
        }
        catch( Exception ex )
        {
            logger.error( ex.getMessage(), ex );
        }
    }
	
    private ValueExpression parseExpression( final String expression, final Class<?> resultClass )
    {
        try
        {
            FacesContext context = FacesContext.getCurrentInstance();
            ExpressionFactory expressionFactory = context.getApplication().getExpressionFactory();
            ELContext elContext = context.getELContext();
            return expressionFactory.createValueExpression( elContext, expression, resultClass );
        }
        catch( Exception ex )
        {
            logger.error( ex.getMessage(), ex );
        }
        
        return null;
    }

	public MenuItem getMenu()
	{
		return menu;
	}

    public int getMenuBadgeValue( final MenuItem item )
    {
        Integer result = null;

        try
        {
            if( item == null || Utils.isBlankString( item.getBadge() ) )
            {
                return 0;
            }

            // для оптимизации при открытии страницы возвращаем сразу 0
            // обновление произойдет через ajax
            if( !FacesContext.getCurrentInstance().getPartialViewContext().isPartialRequest() )
            {
                return 0;
            }

            // if( FacesContext.getCurrentInstance().getCurrentPhaseId() == PhaseId.APPLY_REQUEST_VALUES )
            // {
            // return 0;
            // }

            final FacesContext context = FacesContext.getCurrentInstance();
            final ELContext elContext = context.getELContext();

            result = (Integer)item.badgeExpression.getValue( elContext );
        }
        catch( Exception ex )
        {
            logger.error( ex.getMessage(), ex );
        }

        return result != null ? result.intValue() : 0;
    }

	public void menuBadgeValueUpdate( MenuItem item )
	{
		if( item == null || Utils.isBlankString( item.getBadgeUpdate() ) || Utils.isBlankString( item.getBadge() ) )
		{
			return;
		}

		if( FacesContext.getCurrentInstance().getCurrentPhaseId() == PhaseId.APPLY_REQUEST_VALUES )
		{
			return;
		}

		FacesContext context = FacesContext.getCurrentInstance();
		ELContext elContext = context.getELContext();

		item.badgeUpdateExpression.getValue( elContext );
	}

	private volatile Set<String> badgesIds;

	public Set<String> getBadgesIds()
	{
		if( badgesIds != null )
		{
			return badgesIds;
		}

		Set<String> result = new HashSet<>();
		boolean[] found = new boolean[1];

		FacesContext context = FacesContext.getCurrentInstance();
		UIComponent root = context.getViewRoot();
		VisitContext visitContext = VisitContext.createVisitContext( context );

		root.visitTree( visitContext, new VisitCallback()
		{
			@Override
			public VisitResult visit( VisitContext context, UIComponent component )
			{
				if( component.getId().equals( "badge" ) )
				{
					result.add( component.getClientId() );
				}
				else if( !found[0] && component.getId().equals( "sidebarMenuForm" ) )
				{
					found[0] = true;
				}

				return VisitResult.ACCEPT;
			}
		} );

		if( found[0] )
		{
			this.badgesIds = result;
		}

		return result;
	}

	public boolean checkContainsPage( String page )
	{
		return checkContainsPage( menu, page );
	}
	
    public MenuItem getMenuItemForPage( String page )
    {
        return getMenuItemForPage( menu, page );
    }

    public boolean checkContainsPage( MenuItem menu, String page )
    {
        return getMenuItemForPage( menu, page ) != null;
    }

    public MenuItem getMenuItemForPage( MenuItem menu, String page )
    {
        if( menu.getChildren() == null )
        {
            return null;
        }

        for( MenuItem i : menu.getChildren() )
        {
            if( !i.isShow() )
            {
                continue;
            }

            if( page.equals( i.getPage() ) )
            {
                return i;
            }

            MenuItem i2 = getMenuItemForPage( i, page );
            if( i2 != null )
            {
                return i2;
            }
        }

        return null;
    }

    public MenuItem getMenuItemForModule( int moduleId )
    {
        return getMenuItemForModule( menu, moduleId );
    }

    public MenuItem getMenuItemForModule( MenuItem menu, int moduleId )
    {
        if( menu.getChildren() == null )
        {
            return null;
        }

        for( MenuItem i : menu.getChildren() )
        {
            if( !i.isShow() )
            {
                continue;
            }

            if( i.getModuleId() == moduleId && Utils.notEmptyString( i.getPage() ) )
            {
                return i;
            }

            MenuItem i2 = getMenuItemForModule( i, moduleId );
            if( i2 != null )
            {
                return i2;
            }
        }

        return null;
    }
}
