/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
 */
package ru.bitel.mybgbilling.kernel.navigation;

import java.io.Serializable;

import javax.annotation.PostConstruct;
import javax.enterprise.context.Conversation;
import javax.enterprise.context.SessionScoped;
import javax.enterprise.inject.Instance;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.common.Utils;
import ru.bitel.mybgbilling.kernel.common.Configuration;

@Named
@SessionScoped
public class NavigationBean
    implements Serializable
{
    @Inject
    private Instance<Conversation> conversation;

    @Inject
    private Configuration configuration;

    private static final Logger logger = LogManager.getLogger();

    private int moduleId = 0;

    private String page = "WEB-INF/content/kernel/news";

    private String subPage = "";

    @PostConstruct
    private void init()
    {
        setPage( null );
    }

    public int getModuleId()
    {
        return moduleId;
    }

    public void setModuleId( int moduleId )
    {
        this.moduleId = moduleId;
        logger.info( "moduleId=" + moduleId );
    }

    public String getPage()
    {
        return "/" + page;
    }

    public void setPage( String page )
    {
        String oldPage = this.page;

        if( page == null )
        {
            page = configuration.getStartPage();

            if( Utils.isBlankString( page ) )
            {
                page = "kernel/news";
            }
        }

        logger.info( "page=" + page + " => WEB-INF/content/" + page );
        setModuleId( 0 );

        this.page = "WEB-INF/content/" + page;
        setSubPage( "" );
        // return "#" + page;

        /*if( conversation.get()!=null && !conversation.get().isTransient() )
        {
        	conversation.get().end();
        }*/

        if( oldPage != null && !oldPage.equals( this.page ) )
        {
            FacesContext.getCurrentInstance().getViewRoot().getViewMap().clear();
        }
    }

    public void setPage( String page, int moduleId, String subPage )
    {
        String oldPage = this.page;

        logger.info( "page=" + page + " => WEB-INF/content/" + page );
        setModuleId( moduleId );

        this.page = "WEB-INF/content/" + page;

        setSubPage( subPage );

        if( oldPage != null && !oldPage.equals( this.page ) )
        {
            FacesContext.getCurrentInstance().getViewRoot().getViewMap().clear();
        }
    }

    public void setPage( String page, int moduleId )
    {
        String oldPage = this.page;

        logger.info( "page=" + page + " => WEB-INF/content/" + page );
        setModuleId( moduleId );

        this.page = "WEB-INF/content/" + page;
        setSubPage( "" );

        /*if( conversation.get()!=null && !conversation.get().isTransient() )
        {
        	conversation.get().end();
        }*/

        // return "#" + page;

        if( oldPage != null && !oldPage.equals( this.page ) )
        {
            FacesContext.getCurrentInstance().getViewRoot().getViewMap().clear();
        }
    }

    public String getSubPage()
    {
        return subPage;
    }

    public void setSubPage( String subPage )
    {
        if( subPage == null )
        {
            subPage = "";
        }
        else
        {
            this.subPage = subPage;
        }
    }

    private Object subPageSwitch0( Object... args )
    {
        for( int i = 0; (i + 1) < args.length; i = i + 2 )
        {
            if( args[i].equals( subPage ) )
            {
                return args[i + 1];
            }
        }

        if( args.length % 2 == 1 )
        {
            return args[args.length - 1];
        }

        return null;
    }

    public Object subPageSwitch( Object... args )
    {
        return subPageSwitch0( args );
    }

    public Object subPageSwitch( String s1, Object o1, String s2, Object o2, Object def )
    {
        return subPageSwitch0( s1, o1, s2, o2, def );
    }

    public Object subPageSwitch( String s1, Object o1, Object def )
    {
        return subPageSwitch0( s1, o1, def );
    }

    public boolean isSubPage( String a1 )
    {
        return subPage.equals( a1 );
    }

    public boolean isSubPage( String a1, String a2 )
    {
        return subPage.equals( a1 )
            || subPage.equals( a2 );
    }

    public boolean isSubPage( String a1, String a2, String a3 )
    {
        return subPage.equals( a1 )
            || subPage.equals( a2 )
            || subPage.equals( a3 );
    }

    public boolean isSubPage( String a1, String a2, String a3, String a4 )
    {
        return subPage.equals( a1 )
            || subPage.equals( a2 )
            || subPage.equals( a3 )
            || subPage.equals( a4 );
    }

    public String passwordReset()
    {
        return "WEB-INF/content/kernel/passwordReset.xhtml";
    }
}
