/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.news;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.function.Supplier;

import javax.enterprise.context.SessionScoped;
import javax.inject.Named;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.contract.label.common.service.ContractLabelService;
import ru.bitel.bgbilling.plugins.news.common.bean.News;
import ru.bitel.bgbilling.plugins.news.common.service.NewsService;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.common.model.Page;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;

@Named
@SessionScoped
@BGInjection
public class NewsBean
	extends AbstractBean
	implements Serializable
{
	private static volatile long NEWS_CACHE_TIME = 0L;
	private static volatile List<News> NEWS_CACHE = new ArrayList<>();
	private static final Object NEWS_CACHE_MUTEX = new Object();

	@BGInject( module = false )
	private NewsService newsService;
	
    @BGInject( module = false )
    private ContractLabelService contractLabelService; 

	private Supplier<List<News>> newsList;

	@Override
	protected void init()
		throws BGException
	{
		populate();
	}

	public void populate()
		throws BGException
	{
		final List<Integer> labelIds = contractLabelService.getContractLabelIds( customerBean.getContractId() );
		
        newsList = Async.of( () ->
        {
            Set<News> newsSet = new HashSet<>();
            for ( News news : getAllNews() )
            {
                if ( news.getContractLabelIds() == null || news.getContractLabelIds().isEmpty() )
                {
                    newsSet.add( news );
                }
                else
                {
                    for ( int id : news.getContractLabelIds() )
                    {
                        if ( labelIds.contains( id ) )
                        {
                            newsSet.add( news );
                            break;
                        }
                    }
                }
            }
            return new ArrayList<News>( newsSet );
        } );
	}

	private List<News> getAllNews()
		throws BGException
	{
		synchronized( NEWS_CACHE_MUTEX )
		{
			long millis = System.currentTimeMillis();
			if( millis < NEWS_CACHE_TIME + TimeUnit.MINUTES.toMillis( 13 ) )
			{
				return NEWS_CACHE;
			}

			if ( newsService != null )
			{
    			List<News> newsList = newsService.newsList( new Page( 1, 100 ) );
    			newsList = Utils.maskNull( newsList );
    
    			for( News news : newsList )
    			{
    				String body = news.getBody().trim();
    				if( body.startsWith( "<data>" ) )
    				{
    					body = body.substring( 6, body.length() - 7 );
    				}
    				else if( !body.contains( "<br>" ) && !body.contains( "<br/>" ) )
    				{
    					body = body.replaceAll( "\n", "<br/>" );
    				}
    
    				news.setBody( body );
    			}
    
    			NEWS_CACHE = newsList;
    			NEWS_CACHE_TIME = millis;
			}

			return NEWS_CACHE;
		}
	}

	public List<News> getNewsList()
		throws BGException
	{
		return newsList.get();
	}
}
