/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.payment;

import java.io.IOException;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.function.Supplier;

import javax.enterprise.context.SessionScoped;
import javax.enterprise.inject.Instance;
import javax.inject.Inject;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.common.BGIllegalAccessException;
import ru.bitel.bgbilling.common.BGIllegalArgumentException;
import ru.bitel.bgbilling.common.BGMessageException;
import ru.bitel.bgbilling.kernel.contract.autopayment.common.bean.Autopayment;
import ru.bitel.bgbilling.kernel.contract.autopayment.common.bean.AutopaymentMode;
import ru.bitel.bgbilling.kernel.contract.autopayment.common.bean.ContractAutopaymentMode;
import ru.bitel.bgbilling.kernel.contract.autopayment.common.bean.ContractAutopaymentMode.ContractAutopaymentModeAuto;
import ru.bitel.bgbilling.kernel.contract.autopayment.common.bean.ContractAutopaymentMode.ContractAutopaymentModePeriodic;
import ru.bitel.bgbilling.kernel.contract.autopayment.common.service.ContractAutopaymentService;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Lazy;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.modules.payment.PaymentServiceProviderBean;

@Named
@SessionScoped
@BGInjection
public class AutopaymentBean
	extends AbstractBean
	implements Serializable
{
	private static final Logger logger = LogManager.getLogger();

	@Inject
	private Instance<PaymentBean> paymentBean;

	/**
	 * Список возможных режимов автоплатежа.o
	 */
	private Supplier<List<ContractAutopaymentMode>> contractAutopaymentModeList = () -> null;

	/**
	 * Текущий автоплатеж, если установлен.
	 */
	private Supplier<Autopayment> currentAutopayment = () -> null;

	private Supplier<ContractAutopaymentMode> currentContractAutopaymentMode = () -> null;

	private AutopaymentMode autopaymentMode = AutopaymentMode.AUTO;

	private ContractAutopaymentModeAuto autoMode = new ContractAutopaymentModeAuto();
	private ContractAutopaymentModePeriodic weekMode = new ContractAutopaymentModePeriodic();
	private ContractAutopaymentModePeriodic monthMode = new ContractAutopaymentModePeriodic();

	public AutopaymentMode getAutopaymentMode()
	{
		return autopaymentMode;
	}

	public void setAutopaymentMode( AutopaymentMode autopaymentMode )
	{
		this.autopaymentMode = autopaymentMode;
	}

	public void setAutopaymentMode( int code )
	{
		this.autopaymentMode = AutopaymentMode.getAutopaymentModeByCode( code );
	}

	public ContractAutopaymentModeAuto getAutoMode()
	{
		return autoMode;
	}

	public ContractAutopaymentModePeriodic getWeekMode()
	{
		return weekMode;
	}

	public ContractAutopaymentModePeriodic getMonthMode()
	{
		return monthMode;
	}

	public List<ContractAutopaymentMode> getContractAutopaymentModeList()
	{
		return contractAutopaymentModeList.get();
	}

	public Autopayment getCurrentAutopayment()
	{
		return currentAutopayment.get();
	}

	public ContractAutopaymentMode getCurrentContractAutopaymentMode()
	{
		return currentContractAutopaymentMode.get();
	}

	public void autopayment()
	{
		logger.info( "autopayment" );

		navigationBean.setPage( "kernel/paymentAuto", 0 );
		
		boolean haveAuto = false;
		boolean haveMonth = false;
		boolean haveWeek = false;

		for( ContractAutopaymentMode mode : getContractAutopaymentModeList() )
		{
			switch( mode.getMode() )
			{
				case AUTO:
				{
				    haveAuto = true;
					ContractAutopaymentModeAuto m = (ContractAutopaymentModeAuto)mode;
					autoMode.setMode( m.getMode() );
					autoMode.setDaySum( m.getDaySum() );
					autoMode.setDaySumMax( m.getDaySumMax() );
					autoMode.setWeekSum( m.getWeekSum() );
					autoMode.setWeekSumMax( m.getWeekSumMax() );
					autoMode.setMonthSum( m.getMonthSum() );
					autoMode.setMonthSumMax( m.getMonthSumMax() );

					break;
				}

				case MONTH:
				{
				    haveMonth = true;
					ContractAutopaymentModePeriodic m = (ContractAutopaymentModePeriodic)mode;
					monthMode.setMode( m.getMode() );
					monthMode.setDay( m.getDay() );
					monthMode.setSum( m.getSum() );
					monthMode.setSumMin( m.getSumMin() );
					monthMode.setSumMax( m.getSumMax() );

					break;
				}

				case WEEK:
				{
				    haveWeek = true;
					ContractAutopaymentModePeriodic m = (ContractAutopaymentModePeriodic)mode;
					weekMode.setMode( m.getMode() );
					weekMode.setDay( m.getDay() );
					weekMode.setSum( m.getSum() );
					weekMode.setSumMin( m.getSumMin() );
					weekMode.setSumMax( m.getSumMax() );

					break;
				}

				default:
					break;
            }
        }

        if( !haveAuto )
        {
            this.autopaymentMode = AutopaymentMode.MONTH;
            if( !haveMonth )
            {
                this.autopaymentMode = AutopaymentMode.WEEK;
                if( !haveWeek )
                {
                    this.autopaymentMode = AutopaymentMode.AUTO;
                }
            }
        }

        if( getCurrentContractAutopaymentMode() != null
            && getContractAutopaymentModeList().stream().anyMatch( a -> a.getMode() == getCurrentContractAutopaymentMode().getMode() ) )
        {
            this.autopaymentMode = getCurrentContractAutopaymentMode().getMode();

            switch( this.autopaymentMode )
            {
                case AUTO:
                {
                    ContractAutopaymentModeAuto m = (ContractAutopaymentModeAuto)getCurrentContractAutopaymentMode();
                    autoMode.setMode( m.getMode() );
                    autoMode.setDaySum( m.getDaySum() );
                    autoMode.setWeekSum( m.getWeekSum() );
                    autoMode.setMonthSum( m.getMonthSum() );

                    break;
                }

                case MONTH:
                {
                    ContractAutopaymentModePeriodic m = (ContractAutopaymentModePeriodic)getCurrentContractAutopaymentMode();
                    monthMode.setMode( m.getMode() );
                    monthMode.setDay( m.getDay() );
                    monthMode.setSum( m.getSum() );

                    break;
                }

                case WEEK:
                {
                    ContractAutopaymentModePeriodic m = (ContractAutopaymentModePeriodic)getCurrentContractAutopaymentMode();
                    weekMode.setMode( m.getMode() );
                    weekMode.setDay( m.getDay() );
                    weekMode.setSum( m.getSum() );

                    break;
                }

                default:
                    break;
            }
        }
    }

	public void populate()
	{
		final PaymentServiceProviderBean paymentProvider = paymentBean.get().getPaymentProviderBean();

		if( paymentProvider != null )
		{
			final ContractAutopaymentService contractAutopaymentService = paymentProvider.getContractAutopaymentService();

			this.contractAutopaymentModeList = Async.of( () -> contractAutopaymentService.contractAutopaymentModeList( getContractId() ) );

			this.currentAutopayment = Async.of( () -> paymentProvider.autopaymentGet( getContractId() ) );
			this.currentContractAutopaymentMode = Lazy.of( () -> getCurrentAutopayment() != null ? ContractAutopaymentMode.fromAutopayment( getCurrentAutopayment() ) : null );
		}
	}

	public void apply()
		throws BGException, IOException
	{
		switch( autopaymentMode )
		{
			case AUTO:
			{
				if( autoMode.getDaySum() == null || BigDecimal.ZERO.compareTo( autoMode.getDaySum() ) >= 0 )
				{
					addErrorMessage( "autopaymentForm:autoModeDaySum", "Укажите корректную сумму!" );
					return;
				}

				autoMode.setDaySum( autoMode.getDaySum().setScale( 0, RoundingMode.FLOOR ) );

				if( autoMode.getDaySumMax().compareTo( autoMode.getDaySum() ) < 0 )
				{
					addErrorMessage( "autopaymentForm:autoModeDaySum", "Сумма дневного лимита должна быть не более " + autoMode.getDaySumMax() + "." );
					return;
				}

				if( autoMode.getWeekSum() == null || BigDecimal.ZERO.compareTo( autoMode.getWeekSum() ) >= 0 )
				{
					addErrorMessage( "autopaymentForm:autoModeWeekSum", "Укажите корректную сумму!" );
					return;
				}

				autoMode.setWeekSum( autoMode.getWeekSum().setScale( 0, RoundingMode.FLOOR ) );

				if( autoMode.getWeekSumMax().compareTo( autoMode.getWeekSum() ) < 0 )
				{
					addErrorMessage( "autopaymentForm:autoModeWeekSum", "Сумма недельного лимита должна быть не более " + autoMode.getWeekSumMax() + "." );
					return;
				}

				if( autoMode.getMonthSum() == null || BigDecimal.ZERO.compareTo( autoMode.getMonthSum() ) >= 0 )
				{
					addErrorMessage( "autopaymentForm:autoModeMonthSum", "Укажите корректную сумму!" );
					return;
				}

				autoMode.setMonthSum( autoMode.getMonthSum().setScale( 0, RoundingMode.FLOOR ) );

				if( autoMode.getMonthSumMax().compareTo( autoMode.getMonthSum() ) < 0 )
				{
					addErrorMessage( "autopaymentForm:autoModeMonthSum", "Сумма месячного лимита должна быть не более " + autoMode.getMonthSumMax() + "." );
					return;
				}

				autoMode.setMode( autopaymentMode );

				paymentBean.get().beginTransaction( autoMode.getDaySum(), autoMode );

				break;
			}

			case MONTH:
			{
				if( monthMode.getDay() < 1 || monthMode.getDay() > 31 )
				{
					addErrorMessage( "autopaymentForm:monthModeDay", "Укажите корректный день месяца!" );
					return;
				}

				if( monthMode.getSum() == null || BigDecimal.ZERO.compareTo( monthMode.getSum() ) >= 0 )
				{
					addErrorMessage( "autopaymentForm:monthModeSum", "Укажите корректную сумму!" );
					return;
				}

				monthMode.setSum( monthMode.getSum().setScale( 0, RoundingMode.FLOOR ) );

				if( monthMode.getSumMin().compareTo( monthMode.getSum() ) > 0
					|| monthMode.getSumMax().compareTo( monthMode.getSum() ) < 0 )
				{
					addErrorMessage( "autopaymentForm:monthModeSum",
									 "Сумма пополнения должна быть не менее " + monthMode.getSumMin() + " и не более " + monthMode.getSumMax() + "." );
					return;
				}

				monthMode.setMode( autopaymentMode );

				paymentBean.get().beginTransaction( monthMode.getSumMin(), monthMode );

				break;
			}

			case WEEK:
			{
				if( weekMode.getDay() < 1 || weekMode.getDay() > 7 )
				{
					addErrorMessage( "autopaymentForm:weekModeDay", "Укажите корректный день недели!" );
					return;
				}

				if( weekMode.getSum() == null || BigDecimal.ZERO.compareTo( weekMode.getSum() ) >= 0 )
				{
					addErrorMessage( "autopaymentForm:weekModeSum", "Укажите корректную сумму!" );
					return;
				}

				weekMode.setSum( weekMode.getSum().setScale( 0, RoundingMode.FLOOR ) );

				if( weekMode.getSumMin().compareTo( weekMode.getSum() ) > 0
					|| weekMode.getSumMax().compareTo( weekMode.getSum() ) < 0 )
				{
					addErrorMessage( "autopaymentForm:weekModeSum", "Сумма пополнения должна быть не менее " + weekMode.getSumMin() + " и не более " + weekMode.getSumMax() + "." );
					return;
				}

				weekMode.setMode( autopaymentMode );

				paymentBean.get().beginTransaction( weekMode.getSumMin(), weekMode );

				break;
			}

			default:
				break;
		}
	}

	public void autopaymentDeactivate()
		throws BGIllegalArgumentException, BGMessageException, BGIllegalAccessException, BGException
	{
		paymentBean.get().getPaymentProviderBean().autopaymentDeactivate( getContractId() );

		populate();
	}
}
