/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.payment;

import java.io.IOException;
import java.io.Serializable;
import java.math.BigDecimal;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import javax.enterprise.context.SessionScoped;
import javax.enterprise.inject.Instance;
import javax.faces.FacesException;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;
import javax.servlet.http.HttpServletRequest;
import javax.xml.ws.Holder;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.contract.api.common.service.ContractService;
import ru.bitel.bgbilling.kernel.contract.autopayment.common.bean.ContractAutopaymentMode;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.common.model.KeyValue;
import ru.bitel.common.model.MapHolder;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.Configuration;
import ru.bitel.mybgbilling.kernel.common.MyApplicationBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.contract.BalanceBean;
import ru.bitel.mybgbilling.kernel.payment.PaymentConfig.PaymentItem;
import ru.bitel.mybgbilling.kernel.payment.PaymentConfig.PaymentProvider;
import ru.bitel.mybgbilling.kernel.payment.PaymentConfigProducer.PaymentConfigProduce;
import ru.bitel.mybgbilling.modules.payment.AbstractPaymentBean;
import ru.bitel.mybgbilling.modules.payment.PaymentServiceProviderBean;

@Named
@SessionScoped
@BGInjection
public class PaymentBean
	extends AbstractBean
	implements Serializable
{
	private static final Logger logger = LogManager.getLogger();

    @Inject
    private MyApplicationBean applicationBean;

    @Inject
    private Configuration configuration;

	@Inject
	@PaymentConfigProduce
	private PaymentConfig paymentConfig;

	@Inject
	private BalanceBean balanceBean;

	@Inject
	private Instance<AutopaymentBean> autopaymentBean;
	
	@BGInject( module = false )
	private ContractService contractService;

	private Async<List<PaymentProvider>> paymentProviderList;
	private Async<Map<String, PaymentProvider>> paymentProviderMap;

	private Async<List<PaymentItem>> paymentItemList;

	private String paymentProviderId;

	private PaymentItem paymentItem;

	private Supplier<String> phone = () -> null;

	private Supplier<String> email = () -> null;

	private BigDecimal amount;

	private String mode;
	private String url;
	private Map<String, String> params;

	private String paymentType;

	private String result;

	@Override
	protected void init()
		throws BGException
	{
		super.init();

		final List<PaymentItem> paymentItemList = paymentConfig.getPayments();

		final Set<Integer> contractModuleIds = customerBean.getContractModuleIds();

		this.paymentProviderList = Async.of( () -> {

			return paymentConfig.getProviders().stream().filter( ( a ) -> a.getModuleId() == 0 || contractModuleIds.contains( a.getModuleId() ) ).collect( Collectors.toList() );
		} );

		this.paymentProviderMap = Async.of( () -> {

			return paymentProviderList.get().stream().collect( Collectors.toMap( PaymentProvider::getId, Function.identity() ) );
		} );

		this.paymentItemList = Async.of( () -> Utils.maskNull( paymentItemList ).stream().filter( ( a ) -> paymentProviderMap.get().containsKey( a.getProviderId() ) )
													.collect( Collectors.toList() ) );
		
		this.phone = Async.of( () -> contractService.contractModuleConfigGet( getContractId(), 0, "payment.phone" ) );
		this.email = Async.of( () -> contractService.contractModuleConfigGet( getContractId(), 0, "payment.email" ) );
	}

	public void payment()
	{
		this.paymentItem = null;
		this.paymentProviderId = null;
		this.step = 0;

		navigationBean.setPage( "kernel/payment", 0 );

		if( getPaymentProviderList().size() == 1 )
		{
			setPaymentProviderId( getPaymentProviderId() );
		}
	}

	public List<PaymentProvider> getPaymentProviderList()
	{
		return paymentProviderList.get();
	}

	public PaymentProvider getPaymentProvider( String id )
	{
		return paymentProviderMap.get().get( id );
	}

	public PaymentProvider getPaymentProvider()
	{
		return getPaymentProvider( getPaymentProviderId() );
	}

	public List<PaymentItem> getPaymentItemList()
	{
		return paymentItemList.get();
	}

	public String getPaymentProviderId()
	{
		if( getPaymentProviderList().size() == 1 )
		{
			return getPaymentProviderList().get( 0 ).getId();
		}

		return paymentProviderId;
	}

	public void setPaymentProviderId( String paymentProviderId )
	{
		logger.info( "setPaymentProviderId: " + paymentProviderId );

		PaymentProvider paymentProvider = getPaymentProvider( paymentProviderId );
		if( paymentProvider == null )
		{
			logger.info( "Payment provider not found with id: " + paymentProviderId );
			return;
		}

		this.paymentProviderId = paymentProviderId;

		autopaymentBean.get().populate();
	}

	public PaymentItem getPaymentItem()
	{
		return paymentItem;
	}

	public void setPaymentItem( PaymentItem paymentItem )
		throws IOException
	{
		PaymentProvider paymentProvider = getPaymentProvider( paymentItem.getProviderId() );
		if( paymentProvider == null )
		{
			logger.info( "Payment provider not found with id: " + paymentItem.getProviderId() );
			return;
		}

		this.paymentProviderId = paymentProvider.getId();

		this.paymentItem = paymentItem;

		autopaymentBean.get().populate();
	}

	public String getPhone()
	{
		return phone.get();
	}

	public void setPhone( String phone )
	{
		this.phone = () -> phone;
	}

	public String getEmail()
	{
		return email.get();
	}

	public void setEmail( String email )
	{
		this.email = () -> email;
	}

	public BigDecimal getAmount()
	{
		return amount;
	}

	public void setAmount( BigDecimal amount )
	{
		this.amount = amount;
	}

	private static final Pattern pattern = Pattern.compile( "\\d+" );

	private String currentPaymentServiceProviderId;
	private AbstractPaymentBean paymentServiceProvider;

	public String beginTransaction()
		throws BGException, IOException
	{
		return beginTransaction( amount, null );
	}

	String beginTransaction( BigDecimal amount, ContractAutopaymentMode autopaymentMode )
		throws BGException, IOException
	{
		if( amount == null || amount.compareTo( BigDecimal.ZERO ) <= 0 )
		{
			addErrorMessage( null, "Введите корректное значение в поле Сумма." );
			return null;
		}

		PaymentProvider paymentProvider = getPaymentProvider( getPaymentProviderId() );

		navigationBean.setModuleId( paymentProvider.getModuleId() );

		final FacesContext context = FacesContext.getCurrentInstance();
		final AbstractPaymentBean paymentServiceProvider = getPaymentProviderBean();

		navigationBean.setModuleId( getModuleId() );
		
		String phone = getPhone();
		String email = getEmail();
		
		List<Supplier<?>> asyncActions = new ArrayList<>();

		if( paymentServiceProvider.isNeedPhone() )
		{
			if( Utils.isBlankString( phone ) )
			{
				addErrorMessage( null, "Укажите номер телефона." );
				return null;
			}

			Matcher matcher = pattern.matcher( phone );
			if( !matcher.matches() )
			{
				addErrorMessage( null, "Укажите корректный номер телефона." );
				return null;
			}
			
			asyncActions.add( Async.of( () -> contractService.contractModuleConfigSet( getContractId(), 0, "payment.phone", phone ) ) );

			if( paymentServiceProvider.isNeedReceiptContacts() )
			{
				if( Utils.notBlankString( email ) )
				{
					if( !email.contains( "@" ) )
					{
						addErrorMessage( null, "Укажите корректный email." );
						return null;
					}
					
					asyncActions.add( Async.of( () -> contractService.contractModuleConfigSet( getContractId(), 0, "payment.email", email ) ) );
				}
			}
		}
		else if( paymentServiceProvider.isNeedReceiptContacts() )
		{
            if( autopaymentMode != null && paymentServiceProvider.isNeedAutopaymentEmail() && Utils.isBlankString( email ) )
            {
                addErrorMessage( null, "Укажите email." );
                return null;
            }

            if( Utils.isBlankString( phone ) && Utils.isBlankString( email ) )
            {
                addErrorMessage( null, "Укажите номер телефона или email." );
                return null;
            }

			if( Utils.notBlankString( phone ) )
			{
				Matcher matcher = pattern.matcher( phone );
				if( !matcher.matches() )
				{
					addErrorMessage( null, "Укажите корректный номер телефона." );
					return null;
				}
				
				asyncActions.add( Async.of( () -> contractService.contractModuleConfigSet( getContractId(), 0, "payment.phone", phone ) ) );
			}

			if( Utils.notBlankString( email ) )
			{
				if( !email.contains( "@" ) )
				{
					addErrorMessage( null, "Укажите корректный email." );
					return null;
				}
				
				asyncActions.add( Async.of( () -> contractService.contractModuleConfigSet( getContractId(), 0, "payment.email", email ) ) );
			}
		}
        else if( autopaymentMode != null && paymentServiceProvider.isNeedAutopaymentEmail() && Utils.isBlankString( email ) )
        {
            addErrorMessage( null, "Укажите email." );
            return null;
        }

		final ExternalContext ec = context.getExternalContext();

		String resultUrl = null;
		String successUrl = null;
		String failureUrl = null;

		try
		{
            final URI baseURI = configuration.getBaseURI();
            
            if( baseURI != null )
            {
                URI resultUri = new URI( baseURI.getScheme(), null, baseURI.getHost(), baseURI.getPort(), baseURI.getPath() + "/payment.xhtml", null,
                    null );
                URI successUri = new URI( baseURI.getScheme(), null, baseURI.getHost(), baseURI.getPort(), baseURI.getPath() + "/payment.xhtml",
                    "result=success", null );
                URI failureUri = new URI( baseURI.getScheme(), null, baseURI.getHost(), baseURI.getPort(), baseURI.getPath() + "/payment.xhtml",
                    "result=failure", null );

                resultUrl = resultUri.toASCIIString();
                successUrl = successUri.toASCIIString();
                failureUrl = failureUri.toASCIIString();
            }
            else
            {
                URI resultUri = new URI( ec.getRequestScheme(), null, ec.getRequestServerName(), ec.getRequestServerPort(), ec.getRequestContextPath() + "/payment.xhtml", null,
                    null );
                URI successUri = new URI( ec.getRequestScheme(), null, ec.getRequestServerName(), ec.getRequestServerPort(), ec.getRequestContextPath() + "/payment.xhtml",
                    "result=success", null );
                URI failureUri = new URI( ec.getRequestScheme(), null, ec.getRequestServerName(), ec.getRequestServerPort(), ec.getRequestContextPath() + "/payment.xhtml",
                    "result=failure", null );

                resultUrl = resultUri.toASCIIString();
                successUrl = successUri.toASCIIString();
                failureUrl = failureUri.toASCIIString();
            }
		}
		catch( URISyntaxException e )
		{
			throw new FacesException( e );
		}

		Holder<String> url = new Holder<String>();
		Holder<String> mode = new Holder<String>();
		Holder<MapHolder<String, String>> params = new Holder<MapHolder<String, String>>();

		params.value = new MapHolder<String, String>( new HashMap<String, String>() );

		params.value.getMap().put( "resultUrl", resultUrl );
		params.value.getMap().put( "successUrl", successUrl );
		params.value.getMap().put( "failureUrl", failureUrl );

		HttpServletRequest request = (HttpServletRequest)ec.getRequest();

		params.value.getMap().put( "ipAddress", applicationBean.getRemoteAddr( request ) );

		if( Utils.notBlankString( phone ) )
		{
			params.value.getMap().put( "phone", phone );
		}
		
        if( Utils.notBlankString( email ) )
        {
            params.value.getMap().put( "email", email );
        }

		if( paymentServiceProvider.isNeedReceiptContacts() )
		{
			if( Utils.notBlankString( email ) )
			{
				params.value.getMap().put( "receiptTo", email );
			}
			else if( Utils.notBlankString( phone ) )
			{
				params.value.getMap().put( "receiptTo", phone );
			}
		}

		List<KeyValue<String, String>> paymentTypes = paymentServiceProvider.getPaymentTypes();
		if( paymentTypes == null || paymentTypes.isEmpty() )
		{}
		else if( paymentTypes.size() == 1 )
		{
			params.value.getMap().put( "paymentType", paymentTypes.get( 0 ).getKey() );
		}
		else
		{
			if( paymentTypes.stream().filter( a -> a.getKey().equals( paymentType ) ).count() == 0 )
			{
				addErrorMessage( null, "Укажите корректный тип платежа." );
				return null;
			}

			params.value.getMap().put( "paymentType", paymentType );
		}

		this.paymentServiceProvider = paymentServiceProvider;
		
		// ждем все асинхронные действия
        for( Supplier<?> a : asyncActions )
        {
            a.get();
        }

		if( autopaymentMode != null )
		{
			paymentServiceProvider.beginAutopaymentTransaction( getContractId(), amount, url, mode, params, autopaymentMode );
		}
		else
		{
			paymentServiceProvider.beginTransaction( getContractId(), amount, url, mode, params );
		}

		this.url = url.value;
		this.mode = mode.value;
		this.params = params.value != null ? params.value.getMap() : null;

		logger.info( "Mode: " + this.mode );
		logger.info( "URL: " + this.url );
		logger.info( "Params: " + this.params );

		this.step = 1;

		if( "redirect".equals( getMode() ) )
		{
			ec.redirect( getUrl() );
		}

		return null;
	}

	public String getMode()
	{
		return mode;
	}

	public String getUrl()
	{
		return url;
	}

	public Map<String, String> getParams()
	{
		return params;
    }

    public void endTransaction()
        throws IOException, BGException
    {
        endTransaction( true );
    }

    public void endTransaction( boolean redirect )
        throws IOException, BGException
    {
		if( this.step == 1 )
		{
			if( this.paymentServiceProvider != null )
			{
				Holder<MapHolder<String, String>> params = new Holder<MapHolder<String, String>>();
				params.value = new MapHolder<String, String>( this.params );

				this.paymentServiceProvider.endTransaction( getContractId(), amount, new Holder<String>( url ), new Holder<String>( mode ), params );
			}

			this.step = 2;

			FacesContext context = FacesContext.getCurrentInstance();
			ExternalContext ec = context.getExternalContext();

			result = ec.getRequestParameterMap().get( "result" );

			if( Utils.isBlankString( result ) )
			{
				this.step = 0;
				this.amount = null;
			}
			else if( "success".equals( result ) )
			{
				this.amount = null;
				addInfoMessage( null, "Платеж успешно проведен." );
			}
			else if( "failure".equals( result ) )
			{
				addErrorMessage( null, "Платеж не был проведен." );
			}

			navigationBean.setPage( "kernel/balance", 0 );

			balanceBean.setMonth( new Date() );
			balanceBean.populate();
        }

        if( redirect )
        {
            FacesContext fc = FacesContext.getCurrentInstance();
            ExternalContext ec = fc.getExternalContext();

            ec.redirect( "index.xhtml" );
        }
	}

	public String getResult()
	{
		return result;
	}

	public void checkResult()
	{
		if( !FacesContext.getCurrentInstance().getRenderResponse() )
		{
			return;
		}

		if( step == 2 )
		{
			if( "success".equals( result ) )
			{
				this.amount = null;
				addInfoMessage( null, "Платеж успешно проведен." );
			}
			else if( "failure".equals( result ) )
			{
				addErrorMessage( null, "Платеж не был проведен." );
			}

			step = 0;
		}
	}

	public boolean hasPaymentModes()
	{
		return getPaymentProviderId() != null
			   || (getPaymentProviderList() != null && getPaymentProviderList().size() > 0)
			   || (getPaymentItemList() != null && getPaymentItemList().size() > 0);
	}

	private AbstractPaymentBean getPaymentServiceProvider0()
	{
		PaymentProvider paymentProvider = getPaymentProvider( getPaymentProviderId() );

		final int moduleId = navigationBean.getModuleId();
		try
		{
			navigationBean.setModuleId( paymentProvider.getModuleId() );

			final FacesContext context = FacesContext.getCurrentInstance();
			final AbstractPaymentBean paymentServiceProvider =
				(AbstractPaymentBean)context.getELContext().getELResolver().getValue( context.getELContext(), null, paymentProvider.getModule() + "PaymentBean" );
			if( paymentServiceProvider == null )
			{
				logger.error( "Can't find paymentServiceProvider with name: " + paymentProvider.getModule() + "PaymentBean" );
			}

			return paymentServiceProvider;
		}
		finally
		{
			navigationBean.setModuleId( moduleId );
		}
	}

	public AbstractPaymentBean getPaymentProviderBean()
	{
		String id = getPaymentProviderId();

		if( paymentServiceProvider != null && id != null && id.equals( currentPaymentServiceProviderId ) )
		{
			return paymentServiceProvider;
		}

		paymentServiceProvider = getPaymentServiceProvider0();
		currentPaymentServiceProviderId = id;

		return paymentServiceProvider;
	}

	public String getPaymentType()
	{
		return paymentType;
	}

	public void setPaymentType( String paymentType )
	{
		this.paymentType = paymentType;
	}

	public List<KeyValue<String, String>> getPaymentTypes()
	{
		PaymentServiceProviderBean paymentServiceProvider = getPaymentProviderBean();
		return paymentServiceProvider != null ? paymentServiceProvider.getPaymentTypes() : Collections.emptyList();
	}
}
