/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
 */
package ru.bitel.mybgbilling.kernel.realm;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.enterprise.context.SessionScoped;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;
import javax.security.auth.login.AccountNotFoundException;
import javax.security.auth.login.CredentialException;
import javax.security.auth.login.LoginException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.xml.ws.Holder;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import groovy.lang.Closure;
import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.contract.api.common.bean.Contract;
import ru.bitel.bgbilling.kernel.contract.api.common.bean.parameter.ContractParameterGroupAttr;
import ru.bitel.bgbilling.kernel.contract.api.common.service.ContractService;
import ru.bitel.bgbilling.kernel.directory.api.common.bean.Directory;
import ru.bitel.common.TimeUtils;
import ru.bitel.common.Utils;
import ru.bitel.common.model.Page;
import ru.bitel.mybgbilling.kernel.common.Configuration.AuthenticationMode;
import ru.bitel.mybgbilling.kernel.common.MySessionBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.oss.kernel.entity.common.bean.EntityAttr;
import ru.bitel.oss.kernel.entity.common.bean.EntityAttrEmail;
import ru.bitel.oss.kernel.entity.common.bean.EntityAttrPhone;
import ru.bitel.oss.kernel.entity.common.bean.EntityAttrText;
import ru.bitel.oss.kernel.entity.common.bean.EntitySpecAttrType;
import ru.bitel.oss.kernel.entity.common.bean.filter.FilterEntityAttr;
import ru.bitel.oss.kernel.entity.common.bean.filter.FilterEntityAttrEmail;
import ru.bitel.oss.kernel.entity.common.bean.filter.FilterEntityAttrPhone;
import ru.bitel.oss.kernel.entity.common.bean.filter.FilterEntityAttrText;

@Named
@SessionScoped
@BGInjection
public class KernelAuthenticationBean
    implements AuthenticationBean, Serializable
{
    private static final long serialVersionUID = -2498601413588223544L;

    private static final Logger logger = LogManager.getLogger();

    @Inject
    private LoginBean loginBean;

    @BGInject( type = ContractParameterGroupAttr.class, module = false )
    private Directory<ContractParameterGroupAttr> contractParameterGroupAttrDirectory;

    @Override
    public void login( final HttpServletRequest request, final AuthenticationMode mode, final int moduleId )
        throws BGException, LoginException, ServletException
    {
        switch( mode.getMode() )
        {
            case "textParam":
            case "phoneParam":
            case "emailParam":
            default:
                loginByParam( request, mode, moduleId );
                break;
        }
    }

    private void loginByParam( final HttpServletRequest request, final AuthenticationMode mode, final int moduleId )
        throws BGException, LoginException, ServletException
    {
        ContractParameterGroupAttr parameterSpec = contractParameterGroupAttrDirectory.get( mode.getParameterId() );
        if( parameterSpec == null )
        {
            throw new LoginException( "Cannot find parameter with type=" + mode.getParameterId() );
        }

        String username = loginBean.getUsername();
        final String password = loginBean.getPassword();

        logger.info( "authentication by " + parameterSpec.getTypeName() + " param " + username );

        final Date now = new Date();

        final FacesContext context = FacesContext.getCurrentInstance();

        MySessionBean wsClientBean = MySessionBean.get( context );
        ContractService contractService = wsClientBean.getPort( ContractService.class, 0 );

        List<FilterEntityAttr> entityFilter = new ArrayList<>();

        switch( EntitySpecAttrType.getEntitySpecAttrType( parameterSpec.getType() ) )
        {
            case TEXT:
            {
                username = username.trim();

                entityFilter.add( new FilterEntityAttrText( Collections.singleton( parameterSpec.getId() ), username, FilterEntityAttr.MODE_EQ ) );
                break;
            }

            case PHONE:
            {
                username = EntityAttrPhone.phoneUnformatted( username ); // убираем пробелы, тире и плюсы

                Closure<String> usernameClosure = mode.getUsername();
                if( usernameClosure != null )
                {
                    username = usernameClosure.call( username );
                }

                logger.info( "authentication by phone " + username );

                entityFilter.add( new FilterEntityAttrPhone( Collections.singleton( parameterSpec.getId() ), username, FilterEntityAttr.MODE_EQ ) );
                break;
            }

            case EMAIL:
            {
                username = username.trim();

                entityFilter.add( new FilterEntityAttrEmail( Collections.singleton( parameterSpec.getId() ), username, FilterEntityAttr.MODE_EQ ) );
                break;
            }

            default:
                throw new LoginException( "Cannot search parameter with type=" + mode.getParameterId() );
        }

        Contract contract = null;
        boolean authenticated = false;

        List<Contract> contractList = contractService.contractList( null, null, -1, 0, entityFilter, true, false, false, new Holder<Page>( new Page( 1, 10 ) ) );
        if( contractList != null && contractList.size() > 0 )
        {
            for( Contract c : contractList )
            {
                if( !TimeUtils.dateInRange( now, c.getDateFrom(), c.getDateTo() ) )
                {
                    continue;
                }

                EntityAttr parameter = contractService.contractParameterGet( c.getId(), parameterSpec.getId() );
                if( parameter == null )
                {
                    logger.error( "contract found but parameter is empty" );
                    continue;
                }

                switch( EntitySpecAttrType.getEntitySpecAttrType( parameterSpec.getType() ) )
                {
                    case TEXT:
                        if( !username.equalsIgnoreCase( ((EntityAttrText)parameter).getValue() ) )
                        {
                            continue;
                        }
                        break;

                    case PHONE:
                        if( !((EntityAttrPhone)parameter).getUnformattedPhoneList().contains( username ) )
                        {
                            continue;
                        }
                        break;

                    case EMAIL:
                        String uname = username;
                        if( !((EntityAttrEmail)parameter).getAddressList().stream().anyMatch( a-> uname.equalsIgnoreCase( a ) ) )
                        {
                            continue;
                        }
                        break;

                    default:
                        throw new LoginException( "Cannot search parameter with type=" + mode.getParameterId() );
                }

                contract = c;

                if( Utils.isBlankString( c.getPassword() ) || !password.equals( c.getPassword() ) )
                {
                    continue;
                }

                authenticated = true;
                break;
            }
        }

        if( contract == null )
        {
            throw new AccountNotFoundException( "Contract not found" );
        }

        if( !authenticated )
        {
            Exception ex = new CredentialException( "Contract password incorrect" );
            throw new MyFailedLoginException( null, contract.getId(), ex.getMessage(), ex );
        }

        if( mode.getFilter() != null )
        {
            if( Boolean.FALSE.equals( mode.getFilter().call( contract ) ) )
            {
                contract = null;
            }
        }

        if( "unauthCustomer".equals( mode.getRole() ) )
        {
            request.login( "[unauthCustomer]" + contract.getTitle(), contract.getPassword() );
        }
        else
        {
            request.login( contract.getTitle(), contract.getPassword() );
        }
    }

    @Override
    public void preprocess( HttpServletRequest request, AuthenticationMode mode, int moduleId, LoginBean loginBean )
    {
    }
}
