/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.realm;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.Serializable;
import java.security.Principal;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.enterprise.context.Conversation;
import javax.enterprise.context.ConversationScoped;
import javax.faces.application.FacesMessage;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;
import javax.security.auth.login.AccountNotFoundException;
import javax.security.auth.login.CredentialException;
import javax.security.auth.login.FailedLoginException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.xml.ws.Holder;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import groovy.util.ConfigObject;
import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.contract.api.common.bean.Contract;
import ru.bitel.bgbilling.kernel.contract.api.common.service.ContractService;
import ru.bitel.bgbilling.kernel.contract.selfcare.common.service.ContractLoginService;
import ru.bitel.common.function.Async;
import ru.bitel.common.groovy.GroovyBeanBuilder;
import ru.bitel.common.model.MapHolder;
import ru.bitel.common.model.Page;
import ru.bitel.mybgbilling.kernel.common.Configuration;
import ru.bitel.mybgbilling.kernel.common.Configuration.AuthenticationMode;
import ru.bitel.mybgbilling.kernel.common.MyApplicationBean;
import ru.bitel.mybgbilling.kernel.common.MySessionBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.utils.CommonUtils;
import ru.bitel.mybgbilling.kernel.common.utils.Utils;
import ru.bitel.mybgbilling.kernel.navigation.NavigationBean;
import ru.bitel.mybgbilling.kernel.ws.CustomerBean;
import ru.bitel.mybgbilling.kernel.ws.Pool;
import ru.bitel.mybgbilling.kernel.ws.WSClient;

@Named

//@javax.faces.view.ViewScoped
@ConversationScoped
@BGInjection
public class LoginBean
	implements Serializable
{
	private static final long serialVersionUID = -2421257459318323205L;

	private static final Logger logger = LogManager.getLogger();

	@Inject
	private Configuration configuration;

	@Inject
	private MyApplicationBean application;

	@Inject
	private MySessionBean sessionBean;

	@BGInject( module = false )
	private ContractLoginService contractLoginService;

	@Inject
	private Conversation conversation;

	private AuthenticationMode authenticationMode;

	private String username;
	private String password;
	private String captcha;
	private String email;
	private String appContext;
	private String requestedPage;
	private String requestedQuery;
	private Map<String, String[]> requestedParams;
	
    private WSConnection connection;
	
	public void setConversationTimeout( int minutes )
	{
		conversation.setTimeout( TimeUnit.MINUTES.toMillis( minutes ) );
	}

	@PostConstruct
	private void init()
	{
		logger.info( "LoginBean - init" );

		authenticationMode = configuration.getAuthentication().getModes().get( 0 );

		// окно логина живет 3 минуты
		if( conversation.isTransient() )
		{
			conversation.begin();
			conversation.setTimeout( TimeUnit.MINUTES.toMillis( 3 ) );
		}

		final FacesContext context = FacesContext.getCurrentInstance();
		final HttpServletRequest request = (HttpServletRequest)context.getExternalContext().getRequest();

        if ( request.getRequestURI().startsWith( request.getContextPath() ) )
        {
            requestedPage = request.getRequestURI().substring( request.getContextPath().length() + 1 );
        }

        if ( Utils.notBlankString( request.getQueryString() ) )
        {
            requestedQuery = request.getQueryString();
        }

		requestedParams = request.getParameterMap();

		checkPasswordOnce( request );
	}

	private void checkPasswordOnce( HttpServletRequest request )
	{
		String passwordOnce = request.getParameter( "passwordOnce" );
		if( Utils.notBlankString( passwordOnce ) )
		{
			logger.info( "passwordOnce " + passwordOnce );

			if( application.needCaptcha( application.getRemoteAddr( request ), null ) )
			{
				logger.info( "too many errors with login with remoteAddr: " + application.getRemoteAddr( request ) );
				return;
			}

			try
			{
				final ContractService contractService = sessionBean.getPort( ContractService.class, 0 );

				int contractId = contractService.contractPasswordOnceCheck( passwordOnce );
				if( contractId > 0 )
				{
					logger.info( "found contract by passwordOnce " + contractId );

					Contract contract = contractService.contractGet( contractId );

					setUsername( contract.getTitle() );
					setPassword( contract.getPassword() );
					try
					{
						loginInternal();
					}
					finally
					{
						setUsername( "" );
						setPassword( "" );
					}

					CustomerBean customerBean = FacesContext.getCurrentInstance().getApplication().evaluateExpressionGet( FacesContext.getCurrentInstance(), "#{customerBean}", CustomerBean.class );
					customerBean.setPasswordOnce( true );

					NavigationBean navigationBean = FacesContext.getCurrentInstance().getApplication().evaluateExpressionGet( FacesContext.getCurrentInstance(), "#{navigationBean}", NavigationBean.class );
					navigationBean.setPage( "kernel/password" );

					try
					{
						FacesContext.getCurrentInstance().getExternalContext().redirect( "index.xhtml" );
					}
					catch( IOException e )
					{
						logger.error( e.getMessage(), e );
					}
				}
				else
				{
					logger.info( "not found contract by passwordOnce" );

					application.processLoginFailure( application.getRemoteAddr( request ), null );
				}
			}
			catch( Exception ex )
			{
				logger.error( ex.getMessage(), ex );
			}
		}
	}

	@PreDestroy
	private void destroy()
	{}

	public void preprocessAuthentication()
	{
        if ( this.authenticationMode != null )
        {
            preprocessAuthenticationMode( this.authenticationMode );
        }
	}

	private void preprocessAuthenticationMode( AuthenticationMode authenticationMode )
	{
		if( this.authenticationMode == null || "contract".equals( this.authenticationMode.getMode() ) )
		{
			return;
		}

		final FacesContext context = FacesContext.getCurrentInstance();
		final HttpServletRequest request = (HttpServletRequest)context.getExternalContext().getRequest();
		//request.getSession().setMaxInactiveInterval( 10 );

		final AuthenticationBean authenticationBean =
			(AuthenticationBean)context.getELContext().getELResolver().getValue( context.getELContext(), null,
																				 this.authenticationMode.getModule() + "AuthenticationBean" );
		if( authenticationBean == null )
		{
			return;
		}

		authenticationBean.preprocess( request, this.authenticationMode, this.authenticationMode.getModuleId(), this );
	}

	public String getUsername()
	{
		return this.username;
	}

	public void setUsername( String username )
	{
		this.username = username;
	}

	public String getPassword()
	{
		return this.password;
	}

	public void setPassword( String password )
	{
		this.password = password;
	}

	public String getCaptcha()
	{
		return captcha;
	}

	public void setCaptcha( String captcha )
	{
		this.captcha = captcha;
	}

	public String getEmail()
	{
		return email;
	}

	public void setEmail( String email )
	{
		this.email = email;
	}

	public AuthenticationMode getAuthenticationMode()
	{
		return authenticationMode;
	}

	public void setAuthenticationMode( final AuthenticationMode authenticationMode )
	{
		this.authenticationMode = authenticationMode;

		//preprocessAuthenticationMode( authenticationMode );
	}

	public void populate()
	{
		logger.trace( "populate" );
	}

	public String loginInternal()
		throws IOException
	{
		logger.info( "loginInternal" );

		final FacesContext context = FacesContext.getCurrentInstance();
		final HttpServletRequest request = (HttpServletRequest)context.getExternalContext().getRequest();
		try
		{
		    doLogin( request );

			return "/index.xhtml";
		}
		catch( Exception securityException )
		{
			logger.info( "Failed login: " + securityException.getMessage() );

			processLoginFailure( context, securityException );
		}

		return null;
	}

	public boolean isNeedCaptcha()
	{
		return sessionBean.isNeedCaptcha();
	}

	public String login()
		throws IOException
	{
		if( login( true, true ) )
		{
			return "/index1.xhtml";
		}
		else
		{
			return "/login.xhtml";
		}
	}

	/**
	 * Аутентификация абонента. Возвращает true, если абонент аутентифицировался.<br/>
	 * Метод также используется в BoardPortal.
	 * @param redirect нужно ли вызывать redirect при удачном логине
	 * @param messages нужно ли добавлять сообщения об ошибках при неудачном логине
	 * @return
	 * @throws IOException
	 */
	public boolean login( boolean redirect, boolean messages )
		throws IOException
	{
		logger.info( "login" );

		final FacesContext context = FacesContext.getCurrentInstance();
		final HttpServletRequest request = (HttpServletRequest)context.getExternalContext().getRequest();

		try
		{
			if ( isNeedCaptcha() )
			{
				if( ru.bitel.common.Utils.isBlankString( captcha ) )
				{
					Utils.addErrorMessage( context, "loginForm:captcha", "login.captchaEmpty", "Введите символы с картинки" );
					return false;
				}

				HttpSession session = request.getSession( true );
				String captchaValue = (String)session.getAttribute( "login.captcha.value" );
				if( !captcha.equalsIgnoreCase( captchaValue ) )
				{
					this.captcha = "";
					Utils.addErrorMessage( context, "loginForm:captcha", "login.captchaNotValid", "Введите символы с картинки" );
					return false;
				}

				session.removeAttribute( "login.captcha.value" );
			}

			this.captcha = "";

			if( !configuration.getAuthentication().getModes().contains( authenticationMode ) )
			{
				this.authenticationMode = configuration.getAuthentication().getModes().get( 0 );
			}
			
			logger.info( "authenticationMode.getMode() = " + authenticationMode.getMode() );

			if( "contract".equals( this.authenticationMode.getMode() ) )
			{
			    doLogin( request );
			}
			else
			{
				final AuthenticationBean authenticationBean =
					(AuthenticationBean)context.getELContext().getELResolver().getValue( context.getELContext(), null,
																						 this.authenticationMode.getModule() + "AuthenticationBean" );

				authenticationBean.login( request, this.authenticationMode, this.authenticationMode.getModuleId() );
			}

			application.processLoginOk( application.getRemoteAddr( request ), this.username );

			sessionBean.setNeedCaptcha( false );

            if ( redirect )
            {
                String requestedQuery = this.requestedQuery;
                if ( Utils.notBlankString( requestedQuery ) )
                {
                    requestedQuery = requestedQuery.replaceAll( "\\bcid=\\d+&\\b", "" ).replaceAll( "\\b&cid=\\d+\\b", "" ).replaceAll( "\\bcid=\\d+\\b", "" );
                }
                context.getExternalContext().redirect( "index.xhtml" + (Utils.notBlankString( requestedQuery ) ? ('?' + requestedQuery) : "") );
            }

			Contract contract = (Contract)request.getSession().getAttribute( "mybgbilling.contract" );

			final ContractLoginService contractLoginService = this.contractLoginService;

			HttpServletRequest req = (HttpServletRequest)FacesContext.getCurrentInstance().getExternalContext().getRequest();

			final Integer contractId = (Integer)contract.getId();
			final String remoteAddress = application.getRemoteAddr( req );
			final String sessionId = req.getSession().getId();
			final String username = this.username;

			Async.of( () -> {
				try
				{
					contractLoginService.contractLoginLogAdd( contractId, true, remoteAddress, sessionId, username, "c", null );
				}
				catch( Throwable ex )
				{
				    logger.error( "error #1" );
					logger.error( ex.getMessage(), ex );
				}
			} );

	        try
	        {
	            final String remoteAddr = application.getRemoteAddr( request );

	            Map<String, String> params =
	                request.getParameterMap().entrySet().stream().filter( a -> a.getKey().startsWith( "javax." ) || a.getValue() == null || a.getValue().length == 0 )
	                       //.map( a -> new AbstractMap.SimpleEntry<>( a.getKey(), a.getValue()[0] ) )
	                       .collect( Collectors.toMap( a -> a.getKey(), a -> a.getValue()[0] ) );
	            
                params.put( "remoteAddr", remoteAddr );
	            params.put( "requestURI", request.getRequestURI() );
                params.put( "authenticationMode.mode", authenticationMode.getMode() );
                params.put( "authenticationMode.module", authenticationMode.getModule() );
                params.put( "authenticationMode.moduleId", String.valueOf( authenticationMode.getModuleId() ) );
                params.put( "authenticationMode.parameterId", String.valueOf( authenticationMode.getParameterId() ) );
                params.put( "authenticationMode.role", authenticationMode.getRole() );

                contractLoginService.contractLoginEvent( contractId, new MapHolder<>( params ) );
	        }
	        catch( Throwable ex )
	        {
                logger.error( "error #2" );
	            logger.error( ex.getMessage(), ex );
	        }

			conversation.end();

			return true;
		}
		catch( Exception securityException )
		{
            logger.error( "error #3" );
			logger.info( "Failed login: " + securityException.getMessage() );

			if( messages )
			{
				processLoginFailure( context, securityException );
			}

			processLoginFailure2( context, securityException, request );

			application.processLoginFailure( application.getRemoteAddr( request ), this.username );

			boolean needCaptcha = application.needCaptcha( application.getRemoteAddr( request ), this.username );
			sessionBean.setNeedCaptcha( needCaptcha );

			//contractLoginService.contractLoginLogAdd( contractId, success, remoteAddress, sessionId, username, data );
		}

		return false;
	}
	
	private void doLogin( final HttpServletRequest request )
	    throws BGException, AccountNotFoundException, CredentialException
	{
        ContractService contractService = getWSConnection( request ).pool.take();
        try
        {
            Holder<Page> pageHolder = new Holder<>();
            pageHolder.value = new Page( 1, 1 );

            String regexUsername = username.replaceAll( "\\[", "\\\\[" ).replaceAll( "\\]", "\\\\]" );
            List<Contract> contractList = contractService.contractList( "^" + regexUsername + "$", null, -1, 0L, null, true, false, true, pageHolder );

            if( contractList != null )
            {
                for( Contract contract : contractList )
                {
                    if ( username.equalsIgnoreCase( contract.getTitle() ) )
                    {
                        logger.info( "Contract found: id=" + contract.getId() + ", title=" + contract.getTitle() );
                        
                        if ( contract.getPassword().equals( password ) )
                        {
                            request.getSession().setAttribute( "mybgbilling.contract", contract );
                            return;
                        }

                        throw new CredentialException( "Неверный пароль" );
                    }
                }
            }

            logger.info( "Contract not found: " + username );
            throw new AccountNotFoundException( "Договор не найден." );
        }
        finally
        {
            connection.pool.recycle( contractService );
        }
	}

	private WSConnection getWSConnection( HttpServletRequest request )
	{
        File configFile = new File( request.getServletContext().getRealPath( "/WEB-INF/mybgbilling-conf.groovy" ) );

//        if ( configFile == null || !configFile.exists() )
//        {
//            File configDir = new File( System.getProperty( "jboss.server.config.dir" ) );
//            File deploymentsDir = new File( configDir.getParentFile(), "deployments" );
//            configFile = new File( deploymentsDir, fileName );
//        }

        if ( configFile == null || !configFile.exists() )
        {
            configFile = new File( System.getProperty( "jboss.server.config.dir" ) + "/mybgbilling-conf.groovy" );
        }

        final long lm = configFile.lastModified();

        if ( connection == null || connection.lm == lm )
        {
	        connection = new WSConnection();

            logger.info( "Load configuration file: " + configFile );

            File f = configFile;
	        
            try
            {
                GroovyBeanBuilder.addCommonStaticStars( "ru.bitel.mybgbilling.kernel.common.utils.CommonUtils" );

                ConfigObject config = new GroovyBeanBuilder().load( new FileInputStream( f ), "mybgbilling-conf.groovy" );
                config = (ConfigObject)config.getProperty( "bgbilling" );

                connection.url = (String)config.getOrDefault( "url", connection.url );
                connection.user = (String)config.getOrDefault( "user", connection.user );
                connection.password = (String)config.getOrDefault( "password", connection.password );

                //переопределяем значениями из system properties, если такие указаны
                connection.url = System.getProperty( "my.bgbilling.url", connection.url );
                connection.user = System.getProperty( "my.bgbilling.user", connection.user );
                connection.password = System.getProperty( "my.bgbilling.password", connection.password );
            }
            catch( Exception e )
            {
                logger.error( e.getMessage(), e );
                connection = new WSConnection();
            }

            connection.lm = lm;

            WSClient wsClient = new WSClient( connection.url, connection.user, connection.password, true, true );
            connection.pool = new Pool<ContractService>( () -> wsClient.getPort( ContractService.class, 0 ), 5 );
	    }
	    return connection;
	}

	private void processLoginFailure2( final FacesContext context, final Exception e, HttpServletRequest req )
	{
		if( e == null )
		{
			return;
		}

		int contractId = 0;

		logger.info( e.getMessage() );

		if( e instanceof MyFailedLoginException )
		{
			MyFailedLoginException ex = (MyFailedLoginException)e;
			Principal identity = ex.getIdentity();
//			if( identity instanceof ContractPrincipal )
//			{
//				contractId = ((ContractPrincipal)identity).getContract().getId();
//			}
//			else if( ex.getContractId() > 0 )
//			{
//				contractId = ex.getContractId();
//			}
		}
		else if( e.getCause() instanceof MyFailedLoginException )
		{
			MyFailedLoginException ex = (MyFailedLoginException)e.getCause();
			Principal identity = ex.getIdentity();
//			if( identity instanceof ContractPrincipal )
//			{
//				contractId = ((ContractPrincipal)identity).getContract().getId();
//			}
//			else if( ex.getContractId() > 0 )
//			{
//				contractId = ex.getContractId();
//			}
		}

		final ContractLoginService contractLoginService = this.contractLoginService;

		final int contractId0 = contractId;
		final String remoteAddress = application.getRemoteAddr( req );
		final String sessionId = req.getSession().getId();
		final String username = this.username;
		final String password = this.password;

		final String mode = Optional.ofNullable( this.authenticationMode ).map( a -> a.getMode() ).filter( a -> !"contract".equals( a ) ).map( a -> "[" + a + "] " ).orElse( "" );

		Async.of( () -> {
			try
			{
				contractLoginService.contractLoginLogAdd( contractId0, false, remoteAddress, sessionId, mode + username, "c", "pswd = {" + password + "}" );
			}
			catch( Throwable ex )
			{
				logger.error( ex.getMessage(), ex );
			}
		} );
	}

	private String processLoginFailure( final FacesContext context, Exception e )
	{
		if( e == null )
		{
			return null;
		}

		logger.info( "org.jboss.security.exception: " + e.getLocalizedMessage() );

		if( e instanceof MyFailedLoginException && e.getCause() instanceof Exception )
		{
			e = (Exception)e.getCause();
		}

		if( e.getCause() instanceof AccountNotFoundException )
		{
			Utils.addErrorMessage( context, "loginForm:username", "login.usernameNotFound", e.getCause().getLocalizedMessage() );

			return e.getCause().getLocalizedMessage();
		}

		if( e instanceof AccountNotFoundException )
		{
			Utils.addErrorMessage( context, "loginForm:username", "login.usernameNotFound", e.getLocalizedMessage() );

			return e.getLocalizedMessage();
		}

		if( e.getCause() instanceof CredentialException )
		{
			Utils.addErrorMessage( context, "loginForm:password", "login.passwordError", e.getCause().getLocalizedMessage() );

			return e.getCause().getLocalizedMessage();
		}

		if( e instanceof CredentialException )
		{
			Utils.addErrorMessage( context, "loginForm:password", "login.passwordError", e.getLocalizedMessage() );

			return e.getLocalizedMessage();
		}

		if( e.getCause() instanceof FailedLoginException )
		{
			Utils.addErrorMessage( context, null, "login.fail", e.getCause().getLocalizedMessage() );

			return e.getCause().getLocalizedMessage();
		}

		if( e instanceof FailedLoginException )
		{
			Utils.addErrorMessage( context, null, "login.fail", e.getLocalizedMessage() );

			return e.getLocalizedMessage();
		}

		Utils.addErrorMessage( context, null, "login.fail", e.getLocalizedMessage() );

		return e.getLocalizedMessage();
	}

	public String getLoginFailureMsg( Exception e )
	{
		String result = getLoginFailureMsg0( e );
		if( result != null )
		{
			FacesContext.getCurrentInstance().addMessage( "username", new FacesMessage( FacesMessage.SEVERITY_ERROR, "Error Message", result ) );
		}

		return result;
	}

	private String getLoginFailureMsg0( Exception e )
	{
        if ( e == null )
        {
            return null;
        }

        if ( e.getCause() instanceof AccountNotFoundException )
        {
            return e.getCause().getLocalizedMessage();
        }

        if ( e instanceof AccountNotFoundException )
        {
            return e.getLocalizedMessage();
        }

        if ( e.getCause() instanceof CredentialException )
        {
            return e.getCause().getLocalizedMessage();
        }

        if ( e instanceof CredentialException )
        {
            return e.getLocalizedMessage();
        }

        if ( e.getCause() instanceof FailedLoginException )
        {
            return e.getCause().getLocalizedMessage();
        }

        if ( e instanceof FailedLoginException )
        {
            return e.getLocalizedMessage();
        }

        return e.getLocalizedMessage();
	}

	public void checkLoggedIn()
		throws IOException, BGException
	{
		logger.info( "checkLoggedIn 1" );

		FacesContext context = FacesContext.getCurrentInstance();
		HttpServletRequest request = (HttpServletRequest)context.getExternalContext().getRequest();
		if ( CommonUtils.isCustomer() )
		{
			context.getExternalContext().redirect( "index.xhtml" );
			return;
		}

		logger.info( "checkLoggedIn: authToken = " + request.getParameter( "authToken" ) );
		
		String token = request.getParameter( "authToken" );
		if( ru.bitel.common.Utils.notBlankString( token ) )
		{
			int contractId = application.getContractIdByToken( token );
			if ( contractId > 0 )
			{
				MySessionBean wsClientBean = context.getApplication().evaluateExpressionGet( context, "#{mySessionBean}", MySessionBean.class );
				ContractService contractService = wsClientBean.getPort( ContractService.class, 0 );
				Contract contract = contractService.contractGet( contractId );
				if( contract != null )
				{
					setUsername( contract.getTitle() );
					setPassword( contract.getPassword() );
					login();

					context.getExternalContext().redirect( "index.xhtml" );
				}

				return;
			}
		}

		requestLogin( request );
	}

	private void requestLogin( HttpServletRequest request )
		throws IOException
	{
	    String username = request.getParameter( "username" );
		String password = request.getParameter( "password" );
		String modeString0 = request.getParameter( "authMode" );
		
        logger.info( "requestLogin: username = " + username + "; password = " + password + "; modeString0 = " + modeString0 );
		
		if( modeString0 == null )
		{
			modeString0 = request.getParameter( "mode" );
		}

		final String modeString = modeString0;

        logger.info( "modeString = " + modeString );
		
		if( Utils.isBlankString( modeString ) && (Utils.isBlankString( username ) && Utils.isBlankString( password )) )
		{
			return;
		}

		AuthenticationMode authenticationMode;

		int modeId = Utils.parseInt( modeString, -1 );
        logger.info( "modeId = " + modeId );
		if( modeId >= 0 && configuration.getAuthentication().getModes().size() > modeId )
		{
			authenticationMode = configuration.getAuthentication().getModes().get( modeId );
		}
		else if( Utils.notBlankString( modeString ) )
		{
			authenticationMode = configuration.getAuthentication().getModes().stream().filter( a -> modeString.equals( a.getMode() ) ).findFirst().orElse( null );
		}
		else
		{
			authenticationMode = null;
		}

		if( authenticationMode == null )
		{
			authenticationMode = configuration.getAuthentication().getModes().get( 0 );
		}

        logger.info( "authenticationMode = " + authenticationMode + "; authenticationMode.getMode() = " + authenticationMode.getMode() );
		
		if( "ip".equals( authenticationMode.getMode() ) || (Utils.notBlankString( username ) && Utils.notBlankString( password )) )
		{
			if( "ip".equals( authenticationMode.getMode() ) )
			{
				preprocessAuthentication();
			}

			this.setUsername( username );
			this.setPassword( password );
			this.setAuthenticationMode( authenticationMode );

			String page = login();
			
	        logger.info( "page = " + page );

			ExternalContext ec = FacesContext.getCurrentInstance().getExternalContext();
			if( page != null && !ec.isResponseCommitted() )
			{
				ec.redirect( page.substring( 1 ) );
			}
		}
	}

	public void logout()
	{
		FacesContext context = FacesContext.getCurrentInstance();
		HttpServletRequest request = (HttpServletRequest)context.getExternalContext().getRequest();
		try
		{
			request.logout();
		}
		catch( ServletException e )
		{
			// ...
			context.addMessage( null, new FacesMessage( "Logout failed." ) );
		}
	}

	public String passwordReset()
		throws BGException
	{
		FacesContext context = FacesContext.getCurrentInstance();
		HttpServletRequest request = (HttpServletRequest)context.getExternalContext().getRequest();

		if( ru.bitel.common.Utils.isBlankString( username ) )
		{
			Utils.addErrorMessage( context, "passwordResetForm:passwordResetUsername", "input.required" );
			return "/login.xhtml";
		}

		if( ru.bitel.common.Utils.isBlankString( email ) )
		{
			Utils.addErrorMessage( context, "passwordResetForm:passwordResetEmail", "input.required" );
			return "/login.xhtml";
		}

		if( ru.bitel.common.Utils.isBlankString( captcha ) )
		{
			Utils.addErrorMessage( context, "passwordResetForm:passwordResetCaptcha", "login.captchaEmpty", "Введите символы с картинки" );
			return "/login.xhtml";
		}

		HttpSession session = request.getSession( true );
		String captchaValue = (String)session.getAttribute( "passwordReset.captcha.value" );
		if( !captcha.equalsIgnoreCase( captchaValue ) )
		{
			this.captcha = "";
			Utils.addErrorMessage( context, "passwordResetForm:passwordResetCaptcha", "login.captchaNotValid", "Введите символы с картинки" );
			return "/login.xhtml";
		}

		session.removeAttribute( "passwordReset.captcha.value" );

		this.captcha = "";

		MySessionBean wsClientBean = context.getApplication().evaluateExpressionGet( context, "#{mySessionBean}", MySessionBean.class );
		ContractService contractService = wsClientBean.getPort( ContractService.class, 0 );

		Contract contract = findContract( contractService, this.username );

		if( contract == null )
		{
			Utils.addErrorMessage( context, "passwordResetForm:passwordResetUsername", "passwordReset.notValid" );
			Utils.addErrorMessage( context, "passwordResetForm:passwordResetEmail", "passwordReset.notValid" );
			return "/login.xhtml";
		}

		try
		{
			contractService.contractPasswordReset( contract.getId(), this.email );
		}
		catch( BGException ex )
		{
			if( ex.getTag() != null )
			{
				switch( ex.getTag() )
				{
					case "passwordReset.email.notFound":
					case "passwordReset.email.notValid":
					case "passwordReset.webAuthModes.notValid":
					{
						Utils.addErrorMessage( context, "passwordResetForm:passwordResetUsername", "passwordReset.notValid" );
						Utils.addErrorMessage( context, "passwordResetForm:passwordResetEmail", "passwordReset.notValid" );
						return "/login.xhtml";
					}

					default:
						break;
				}
			}

			throw ex;
		}

		Utils.addInfoMessage( context, null, "passwordReset.ok" );

		return "/login.xhtml";
	}
	
	/**
	 * используется в *.xhtml файлах для постороение полного пути в url (css, js, img, ...) 
	 * @return
	 */
	public String getAppContext()
	{
	    if ( appContext == null )
	    {
	        appContext = ((HttpServletRequest)FacesContext.getCurrentInstance().getExternalContext().getRequest()).getContextPath();
	    }
	    return appContext;
	}

	private Contract findContract( ContractService contractService, String title )
		throws BGException
	{
		List<Contract> contractList = contractService.contractList( "^" + username + "$", null, -1, 0, null, true, false, false, new Holder<Page>( new Page( 1, 10 ) ) );
		if( contractList != null && contractList.size() > 0 )
		{
			for( Contract contract : contractList )
			{
				if( username.equals( contract.getTitle() ) )
				{
					logger.info( "Contract found: " + contract );

					return contract;
				}
			}
		}

		return null;
	}
	
    private static final class WSConnection
    {
        private long lm = 0L;

        private String url = "http://127.0.0.1:8080/bgbilling/executer";
        private String user = "admin";
        private String password = "admin";

        private Pool<ContractService> pool;
    }
}
