/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.ws;

import java.io.IOException;
import java.io.Serializable;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Supplier;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.enterprise.context.SessionScoped;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.xml.ws.Holder;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.common.BGIllegalAccessException;
import ru.bitel.bgbilling.common.BGIllegalArgumentException;
import ru.bitel.bgbilling.common.BGRuntimeException;
import ru.bitel.bgbilling.kernel.contract.api.common.bean.Contract;
import ru.bitel.bgbilling.kernel.contract.api.common.service.ContractService;
import ru.bitel.bgbilling.kernel.module.common.service.ModuleService;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Lazy;
import ru.bitel.common.model.Page;
import ru.bitel.mybgbilling.kernel.common.Configuration;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.utils.CommonUtils;
import ru.bitel.mybgbilling.kernel.contract.ParameterBean;
import ru.bitel.mybgbilling.kernel.realm.LoginBean;

@Named
@SessionScoped
@BGInjection
public class CustomerBean
	implements Serializable
{
	private static final long serialVersionUID = -2055383156493331443L;

	private static final Logger logger = LogManager.getLogger();

	@Inject
	private Configuration configuration;

	@Inject
	private ParameterBean parameterBean;

	@BGInject( module = false )
	private ContractService contractService;

	@BGInject( module = false )
	private ModuleService moduleService;

	/**
	 * Больше 0, только если произошел переход от супердоговора к субдоговору.
	 */
	private int superContractId;
	
	private boolean passwordOnce;

	private String name;

	private Supplier<Contract> contract;
	private Supplier<AtomicInteger> contractId;

	private Async<Set<Integer>> contractModuleIds;

	/**
	 * Субдоговора, даже если переключились с супердоговора
	 */
	private Lazy<List<Contract>> subContractList;

	private Lazy<String> customerTitle;

	public CustomerBean()
	{}

	public int getSuperContractId()
	{
		return superContractId;
	}
	
	public boolean isPasswordOnce()
	{
		return passwordOnce;
	}
	
	public void setPasswordOnce( boolean passwordOnce )
	{
		this.passwordOnce = passwordOnce;
	}

	@PostConstruct
	void init()
	{
		logger.info( "CustomerBean.init" );

		//super.init();

//		Principal principal = FacesContext.getCurrentInstance().getExternalContext().getUserPrincipal();
//
//		if( principal == null )
//		{
//			logger.error( "CustomerBean used before user authenticated! Check CDI dependencies and injects." );
//		}
//
//		this.name = principal.getName();
//
//		logger.info( "name=" + name );

//		Contract contract;
//
//		if( principal instanceof ContractPrincipal )
//		{
//			contract = ((ContractPrincipal)principal).getContract();
//		}
//		else
//		{
//			contract = null;
//		}

		FacesContext context = FacesContext.getCurrentInstance();
		HttpSession session = (HttpSession)context.getExternalContext().getSession( true );

		Integer superContractId = (Integer)session.getAttribute( "mybgbilling.auth.superContractId" );
		if( superContractId != null && superContractId > 0 )
		{
			this.superContractId = superContractId;
			session.removeAttribute( "mybgbilling.auth.superContractId" );
		}

		/*if( contract != null )
		{
			this.contract = () -> contract;
			this.contractId = () -> new AtomicInteger( contract.getId() );
		
			this.contractModuleIds = Async.of( () -> Optional.ofNullable( moduleService.contractModuleIds( this.contractId.get().get() ) ).orElse( Collections.emptySet() ) );
		
			this.subContractList = Lazy.of( () -> contractService.contractSubList( this.contractId.get().get(), -1, false ) );
		}
		else
		{
			populate();
		}*/

	    final Contract contract = (Contract)session.getAttribute( "mybgbilling.contract" );
		
		if( contract != null )
		{
		    this.name = contract.getTitle();
			this.contract = () -> contract;
			this.contractId = () -> new AtomicInteger( contract.getId() );
		}

		populate();
	}

	@PreDestroy
	void destroy()
	{}

	public void populate()
	{
		Contract oldContract = null;
		try
		{
			oldContract = this.contract != null ? this.contract.get() : null;
		}
		catch( BGRuntimeException ex )
		{
			logger.error( ex.getMessage(), ex );
		}

		final Contract _oldContract = oldContract;
		this.contract = Async.of( () -> getContract( _oldContract ) );
		this.contractId = Async.of( () -> new AtomicInteger( contract.get().getId() ) );
		this.contractModuleIds = Async.of( () -> Optional.ofNullable( moduleService.contractModuleIds( this.contractId.get().get() ) ).orElse( Collections.emptySet() ) );
		this.subContractList = Lazy.of( () -> contractService.contractSubList( this.superContractId > 0 ? this.superContractId : this.contractId.get().get(), -1, false ) );
		this.customerTitle = Lazy.of( () -> (String)configuration.get( "content.kernel.customerTitle",
																	   () -> new Object[] { contract.get(), parameterBean.getFullContractParameterMap() }, null ) );
	}

	private Contract getContract( Contract oldContract )
	{
		try
		{
			if( oldContract != null )
			{
				Contract contract = contractService.contractGet( oldContract.getId() );
				if( contract != null )
				{
					contract.setPassword( "" );
					return contract;
				}
				else
				{
					return oldContract;
				}
			}
			else
			{
				Holder<Page> pageHolder = new Holder<Page>();
				pageHolder.value = new Page( 1, 1 );

				List<Contract> contractList = contractService.contractList( "^" + name + "$", null, -1, 0L, null, true, false, true, pageHolder );
				if( contractList != null && contractList.size() > 0 )
				{
					for( Contract contract : contractList )
					{
						if( name.equals( contract.getTitle() ) )
						{
							logger.info( "Contract : " + contract );

							contract.setPassword( "" );

							return contract;
						}
					}
				}

				return null;
			}
		}
		catch( BGException e )
		{
			logger.error( e.getMessage() );
		}

		return oldContract;
	}

	public Contract getContract()
		throws BGException
	{
		return contract.get();
	}

	public int getContractId()
		throws BGException
	{
		return getContract().getId();
	}

	public Set<Integer> getContractModuleIds()
	{
		return contractModuleIds != null ? contractModuleIds.get() : null;
	}

	public AtomicInteger getContractIdRef()
		throws BGException
	{
		return contractId.get();
	}

	public void logout()
		throws IOException
	{
		final FacesContext context = FacesContext.getCurrentInstance();
		final ExternalContext ec = context.getExternalContext();

		try
		{
			((HttpServletRequest)ec.getRequest()).logout();
		}
		catch( ServletException e )
		{
			logger.error( e.getMessage(), e );
		}

		ec.invalidateSession();

		ec.redirect( "index.xhtml" );
	}

	public List<Contract> getSubContractList()
		throws BGException
	{
		return Utils.maskNull( subContractList.get() );
	}

	public String getCustomerTitle()
	{
		return this.customerTitle.get();
	}

	public synchronized void goToSubContract( int contractId )
		throws BGException, IOException
	{
		if( !CommonUtils.isCustomer() )
		{
			throw new BGIllegalAccessException();
		}

		logger.info( "goToSubContract" );

		populate();

		final int superContractId = this.superContractId > 0 ? this.superContractId : this.contract.get().getId();

		Contract subContract = null;

		for( Contract contract : getSubContractList() )
		{
			if( contract.getId() == contractId )
			{
				subContract = contract;
				break;
			}
		}

		if( subContract == null )
		{
			throw new BGIllegalArgumentException();
		}

		/*String token = application.generateToken( subContract.getId() );
		
		logger.info( "aaa1111" );
		
		FacesContext.getCurrentInstance().getExternalContext().invalidateSession();
		
		FacesContext fc = FacesContext.getCurrentInstance();
		ExternalContext ec = fc.getExternalContext();
		
		logger.info( "aaa2222" );
		
		ec.redirect( "login.xhtml?authToken=" + token );
		
		logger.info( "aaa3333" );*/

		//	fc.respo

		//logout();

		FacesContext context = FacesContext.getCurrentInstance();

		try
		{
			((HttpServletRequest)context.getExternalContext().getRequest()).logout();
		}
		catch( ServletException e )
		{
			logger.error( e.getMessage(), e );
		}

		context.getExternalContext().invalidateSession();

		//((HttpSession)context.getExternalContext().getSession( true )).invalidate();

		context = FacesContext.getCurrentInstance();
		HttpSession session = (HttpSession)context.getExternalContext().getSession( true );

		LoginBean loginBean = context.getApplication().evaluateExpressionGet( context, "#{loginBean}", LoginBean.class );
		loginBean.setUsername( subContract.getTitle() );
		loginBean.setPassword( subContract.getPassword() );
		//TODO: договора без паролей - запретить вход, но разрешить переход с супер
		try
		{
			loginBean.loginInternal();

			session.setAttribute( "mybgbilling.auth.superContractId", superContractId );
		}
		catch( IOException e )
		{
			throw new BGException( e );
		}
		finally
		{
			try
			{
				ExternalContext ec = context.getExternalContext();
				ec.redirect( "index.xhtml" );
			}
			catch( IOException e )
			{
				logger.error( e.getMessage(), e );
			}
		}
	}

	public synchronized void goToSuperContract()
		throws BGException, IOException
	{
		if( !CommonUtils.isCustomer() )
		{
			throw new BGIllegalAccessException();
		}

		logger.info( "goToSuperContract" );

		populate();

		if( this.superContractId <= 0 )
		{
			throw new BGIllegalArgumentException();
		}

		if( this.contract.get().getSuperCid() != this.superContractId )
		{
			throw new BGIllegalArgumentException();
		}

		final int superContractId = this.superContractId;

		Contract superContract = contractService.contractGet( superContractId );

		if( superContract == null )
		{
			throw new BGException( "Contract not found", "contract.not.found" );
		}

		FacesContext context = FacesContext.getCurrentInstance();

		try
		{
			((HttpServletRequest)context.getExternalContext().getRequest()).logout();
		}
		catch( ServletException e )
		{
			logger.error( e.getMessage(), e );
		}

		context.getExternalContext().invalidateSession();

		this.superContractId = 0;

		//((HttpSession)context.getExternalContext().getSession( true )).invalidate();

		context = FacesContext.getCurrentInstance();
		//HttpSession session = (HttpSession)context.getExternalContext().getSession( true );

		LoginBean loginBean = context.getApplication().evaluateExpressionGet( context, "#{loginBean}", LoginBean.class );
		loginBean.setUsername( superContract.getTitle() );
		loginBean.setPassword( superContract.getPassword() );

		try
		{
			loginBean.loginInternal();
		}
		catch( IOException e )
		{
			throw new BGException( e );
		}
		finally
		{
			try
			{
				ExternalContext ec = context.getExternalContext();
				ec.redirect( "index.xhtml" );
			}
			catch( IOException e )
			{
				logger.error( e.getMessage(), e );
			}
		}
	}

	public synchronized void relogin()
		throws BGException, IOException
	{
		logger.info( "relogin" );

		populate();

		if( this.contract.get() == null )
		{
			throw new BGIllegalArgumentException();
		}

		Contract contract = contractService.contractGet( getContractId() );

		if( contract == null )
		{
			throw new BGException( "Contract not found", "contract.not.found" );
		}

		FacesContext context = FacesContext.getCurrentInstance();

		try
		{
			((HttpServletRequest)context.getExternalContext().getRequest()).logout();
		}
		catch( ServletException e )
		{
			logger.error( e.getMessage(), e );
		}

		context.getExternalContext().invalidateSession();

		context = FacesContext.getCurrentInstance();

		LoginBean loginBean = context.getApplication().evaluateExpressionGet( context, "#{loginBean}", LoginBean.class );
		loginBean.setUsername( contract.getTitle() );
		loginBean.setPassword( contract.getPassword() );

		try
		{
			loginBean.loginInternal();
		}
		catch( IOException e )
		{
			throw new BGException( e );
		}
		finally
		{
			try
			{
				ExternalContext ec = context.getExternalContext();
				ec.redirect( "index.xhtml" );
			}
			catch( IOException e )
			{
				logger.error( e.getMessage(), e );
			}
		}
	}
}
