/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.ws;

import java.lang.ref.WeakReference;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Supplier;

import ru.bitel.common.function.Async;

public class Pool<O>
{
	private final Supplier<O> supplier;

	private final int coreSize;

	private final AtomicInteger coreCount = new AtomicInteger();

	private final ConcurrentLinkedQueue<O> coreQueue = new ConcurrentLinkedQueue<O>();

	private volatile WeakReference<ConcurrentLinkedQueue<O>> queueRef;

	public Pool( final Supplier<O> supplier, final int coreSize )
	{
		this.supplier = supplier;
		this.coreSize = coreSize;
	}

	private ConcurrentLinkedQueue<O> getPoolQueue()
	{
		final WeakReference<ConcurrentLinkedQueue<O>> q = queueRef;
		ConcurrentLinkedQueue<O> d;

		if( q != null && (d = q.get()) != null )
		{
			return d;
		}

		return null;
	}

	private ConcurrentLinkedQueue<O> getOrCreatePoolQueue()
	{
		final WeakReference<ConcurrentLinkedQueue<O>> q = queueRef;
		ConcurrentLinkedQueue<O> d;

		if( q != null && (d = q.get()) != null )
		{
			return d;
		}

		d = new ConcurrentLinkedQueue<O>();
		queueRef = new WeakReference<ConcurrentLinkedQueue<O>>( d );
		return d;
	}

	public O take()
	{
		O t = coreQueue.poll();
		if( t != null )
		{
			coreCount.decrementAndGet();
			return t;
		}

		final ConcurrentLinkedQueue<O> queue = getPoolQueue();
		if( queue != null )
		{
			t = queue.poll();
			if( t != null )
			{
				return t;
			}
		}
		
		t = supplier.get();

		if( queue == null )
		{
			Async.of( ( ) -> recycle( supplier.get() ) );
		}

		return t;
	}

	public void recycle( final O t )
	{
		//XXX
		if( coreCount.get() < coreSize )
		{
			if( coreQueue.offer( t ) )
			{
				coreCount.incrementAndGet();
			}
		}
		else
		{
			getOrCreatePoolQueue().offer( t );
		}
	}
}
