/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.ws;

import java.io.Serializable;
import java.util.Collection;
import java.util.MissingResourceException;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Priority;
import javax.enterprise.context.Dependent;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.interceptor.AroundInvoke;
import javax.interceptor.Interceptor;
import javax.interceptor.InvocationContext;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGIllegalAccessException;
import ru.bitel.bgbilling.common.BGRuntimeException;
import ru.bitel.common.model.Page;
import ru.bitel.mybgbilling.kernel.common.MySessionBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.utils.Utils;
import ru.bitel.mybgbilling.kernel.navigation.NavigationBean;

@Interceptor
@Priority(Interceptor.Priority.APPLICATION)
@BGInjection
@Dependent
public class WSInjectInterceptor
	implements Serializable
{
	private static final long serialVersionUID = 2097214231907883260L;

	private static final Logger logger = LogManager.getLogger();

	@Inject
	private MySessionBean wsClientBean;

	@Inject
	private NavigationBean navigationBean;

	@Inject
	private WSInjectManager wsInjectManager;

	@AroundInvoke
	public Object aroundInvoke( InvocationContext ctx )
		throws Exception
	{
		//		logger.info( String.valueOf( ctx.getMethod() ) );
		//		logger.info( String.valueOf( ctx.getParameters() ) );
		//		logger.info( String.valueOf( ctx.getContextData() ) );

		try
		{
			return ctx.proceed();
		}
		catch( BGRuntimeException ex )
		{
			if( ex.getCause() instanceof BGIllegalAccessException )
			{
				logger.error( ex.getMessage() );

				Utils.addErrorMessage( FacesContext.getCurrentInstance(), null, ex.getCause().getMessage() );

				return null;
			}

			throw ex;
		}
        catch( BGIllegalAccessException ex )
        {
            if( FacesContext.getCurrentInstance() != null )
            {
                logger.error( ex.getMessage() );

                Utils.addErrorMessage( FacesContext.getCurrentInstance(), null, ru.bitel.common.Utils.maskBlank( ex.getMessage(), "Ошибка доступа" ) );

                return null;
            }

            logger.error( ex.getMessage(), ex );

            throw ex;
        }
		catch( Exception ex )
		{
			Throwable cause = ex;
			while( cause.getCause() != null && cause.getCause() != cause )
			{
				cause = cause.getCause();
			}

			if( cause instanceof java.net.ConnectException )
			{
				logger.error( ex.getMessage() );

				final FacesContext context = FacesContext.getCurrentInstance();

				if( !context.getMessageList().stream().filter( a -> "Connection error".equals( a.getSummary() ) ).findAny().isPresent() )
				{
					String message = "В соединении отказано";

					try
					{
						message = context.getApplication().getResourceBundle( context, "msg" ).getString( message );
					}
					catch( MissingResourceException e )
					{}

					FacesMessage faceMessage = new FacesMessage( FacesMessage.SEVERITY_ERROR, "Connection error", message );
					context.addMessage( null, faceMessage );
				}

				if( Collection.class.isAssignableFrom( ctx.getMethod().getReturnType() ) )
				{
					return null;
				}
				else if( Page.class.isAssignableFrom( ctx.getMethod().getReturnType() ) )
				{
					return new Page( 0, 0 );
				}

				return null;
			}

			logger.debug( ex.getMessage(), ex );
			throw ex;
		}
	}

	@PostConstruct
	void initialize( InvocationContext ctx )
		throws Exception
	{
		logger.debug( "initialize " + ctx.getTarget() );

		// ctx.getContextData().put( "moduleId", navigationBean.getModuleId() );

		wsInjectManager.process( wsClientBean, navigationBean, ctx, ctx.getTarget() );

		ctx.proceed();
	}

	@PreDestroy
	void cleanup( InvocationContext ctx )
		throws Exception
	{
		logger.debug( "cleanup" );

		wsInjectManager.cleanup( wsClientBean, navigationBean, ctx, ctx.getTarget() );

		ctx.proceed();
	}
}
