/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.kernel.ws;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.concurrent.TimeUnit;
import java.util.function.Supplier;

import javax.enterprise.context.ApplicationScoped;
import javax.faces.context.FacesContext;
import javax.interceptor.InvocationContext;
import javax.jws.WebService;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.common.function.Expirable;
import ru.bitel.common.function.ThrowingSupplier;
import ru.bitel.mybgbilling.kernel.common.MySessionBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.InjectManager;
import ru.bitel.mybgbilling.kernel.common.inject.InjectPlanField;
import ru.bitel.mybgbilling.kernel.navigation.NavigationBean;

@ApplicationScoped
public class WSInjectManager
	extends InjectManager
	implements Serializable
{
	private static final Logger logger = LogManager.getLogger();

	public WSInjectManager()
	{
		super();

		logger.info( "WSInjectManager.<init> " );
	}

	@Override
	protected InjectPlanField buildInjection( final Object o, final Field f )
	{
		try
		{
			final boolean accessible = f.isAccessible();

			if( !accessible )
			{
				f.setAccessible( true );
			}

			if( f.get( o ) != null )
			{
				if( !accessible )
				{
					f.setAccessible( accessible );
				}

				return null;
			}

			final BGInject bgInject = f.getAnnotation( BGInject.class );

			if( bgInject == null )
			{
				if( !accessible )
				{
					f.setAccessible( accessible );
				}

				return null;
			}

			boolean supplier = false;
			Class<?> clazz = f.getType();

			if( Supplier.class.isAssignableFrom( clazz ) )
			{
				Type type = f.getGenericType();
				if( type instanceof ParameterizedType )
				{
					ParameterizedType pType = (ParameterizedType)type;
					clazz = (Class<?>)pType.getActualTypeArguments()[0];
					supplier = true;
				}
			}

			if( clazz.getAnnotation( WebService.class ) == null )
			{
				if( !accessible )
				{
					f.setAccessible( accessible );
				}

				return null;
			}

			logger.info( f.getName() );

			return new InjectPlanField( f, supplier )
			{
				private int moduleId = 0;

				@Override
				public void process( final MySessionBean wsClientBean, final NavigationBean navigationBean, final InvocationContext ctx, final Object o )
					throws IllegalArgumentException, IllegalAccessException
				{
					// final Integer moduleId = (Integer)ctx.getContextData().get( "moduleId" );

					if( bgInject.module() )
					{
						final Integer moduleId = navigationBean.getModuleId();

						this.moduleId = moduleId != null ? moduleId : 0;
					}
					else
					{
						this.moduleId = 0;
					}

					if( !supplier )
					{
						final Object port = wsClientBean.getPort( this.field.getType(), this.moduleId );
						this.field.set( o, port );
					}
					else
					{
						Supplier<Object> supplier;

						try
						{
							final Object port = wsClientBean.getPort( this.field.getType(), this.moduleId );
							supplier = () -> port;
						}
						catch( RuntimeException ex )
						{
							final ThrowingSupplier<Object> delegate = () -> wsClientBean.getPort( this.field.getType(), this.moduleId );

							final ThrowingSupplier<Object> nonNullSupplier = new ThrowingSupplier<Object>()
							{
								private Object result;

								@Override
								public synchronized Object getThrows()
									throws BGException
								{
									if( result != null )
									{
										return result;
									}
									else
									{
										return result = delegate.get();
									}
								}
							};

							supplier = Expirable.of( () -> null, nonNullSupplier, 31, TimeUnit.SECONDS );
						}

						this.field.set( o, supplier );
					}
				}

				@Override
				public void cleanup( MySessionBean wsClientBean, NavigationBean navigationBean, InvocationContext ctx, Object o )
					throws IllegalArgumentException, IllegalAccessException
				{
					if( FacesContext.getCurrentInstance() == null )
					{
						return;
					}

					final Object port = this.field.get( o );
					if( port != null )
					{
						wsClientBean.returnPort( this.field.getType(), moduleId, port );
					}
				}
			};
		}
		catch( Exception e )
		{
			logger.error( e.getMessage(), e );
		}

		return null;
	}
}
