/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.modules.bill;

import java.io.IOException;
import java.util.List;
import java.util.Optional;

import javax.activation.DataHandler;
import javax.inject.Named;
import javax.servlet.ServletException;
import javax.xml.ws.Holder;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.modules.bill.common.bean.BillDocument;
import ru.bitel.bgbilling.modules.bill.common.bean.InvoiceDocument;
import ru.bitel.bgbilling.modules.bill.common.service.BillService;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Lazy;
import ru.bitel.common.model.Page;
import ru.bitel.common.model.Result;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.scope.SessionModuleScoped;

@Named
@SessionModuleScoped
@BGInjection
public class BillBean
	extends AbstractBean
{
	@BGInject
	private BillService billService;

	private Async<Result<BillDocument>> billDocumentList;
	private Lazy<Page> billDocumentListPage = Lazy.of( () -> new Page( 1, 15 ) );

	private Async<Result<InvoiceDocument>> invoiceDocumentList;
	private Lazy<Page> invoiceDocumentListPage = Lazy.of( () -> new Page( 1, 15 ) );

	@Override
	protected void init()
		throws BGException
	{
		super.init();

		populate();
	}

	public void populate()
		throws BGException
	{
		switch( navigationBean.getSubPage() )
		{
			case "invoices":
				populateInvoices();
				break;

			default:
				populateBills();
				break;
		}
	}

	public void populateBills()
		throws BGException
	{
		final Page billDocumentPage = this.billDocumentListPage.get();

		final Async<Result<BillDocument>> result =
			this.billDocumentList = Async.of( () -> billService.billDocumentListForContract( getContractId(), billDocumentPage ) );

		this.billDocumentListPage = Lazy.of( () -> Optional.ofNullable( result ).map( Async::get ).map( Result::getPage ).orElseGet( () -> new Page( 1, 15 ) ) );
	}

	public void populateInvoices()
		throws BGException
	{
		final Page invoiceDocumentPage = this.invoiceDocumentListPage.get();

		final Async<Result<InvoiceDocument>> result =
			this.invoiceDocumentList = Async.of( () -> billService.invoiceDocumentListForContract( getContractId(), true, invoiceDocumentPage ) );

		this.invoiceDocumentListPage = Lazy.of( () -> Optional.ofNullable( result ).map( Async::get ).map( Result::getPage ).orElseGet( () -> new Page( 1, 15 ) ) );
	}

	public List<BillDocument> getBillDocumentList()
	{
		return Optional.ofNullable( billDocumentList ).map( Async::get ).map( Result::getList ).orElse( null );
	}

	public Page getBillDocumentListPage()
	{
		return billDocumentListPage.get();
	}

	public void setBillDocumentPage( Page billDocumentListPage )
	{
		this.billDocumentListPage = Lazy.of( () -> billDocumentListPage != null ? billDocumentListPage : new Page( 1, 15 ) );
	}

	public List<InvoiceDocument> getInvoiceDocumentList()
	{
		return Optional.ofNullable( invoiceDocumentList ).map( Async::get ).map( Result::getList ).orElse( null );
	}

	public Page getInvoiceDocumentListPage()
	{
		return invoiceDocumentListPage.get();
	}

	public void setInvoiceDocumentListPage( Page invoiceDocumentListPage )
	{
		this.invoiceDocumentListPage = Lazy.of( () -> invoiceDocumentListPage != null ? invoiceDocumentListPage : new Page( 1, 15 ) );
	}

	public void downloadBillDocument( int id )
		throws ServletException, IOException, BGException
	{
		downloadDocument( false, id );
	}

	public void downloadInvoiceDocument( int id )
		throws ServletException, IOException, BGException
	{
		downloadDocument( true, id );
	}

	private void downloadDocument( boolean invoice, int id )
		throws ServletException, IOException, BGException
	{
		final Holder<String> name = new Holder<String>();
		final Holder<DataHandler> data = new Holder<DataHandler>();

		if( invoice )
		{
			billService.invoiceDocumentDownload( getContractId(), id, name, data );
		}
		else
		{
			billService.billDocumentDownload( getContractId(), id, name, data );
		}

		//String titleCard = Base64.decode( Base64.decodeWeb( ge ), "utf-8" );
		
		String fileName = name.value != null ? name.value : data.value.getDataSource().getName();
		
		writeToOutput( data.value, fileName, false );
	}

	public void deleteBillDocument( int id )
		throws BGException
	{
		billService.billDocumentDelete( getContractId(), id, true );
	}
}
