/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.modules.cerbercrypt;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import javax.inject.Inject;
import javax.inject.Named;
import javax.xml.ws.Holder;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.modules.cerbercrypt.common.bean.CardPacket;
import ru.bitel.bgbilling.modules.cerbercrypt.common.bean.Packet;
import ru.bitel.bgbilling.modules.cerbercrypt.common.bean.UserCard;
import ru.bitel.bgbilling.modules.cerbercrypt.common.service.CardPacketService;
import ru.bitel.bgbilling.modules.cerbercrypt.common.service.PacketService;
import ru.bitel.bgbilling.modules.cerbercrypt.common.service.SubscriptionService;
import ru.bitel.bgbilling.modules.cerbercrypt.common.service.UserCardService;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Lazy;
import ru.bitel.common.model.Id;
import ru.bitel.common.model.MapHolder;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.scope.SessionModuleScoped;

@Named
@SessionModuleScoped
@BGInjection
public class CerbercryptBean
	extends AbstractBean
	implements Serializable
{
	private static final Logger logger = LogManager.getLogger();

	@Inject
	private CerbercryptActivateBean cerbercryptActivateBean;

	@BGInject
	private PacketService packetService;

	@BGInject
	private UserCardService userCardService;

	@BGInject
	private SubscriptionService subscriptionService;

	int moduleId;

	private Async<List<UserCard>> accountList;

	private int accountId;
	private UserCard account;

	private Supplier<BigDecimal> daySubscriptionCost;

	@BGInject
	private CardPacketService cardPacketService;

	private Async<List<CardPacket>> productList;
	private Lazy<List<CardPacket>> historyProductList;

	private Async<Map<Integer, Packet>> packetMap;

	private Supplier<List<Packet>> packetOfferingList;
	private Async<List<LocalDate>> activateDateFromList;
	private Async<List<LocalDate>> activateDateToList;

	@Override
	protected void init()
		throws BGException
	{
		super.init();

		this.moduleId = navigationBean.getModuleId();

		accountList = Async.of( () -> Optional.ofNullable( userCardService.getUserCardList( getContractId(), false ) ).orElse( Collections.emptyList() ) );

		packetMap = Async.of( () -> Utils.maskNull( packetService.packetList( false ) ).stream().collect( Collectors.toMap( Packet::getId, Function.identity() ) ) );

		//packetService.packetOfferingList( getContractId(), userCardId, virtualCinema, packetList, params );
	}

	public List<UserCard> getAccountList()
		throws BGException
	{
		return Optional.ofNullable( this.accountList ).map( Async::get ).orElse( Collections.emptyList() );
	}

	public int getAccountId()
	{
		return accountId;
	}

	public void setAccountId( final int accountId )
		throws BGException
	{
		if( this.accountId != accountId )
		{
			cerbercryptActivateBean.setStep( 0 );
		}

		this.accountId = accountId;

		this.account = null;

		for( UserCard tvAccount : getAccountList() )
		{
			if( tvAccount.getId() == accountId )
			{
				this.account = tvAccount;
				break;
			}
		}

		if( account == null )
		{
			this.accountId = 0;

			activateDateFromList = Async.completed( Collections.emptyList() );
			activateDateToList = Async.completed( Collections.emptyList() );
			packetOfferingList = () -> Collections.emptyList();
		}
		else
		{
			activateDateFromList = Async.uncompleted();
			activateDateToList = Async.uncompleted();

			packetOfferingList = Async.of( () -> {

				try
				{
					Holder<List<Packet>> packetList = new Holder<>();
					Holder<MapHolder<String, Object>> params = new Holder<>();

					packetService.packetOfferingList( getContractId(), accountId, false, packetList, params );

					Map<String, Object> paramsMap = params.value != null ? params.value.getMap() : Collections.emptyMap();

					List<Date> dateListOpen = (List<Date>)paramsMap.get( "date_list_open" );
					List<Date> dateListClose = (List<Date>)paramsMap.get( "date_list_close" );

					if( dateListOpen != null )
					{
						activateDateFromList.complete( dateListOpen.stream().map( ( a ) -> a.toInstant().atZone( ZoneId.systemDefault() ).toLocalDate() )
																   .collect( Collectors.toList() ) );
					}

					if( dateListClose != null )
					{
						activateDateToList.complete( dateListClose.stream().map( ( a ) -> a.toInstant().atZone( ZoneId.systemDefault() ).toLocalDate() )
																  .collect( Collectors.toList() ) );
					}

					return packetList.value;
				}
				finally
				{
					activateDateFromList.complete( null );
					activateDateToList.complete( null );
				}
			} );
		}
	}

	public UserCard getAccount()
	{
		return account;
	}

	public BigDecimal getDaySubscriptionCost()
	{
		return daySubscriptionCost.get();
	}

	public List<CardPacket> getProductList()
	{
		return productList != null ? productList.get() : null;
	}

	public List<CardPacket> getHistoryProductList()
	{
		return historyProductList != null ? historyProductList.get() : null;
	}

	public Packet getPacket( int id )
	{
		return packetMap.get().get( id );
	}

	public List<LocalDate> getDateListOpen()
	{
		return activateDateFromList.get();
	}

	public List<LocalDate> getDateListClose()
	{
		return activateDateToList.get();
	}

	public List<Packet> getPacketOfferingList()
	{
		return packetOfferingList.get();
	}

	public void populate()
		throws BGException
	{
		final Calendar calendar = new GregorianCalendar();
		//calendar.add( Calendar.MINUTE, -10 );

		final int accountId = this.accountId;

		if( accountId > 0 )
		{
			//productList = Async.of( () -> Utils.maskNull( fillProductSpecName( productService.productList( moduleId, getContractId(), tvAccountId, false, null, null, calendar.getTime(), null, false, true ) ) ) );
			//historyProductList = Lazy.of( () -> Utils.maskNull( fillProductSpecName( productService.productList( moduleId, getContractId(), tvAccountId, false, null, new Date(), null, null, false, false ) ) ) );

			productList = Async.of( () -> {

				List<CardPacket> result = cardPacketService.cardPacketTable( -1, accountId, calendar.getTime(), false, getContractId() );
				if( result == null )
				{
					result = new ArrayList<>();
				}

				result.addAll( Utils.maskNull( cardPacketService.cardPacketFutureList( getContractId(), accountId, calendar.getTime() ) ) );

				return result;
			} );

			historyProductList = Lazy.of( () -> {
				List<CardPacket> result = Utils.maskNull( cardPacketService.cardPacketTable( -1, accountId, null, false, getContractId() ) );

				List<CardPacket> active = productList.get();
				Set<Integer> ids = Id.newIdSet( active );

				return result.stream().filter( a -> !ids.contains( a.getId() ) ).collect( Collectors.toList() );
			} );
			
			daySubscriptionCost = Async.of( () -> subscriptionService.subscriptionCost( getContractId(), accountId, new Date() ) );
		}
		else
		{
			productList = Async.of( () -> Collections.emptyList() );
			historyProductList = Lazy.of( () -> Collections.emptyList() );
			
			daySubscriptionCost = () -> BigDecimal.ZERO;
		}
	}

	public void activate( int packetId, LocalDate dateFrom )
		throws BGException
	{
		String message =
			subscriptionService.subscriptionOpen( getContractId(), accountId, packetId, Date.from( dateFrom.atStartOfDay( ZoneId.systemDefault() ).toInstant() ), null );

		if( Utils.notBlankString( message ) )
		{
			addInfoMessage( null, message );
		}

		populate();
	}

	public void deactivate( int cardPacketId, LocalDate date )
		throws BGException
	{
		for( CardPacket cardPacket : getProductList() )
		{
			if( cardPacket.getId() == cardPacketId )
			{
				if( cardPacket.getDateFrom().toInstant().atZone( ZoneId.systemDefault() ).toLocalDate().isAfter( LocalDate.now() ) )
				{
					subscriptionService.subscriptionDelete( getContractId(), cardPacket.getUsercardId(), cardPacket.getId() );
				}
				else
				{
					if( date == null )
					{
						addErrorMessage( null, "cerbercrypt.subscription.close.needDate" );
						return;
					}

					subscriptionService.subscriptionClose( getContractId(), cardPacket.getUsercardId(), cardPacket.getPacketId(),
														   Date.from( date.atStartOfDay( ZoneId.systemDefault() ).toInstant() ), null );
				}
			}
		}

		populate();
	}
}
