package ru.bitel.mybgbilling.modules.inet;

import java.io.Serializable;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

import javax.enterprise.context.SessionScoped;
import javax.faces.context.FacesContext;
import javax.inject.Inject;
import javax.inject.Named;
import javax.security.auth.login.AccountNotFoundException;
import javax.security.auth.login.CredentialException;
import javax.security.auth.login.FailedLoginException;
import javax.security.auth.login.LoginException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.contract.api.common.bean.Contract;
import ru.bitel.bgbilling.kernel.contract.api.common.service.ContractService;
import ru.bitel.bgbilling.modules.inet.common.service.InetServService;
import ru.bitel.bgbilling.modules.inet.common.service.InetSessionService;
import ru.bitel.bgbilling.modules.inet.common.bean.InetServ;
import ru.bitel.bgbilling.modules.inet.common.bean.InetSessionLog;
import ru.bitel.common.TimeUtils;
import ru.bitel.common.Utils;
import ru.bitel.common.inet.IpAddress;
import ru.bitel.common.model.Page;
import ru.bitel.common.model.Result;
import ru.bitel.mybgbilling.kernel.common.Configuration.AuthenticationMode;
import ru.bitel.mybgbilling.kernel.common.MyApplicationBean;
import ru.bitel.mybgbilling.kernel.common.MySessionBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.realm.AuthenticationBean;
import ru.bitel.mybgbilling.kernel.realm.LoginBean;
import ru.bitel.mybgbilling.kernel.realm.MyFailedLoginException;

@Named
@SessionScoped
@BGInjection
public class InetAuthenticationBean
	implements AuthenticationBean, Serializable
{
	private static final Logger logger = LogManager.getLogger();

	@Inject
	private MyApplicationBean application;

	@Inject
	private LoginBean loginBean;

	private long findMillis = -1L;

	private int contractId = -1;
	private String contractTitle;

	@Override
	public void login( final HttpServletRequest request, final AuthenticationMode mode, final int moduleId )
		throws BGException, LoginException, ServletException
	{
		switch( mode.getMode() )
		{
			case "ip":
				loginByIp( request, mode, moduleId );
				break;

			case "login":
			default:
				loginByLogin( request, mode, moduleId );
				break;
		}
	}

	private void loginByLogin( final HttpServletRequest request, final AuthenticationMode mode, final int moduleId )
		throws BGException, LoginException, ServletException
	{
		final String username = loginBean.getUsername();
		final String password = loginBean.getPassword();
		
		logger.info( "authentication by Inet login " + username );

		final Date now = new Date();

		final FacesContext context = FacesContext.getCurrentInstance();

		MySessionBean wsClientBean = MySessionBean.get( context );
		InetServService inetServService = wsClientBean.getPort( InetServService.class, moduleId );

		List<InetServ> inetServList = Utils.maskNull( inetServService.searchInetServ( username, null, 0, -1, null, null, null, true ) );
		InetServ inetServ = null;

		boolean authenticated = false;

		for( InetServ s : inetServList )
		{
			if( !TimeUtils.dateInRange( now, s.getDateFrom(), s.getDateTo() ) )
			{
				continue;
			}

			if( !username.equalsIgnoreCase( s.getLogin() ) )
			{
				continue;
			}

			inetServ = s;

			if( Utils.isBlankString( s.getPassword() ) || !password.equals( s.getPassword() ) )
			{
				continue;
			}

			authenticated = true;
			break;
		}

		if( inetServ == null )
		{
			throw new AccountNotFoundException( "Login not found" );
		}

		if( !authenticated )
		{
			Exception ex = new CredentialException( "Login password incorrect" );
			throw new MyFailedLoginException( null, inetServ.getContractId(), ex.getMessage(), ex );
		}

		ContractService contractService = wsClientBean.getPort( ContractService.class, 0 );
		Contract contract = contractService.contractGet( inetServ.getContractId() );

		if( mode.getFilter() != null )
		{
			if( Boolean.FALSE.equals( mode.getFilter().call( contract ) ) )
			{
				contract = null;
			}
		}

		if( contract == null )
		{
			throw new FailedLoginException( "Contract not found" );
		}

        request.getSession().setAttribute( "mybgbilling.contract", contract );

//		if( "unauthCustomer".equals( mode.getRole() ) )
//		{
//			request.login( "[unauthCustomer]" + contract.getTitle(), contract.getPassword() );
//		}
//		else
//		{
//			request.login( contract.getTitle(), contract.getPassword() );
//		}
	}

	@Override
	public void preprocess( HttpServletRequest request, AuthenticationMode mode, int moduleId, LoginBean loginBean )
	{
		try
		{
			switch( mode.getMode() )
			{
				case "ip":
					preprocessIp( request, mode, moduleId, loginBean );
					break;

				case "login":
				default:
					break;
			}
		}
		catch( BGException | UnknownHostException ex )
		{
			logger.error( ex.getMessage(), ex );
		}
	}

	private void preprocessIp( HttpServletRequest request, AuthenticationMode mode, int moduleId, LoginBean loginBean )
		throws BGException, UnknownHostException
	{
		final FacesContext context = FacesContext.getCurrentInstance();

		if( contractId == 0 && (System.currentTimeMillis() - this.findMillis) < TimeUnit.MINUTES.toMillis( 2 ) )
		{
			loginBean.setUsername( this.contractTitle );
			loginBean.setPassword( "" );

			ru.bitel.mybgbilling.kernel.common.utils.Utils.addErrorMessage( context, null, "login.autoAuthError", "Договор не найден" );
			//ru.bitel.mybgbilling.kernel.common.utils.Utils.addErrorMessage( context, "loginForm:username", "login.usernameNotFound", "Договор не найден" );

			return;
		}
		else if( contractId > 0 && (System.currentTimeMillis() - this.findMillis) < TimeUnit.MINUTES.toMillis( 4 ) )
		{
			loginBean.setUsername( this.contractTitle );
			loginBean.setPassword( "********" );

			return;
		}

		final String ip = application.getRemoteAddr( request );

		logger.info( "authentication by Inet IP-address " + ip );

		final MySessionBean wsClientBean = MySessionBean.get( context );

		final InetSessionLog log = findSession( wsClientBean, moduleId, ip );
		if( log == null )
		{
			this.contractId = 0;
			this.contractTitle = "";

			loginBean.setUsername( this.contractTitle );
			loginBean.setPassword( "" );

			ru.bitel.mybgbilling.kernel.common.utils.Utils.addErrorMessage( context, null, "login.autoAuthError", "Договор не найден" );
			//ru.bitel.mybgbilling.kernel.common.utils.Utils.addErrorMessage( context, "loginForm:username", "login.usernameNotFound", "Договор не найден" );

			return;
		}

		ContractService contractService = wsClientBean.getPort( ContractService.class, 0 );
		Contract contract = contractService.contractGet( log.getContractId() );

		if( mode.getFilter() != null )
		{
			if( Boolean.FALSE.equals( mode.getFilter().call( contract ) ) )
			{
				contract = null;
			}
		}

		if( contract == null )
		{
			this.contractId = 0;
			this.contractTitle = "";

			loginBean.setUsername( this.contractTitle );
			loginBean.setPassword( "" );
			
			this.findMillis = System.currentTimeMillis();

			ru.bitel.mybgbilling.kernel.common.utils.Utils.addErrorMessage( context, null, "login.autoAuthError", "Договор не найден" );
			//ru.bitel.mybgbilling.kernel.common.utils.Utils.addErrorMessage( context, "loginForm:username", "login.usernameNotFound", "Договор не найден" );

			return;
		}

		this.contractId = contract.getId();
		this.contractTitle = contract.getTitle();

		this.findMillis = System.currentTimeMillis();

		loginBean.setUsername( this.contractTitle );
		loginBean.setPassword( "********" );
	}

	private InetSessionLog findSession( final MySessionBean wsClientBean, int moduleId, String ip )
		throws UnknownHostException, BGException
	{
		InetSessionService inetSessionService = wsClientBean.getPort( InetSessionService.class, moduleId );

		Result<InetSessionLog> list = inetSessionService.inetSessionAliveList( null, null, null, null, ip, null, null, null, new Page( 1, 25 ) );

		if( list == null || list.getList() == null )
		{
			return null;
		}

		byte[] address = InetAddress.getByName( ip ).getAddress();

		for( InetSessionLog log : list.getList() )
		{
			if( !IpAddress.equals( log.getInetAddressBytes(), address ) )
			{
				continue;
			}

			if( log.getStatus() != InetSessionLog.STATUS_ALIVE )
			{
				continue;
			}

			return log;
		}

		return null;
	}

	private void loginByIp( final HttpServletRequest request, final AuthenticationMode mode, final int moduleId )
		throws BGException, LoginException, ServletException
	{
		logger.info( "loginByIp" );

		if( this.contractId <= 0 )
		{
			throw new FailedLoginException( "Contract not found" );
		}

		final FacesContext context = FacesContext.getCurrentInstance();
		final MySessionBean wsClientBean = MySessionBean.get( context );

		ContractService contractService = wsClientBean.getPort( ContractService.class, 0 );
		Contract contract = contractService.contractGet( this.contractId );

		if( "unauthCustomer".equals( mode.getRole() ) )
		{
			request.login( "[unauthCustomer]" + contract.getTitle(), contract.getPassword() );
		}
		else
		{
			request.login( contract.getTitle(), contract.getPassword() );
		}

		this.contractId = -1;
		this.contractTitle = "";
		this.findMillis = -1L;
	}
}
