/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.modules.inet;

import java.io.Serializable;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.directory.api.common.bean.Directory;
import ru.bitel.bgbilling.modules.inet.common.bean.InetServ;
import ru.bitel.bgbilling.modules.inet.common.bean.InetServType;
import ru.bitel.bgbilling.modules.inet.common.bean.InetSessionLog;
import ru.bitel.bgbilling.modules.inet.common.bean.TrafficType;
import ru.bitel.bgbilling.modules.inet.common.bean.TrafficTypeLink;
import ru.bitel.bgbilling.modules.inet.common.bean.TrafficTypeLinkRule;
import ru.bitel.bgbilling.modules.inet.common.service.InetServService;
import ru.bitel.bgbilling.modules.inet.common.service.InetSessionService;
import ru.bitel.common.model.Id;
import ru.bitel.common.model.Page;
import ru.bitel.common.model.Result;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.scope.SessionModuleScoped;
import ru.bitel.mybgbilling.kernel.navigation.LocaleBean;
import ru.bitel.mybgbilling.kernel.navigation.NavigationBean;
import ru.bitel.mybgbilling.kernel.ws.CustomerBean;

@Named
@SessionModuleScoped
@BGInjection
public class InetBean
	implements Serializable
{
	private static final Logger logger = LogManager.getLogger();

	@Inject
	private LocaleBean localeBean;

	@Inject
	private CustomerBean customerBean;

	@Inject
	private NavigationBean navigationBean;

	@BGInject
	private InetServService inetServService;

	@BGInject
	private InetSessionService inetSessionService;

	@BGInject(type = InetServType.class)
	private Directory<InetServType> inetServTypeDirectory;

	@BGInject(type = TrafficTypeLink.class)
	private Directory<TrafficTypeLink> trafficTypeLinkDirectory;

	int moduleId;

	private List<TrafficType> trafficTypeList;
	private Map<Integer, TrafficType> trafficTypeMap;

	private List<InetServ> inetServList;

	private int inetServId;
	private InetServ inetServ;
	private List<TrafficType> inetServTrafficTypeList;

	@PostConstruct
	void init()
	{
		this.moduleId = navigationBean.getModuleId();

		logger.info( "init" );
	}

	@PreDestroy
	void destroy()
	{
		logger.info( "destroy" );
	}

	public int getInetServId()
	{
		return inetServId;
	}

	public void setInetServId( int inetServId )
		throws BGException
	{
		this.inetServId = inetServId;

		this.inetServTrafficTypeList = null;

		this.inetServ = null;

		for( InetServ inetServ : getInetServList() )
		{
			if( inetServ.getId() == inetServId )
			{
				this.inetServ = inetServ;
				break;
			}
		}

		if( this.inetServ == null )
		{
			this.inetServId = 0;
		}
	}

	public InetServ getInetServ()
	{
		return inetServ;
	}

	public List<InetServ> getInetServList()
		throws BGException
	{
		if( inetServList == null )
		{
			InetServ root = inetServService.inetServTree( customerBean.getContractId() );

			inetServList = new ArrayList<InetServ>();

			if( root != null && root.getChildren() != null )
			{
				MessageFormat fmt = new MessageFormat( "{0} [{1,date}-{2,date}]", localeBean.getLocale() );

				List<InetServ> list = new ArrayList<InetServ>( inetServList.size() );

				root.getChildren().stream().filter( a -> a.getDateTo() == null ).forEach( list::add );
				root.getChildren().stream().filter( a -> a.getDateTo() != null ).forEach( list::add );

				for( InetServ serv : list )
				{
					inetServList.add( serv );

					if( serv.getChildren() != null )
					{
						serv.getChildren().stream().forEach( a -> a.setTitle( "↳ " + a.getTitle() ) );

						serv.getChildren().stream().filter( a -> a.getDateTo() == null ).forEach( inetServList::add );
						serv.getChildren().stream().filter( a -> a.getDateTo() != null ).forEach( inetServList::add );
					}
				}

				inetServList.stream().filter( a -> a.getDateTo() != null ).forEach( a -> a.setTitle( fmt.format( new Object[] { a, a.getDateFrom(), a.getDateTo() } ) ) );
			}
		}

		return inetServList;
	}

	public void populate()
		throws BGException
	{
		inetServList = null;
		getInetServList();
	}

	public List<TrafficType> getTrafficTypeList()
		throws BGException
	{
		if( trafficTypeList == null )
		{
			trafficTypeList = inetServService.trafficTypeList();
		}

		return trafficTypeList;
	}

	private Map<Integer, TrafficType> getTrafficTypeMap()
		throws BGException
	{
		if( trafficTypeMap == null )
		{
			trafficTypeMap = Id.newMap( getTrafficTypeList() );
		}

		return trafficTypeMap;
	}

	public List<TrafficType> getInetServTrafficTypeList()
		throws BGException
	{
		if( inetServTrafficTypeList != null )
		{
			return inetServTrafficTypeList;
		}

		InetServ inetServ = getInetServ();
		if( inetServ == null )
		{
			if( inetServList == null || inetServList.size() == 0 )
			{
				return Collections.emptyList();
			}

			inetServ = inetServList.get( 0 );
		}

		InetServType inetServType = getInetServType( inetServ.getTypeId() );

		Set<Integer> trafficTypeIds = new HashSet<Integer>();
		TrafficTypeLink trafficTypeLink = trafficTypeLinkDirectory.get( inetServType.getTrafficTypeLinkId() );
		if( trafficTypeLink != null )
		{
			for( TrafficTypeLinkRule rule : trafficTypeLink.getTrafficTypeLinkRuleList() )
			{
				trafficTypeIds.add( rule.trafficTypeId );
			}
		}

		List<TrafficType> inetServTrafficTypeList = new ArrayList<TrafficType>();
		for( TrafficType type : getTrafficTypeList() )
		{
			if( !trafficTypeIds.contains( type.getId() ) )
			{
				continue;
			}

			inetServTrafficTypeList.add( type );
		}

		this.inetServTrafficTypeList = inetServTrafficTypeList;

		return inetServTrafficTypeList;
	}

	public Result<InetSessionLog> getInetSessionAliveList( int page )
		throws BGException
	{
		Set<Integer> trafficTypeIds = new HashSet<Integer>();

		List<TrafficType> trafficTypes = getTrafficTypeList();
		for( TrafficType type : trafficTypes )
		{
			trafficTypeIds.add( type.getId() );
		}

		return inetSessionService.inetSessionAliveContractList( customerBean.getContractId(), Collections.singleton( inetServId ), null, null, trafficTypeIds, null,
																new Page( page, 25 ) );
	}

	public InetServType getInetServType( int id )
		throws BGException
	{
		//logger.info( "getInetServType" );
		return inetServTypeDirectory.get( id + 0 );
	}
}
