/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.modules.inet;

import java.util.List;

import javax.annotation.PostConstruct;
import javax.enterprise.context.RequestScoped;
import javax.inject.Inject;
import javax.inject.Named;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import javax.validation.executable.ValidateOnExecution;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.modules.inet.common.service.InetServService;
import ru.bitel.bgbilling.modules.inet.common.bean.InetServ;
import ru.bitel.bgbilling.modules.inet.common.bean.InetServOption;
import ru.bitel.common.Utils;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;

@Named
@RequestScoped
@BGInjection
public class InetPasswordBean
	extends AbstractBean
{
	private static final Logger logger = LogManager.getLogger();

	@BGInject
	private InetServService inetServService;

	@Inject
	protected InetBean inetBean;

	@NotNull
	@Size(min = 1)
	private String oldPassword;

	@NotNull
	@Size(min = 1)
	private String newPassword;

	@NotNull
	@Size(min = 1)
	private String newPassword2;

	@PostConstruct
	protected void init()
	{

	}

	public String getOldPassword()
	{
		return oldPassword;
	}

	public void setOldPassword( String oldPassword )
	{
		this.oldPassword = oldPassword;
	}

	public String getNewPassword()
	{
		return newPassword;
	}

	public void setNewPassword( String newPassword )
	{
		this.newPassword = newPassword;
	}

	public String getNewPassword2()
	{
		return newPassword2;
	}

	public void setNewPassword2( String newPassword2 )
	{
		this.newPassword2 = newPassword2;
	}

	@ValidateOnExecution
	public void changePassword()
		throws BGException
	{
		if( Utils.isBlankString( oldPassword ) || Utils.isBlankString( newPassword ) || Utils.isBlankString( newPassword2 ) )
		{
			return;
		}

		logger.info( "changePassword " + newPassword2 );
		if( !newPassword.equals( newPassword2 ) )
		{
			// throw new ValidatorException( new FacesMessage( FacesMessage.SEVERITY_ERROR, "AAAA", "AAAA" ) );
			addErrorMessage( "passwordForm:newPassword", "Пароли не совпадают!" );
			addErrorMessage( "passwordForm:newPassword2", "Пароли не совпадают!" );

			return;
		}

		final InetServ inetServ = inetServService.inetServGet( inetBean.getInetServId() );
		if( inetServ.getContractId() != getContractId() )
		{
			logger.warn( "Get foreign object" );
			return;
		}

		if( !inetServ.getPassword().equals( oldPassword ) )
		{
			addErrorMessage( "passwordForm:oldPassword", "Неверно введен старый пароль!" );

			return;
		}

		if( inetServ.getPassword().equals( oldPassword ) )
		{
			final List<InetServOption> optionList = inetServService.inetServOptionList( inetServ.getId() );
			
			inetServ.setPassword( newPassword );
			
			inetServService.inetServUpdate( inetServ, optionList, false, false, 0 );

			addInfoMessage( null, "Пароль успешно изменен!" );

			inetBean.populate();
		}
	}
}
