/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.modules.inet;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.annotation.PreDestroy;
import javax.enterprise.context.ConversationScoped;
import javax.inject.Inject;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.modules.inet.common.bean.InetSessionLog;
import ru.bitel.bgbilling.modules.inet.common.bean.TrafficType;
import ru.bitel.bgbilling.modules.inet.common.service.InetSessionService;
import ru.bitel.common.TimeUtils;
import ru.bitel.common.model.Page;
import ru.bitel.common.model.Result;
import ru.bitel.mybgbilling.kernel.common.AbstractConversationBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;

@Named
@ConversationScoped
@BGInjection
public class InetSessionBean
	extends AbstractConversationBean
	implements Serializable
{
	private static final long serialVersionUID = 1L;

	private static final Logger logger = LogManager.getLogger();

	@Inject
	private InetBean inetBean;

	private Page page;

	private Date dateFrom;
	private Date dateTo;

	@BGInject
	private InetSessionService inetSessionService;

	private Result<InetSessionLog> inetSessionList;

	private Result<InetSessionLog> inetSessionLogList;

	@Override
	protected void init()
		throws BGException
	{
		super.init();

		Calendar calendar = new GregorianCalendar();
		TimeUtils.clear_HOUR_MIN_MIL_SEC( calendar );
		calendar.set( Calendar.DATE, 1 );
		dateFrom = calendar.getTime();
		calendar.add( Calendar.MONTH, 1 );
		calendar.add( Calendar.DATE, -1 );
		dateTo = calendar.getTime();

		logger.info( "init" );
	}

	@PreDestroy
	void destroy()
	{
		logger.info( "destroy" );
	}

	public Page getPage()
	{
		return page;
	}

	public void setPageIndex( int pageIndex )
	{
		this.page.setPageIndex( pageIndex );
	}

	public void setPageSize( int pageSize )
	{
		this.page.setPageSize( pageSize );
	}

	public Date getDateFrom()
	{
		return dateFrom;
	}

	public void setDateFrom( Date dateFrom )
	{
		this.dateFrom = dateFrom;
	}

	public Date getDateTo()
	{
		return dateTo;
	}

	public void setDateTo( Date dateTo )
	{
		this.dateTo = dateTo;
	}

	public List<InetSessionLog> getInetSessionLogList()
	{
		logger.info( "getInetSessionLogList" );
		return inetSessionLogList != null ? inetSessionLogList.getList() : null;
	}

	public List<InetSessionLog> getInetSessionList()
	{
		logger.info( "getInetSessionList" );
		return inetSessionList != null ? inetSessionList.getList() : null;
	}

	public void populate()
		throws BGException
	{
		Set<Integer> trafficTypeIds = new HashSet<Integer>();

		List<TrafficType> trafficTypes = inetBean.getTrafficTypeList();
		for( TrafficType type : trafficTypes )
		{
			trafficTypeIds.add( type.getId() );
		}

		if( page == null )
		{
			page = new Page( 1, 25 );
		}

		Result<InetSessionLog> result;

		switch( navigationBean.getSubPage() )
		{
			case "history":
				result = inetSessionLogList = inetSessionService.inetSessionLogContractList( customerBean.getContractId(), Collections.singleton( inetBean.getInetServId() ),
																							 dateFrom, dateTo, null, null, page );
				break;

			default:
				result = inetSessionList = inetSessionService.inetSessionAliveContractList( getContractId(), Collections.singleton( inetBean.getInetServId() ),
																							null, null, trafficTypeIds, null, page );
				break;
		}

		if( result != null && result.getPage() != null )
		{
			page = result.getPage();
		}
	}
}
