/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.modules.inet;

import java.io.Serializable;
import java.time.LocalDate;
import java.time.YearMonth;
import java.time.ZoneId;
import java.time.temporal.ChronoUnit;
import java.time.temporal.Temporal;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

import javax.annotation.PreDestroy;
import javax.inject.Inject;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.modules.inet.common.bean.InetReportUtils;
import ru.bitel.bgbilling.modules.inet.common.bean.TrafficAmount;
import ru.bitel.bgbilling.modules.inet.common.bean.TrafficMaxDetailReport;
import ru.bitel.bgbilling.modules.inet.common.bean.TrafficRangeDetail;
import ru.bitel.bgbilling.modules.inet.common.bean.TrafficRangeDetailReport;
import ru.bitel.bgbilling.modules.inet.common.bean.TrafficType;
import ru.bitel.bgbilling.modules.inet.common.service.InetSessionService;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Lazy;
import ru.bitel.common.model.Id;
import ru.bitel.common.model.Period;
import ru.bitel.common.reports.TimeSeriesCollection;
import ru.bitel.mybgbilling.kernel.common.AbstractConversationBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.scope.ViewModuleScoped;

@Named
@ViewModuleScoped
@BGInjection
public class InetTrafficBean
	extends AbstractConversationBean
	implements Serializable
{
	private static final Logger logger = LogManager.getLogger();

	@Inject
	private InetBean inetBean;

	private LocalDate dateFrom;
	private LocalDate dateTo;

	private YearMonth month;

	private Set<Integer> trafficTypeIds;

	@BGInject
	private InetSessionService inetSessionService;

	private Async<List<TrafficAmount>> trafficAmountList;

	private Async<List<TrafficRangeDetailReport>> trafficRangeDetailReportList;

	private Lazy<TimeSeriesCollection<LocalDate, Double>> trafficRangeDetailReportChart;

	private Async<List<TrafficMaxDetailReport>> trafficMaxDetailReportList;

	private Lazy<TimeSeriesCollection<Temporal, Double>> trafficMaxDetailReportChart;

	@Override
	protected void init()
	{
		dateFrom = LocalDate.now().withDayOfMonth( 1 );
		dateTo = YearMonth.from( dateFrom ).atEndOfMonth();
		//dateTo = dateFrom.plus( 1, ChronoUnit.MONTHS ).minus( 1, ChronoUnit.DAYS );
		
		month = YearMonth.now();

		logger.info( "init" );
	}

	@PreDestroy
	void destroy()
	{
		logger.info( "destroy" );
	}

	public LocalDate getDateFrom()
	{
		return dateFrom;
	}

	public void setDateFrom( LocalDate dateFrom )
	{
		this.dateFrom = dateFrom;
	}

	public LocalDate getDateTo()
	{
		return dateTo;
	}

	public void setDateTo( LocalDate dateTo )
	{
		this.dateTo = dateTo;
	}

	public YearMonth getMonth()
	{
		return month;
	}

	public void setMonth( YearMonth month )
	{
		this.month = month;
	}

	public Set<Integer> getTrafficTypeIds()
	{
		return trafficTypeIds;
	}

	public void setTrafficTypeIds( Set<Integer> trafficTypeIds )
	{
		this.trafficTypeIds = trafficTypeIds;
	}

	public List<TrafficAmount> getTrafficAmountList()
		throws BGException
	{
		return trafficAmountList != null ? trafficAmountList.get() : null;
	}

	public List<TrafficRangeDetailReport> getTrafficRangeDetailReportList()
		throws BGException
	{
		return trafficRangeDetailReportList != null ? trafficRangeDetailReportList.get() : null;
	}

	public TimeSeriesCollection<LocalDate, Double> getTrafficRangeDetailReportChart()
		throws BGException
	{
		return trafficRangeDetailReportChart != null ? trafficRangeDetailReportChart.get() : null;
	}

	public List<TrafficMaxDetailReport> getTrafficMaxDetailReportList()
		throws BGException
	{
		return trafficMaxDetailReportList != null ? trafficMaxDetailReportList.get() : null;
	}

	public TimeSeriesCollection<Temporal, Double> getTrafficMaxDetailReportChart()
		throws BGException
	{
		return trafficMaxDetailReportChart != null ? trafficMaxDetailReportChart.get() : null;
	}

	public void populate()
		throws BGException
	{
		Set<Integer> trafficTypeIds = new HashSet<Integer>();

		List<TrafficType> trafficTypes = inetBean.getTrafficTypeList();
		for( TrafficType type : trafficTypes )
		{
			trafficTypeIds.add( type.getId() );
		}

		if( this.trafficTypeIds != null && this.trafficTypeIds.size() > 0 )
		{
			trafficTypeIds.retainAll( this.trafficTypeIds );
		}

		final int inetServId = inetBean.getInetServId();

		final YearMonth month = this.month;

		if( this.dateFrom == null )
		{
			this.dateFrom = LocalDate.now().withDayOfMonth( 1 );
		}

		if( this.dateTo == null || !YearMonth.from( this.dateFrom ).equals( YearMonth.from( this.dateTo ) ) )
		{
			this.dateTo = YearMonth.from( this.dateFrom ).atEndOfMonth();
		}

		final LocalDate dateFrom = this.dateFrom;
		final LocalDate dateTo = this.dateTo;

		final Date dateFromDate = Date.from( dateFrom.atStartOfDay().atZone( ZoneId.systemDefault() ).toInstant() );
		final Date dateToDate = Date.from( dateTo.atStartOfDay().atZone( ZoneId.systemDefault() ).toInstant() );

		//boolean byDay = !TimeUtils.dateEqual( dateFrom, dateTo );//&& !TimeUtils.dateEqual( TimeUtils.getNextDay( dateFrom ), dateTo );
		boolean byDay = !dateFrom.equals( dateTo );

		switch( navigationBean.getSubPage() )
		{
			case "range":
			{
				//				final Date dateTo2 = Date.from( dateFrom.toInstant().atZone( ZoneId.systemDefault() ).toLocalDate().plus( 1, ChronoUnit.MONTHS ).minus( 1, ChronoUnit.DAYS ).atStartOfDay()
				//											.atZone( ZoneId.systemDefault() ).toInstant() );

				final Date dateFrom2 = Date.from( month.atDay( 1 ).atStartOfDay().atZone( ZoneId.systemDefault() ).toInstant() );
				final Date dateTo2 = Date.from( month.atEndOfMonth().atStartOfDay()
													 .atZone( ZoneId.systemDefault() ).toInstant() );

				trafficRangeDetailReportList =
					Async.of( () -> Utils.maskNull( inetSessionService.inetSessionTrafficRange( getContractId(), 0, dateFrom2, dateTo2, 1024 * 1024 ) ) );

				trafficRangeDetailReportChart = Lazy.of( () -> {

					final TimeSeriesCollection<LocalDate, Double> chart = InetReportUtils.rangeChart( dateFrom2, getTrafficRangeDetailReportList(), 1024 * 1024 );

					if( chart != null )
					{
						chart.setXMin( month.atDay( 1 ) )
							 .setXMax( month.atEndOfMonth() )
							 .normalize();
					}

					return chart;
				} );

				break;
			}

			case "max":
			{
				//trafficRangeDetailReportList = Async.of( ( ) -> inetSessionService.inetSessionTrafficMax( getContractId(), 0, dateFrom, dateTo, 1024 * 1024 ) );
				trafficMaxDetailReportList =
					Async.of( () -> Utils.maskNull( inetSessionService.inetSessionTrafficMax( getContractId(), null, dateFromDate, dateToDate, 1024 * 1024 ) ) );

				final Map<Integer, TrafficType> trafficTypeMap = Id.newMap( inetBean.getTrafficTypeList() );

				trafficMaxDetailReportChart = Lazy.of( () -> {

					final TimeSeriesCollection<Temporal, Double> chart =
						InetReportUtils.maxChart( dateFromDate, dateToDate, getTrafficMaxDetailReportList(), 1024 * 1024, trafficTypeMap );

					if( chart != null )
					{
						if( byDay )
						{
							chart.setXMin( dateFrom )
								 .setXMax( dateTo )
								 .normalize();
						}
						else
						{
							chart.setXMin( dateFrom.atStartOfDay() )
								 .setXMax( dateTo.atStartOfDay().plus( 1, ChronoUnit.DAYS ).minus( 1, ChronoUnit.HOURS ) )
								 .normalize();
						}
					}

					return chart;
				} );

				break;
			}

			default:
			{
				
				logger.debug( "trafficTypeIds " + trafficTypeIds );
				
				trafficAmountList = Async.of( () -> Utils.maskNull( inetSessionService.inetSessionTraffic( getContractId(), Collections.singleton( inetServId ),
																										   -1, trafficTypeIds, new Period( dateFromDate, dateToDate ), 1024 * 1024,
																										   byDay, true ) ) );
				break;
			}
		}
	}

	public Map<Integer, TrafficRangeDetail> toMap( List<TrafficRangeDetail> list )
	{
		return list.stream().collect( Collectors.toMap( a -> a.getDay(), Function.identity() ) );
	}
}
