/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.modules.license;

import java.io.ByteArrayInputStream;
import java.io.UnsupportedEncodingException;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.inject.Inject;
import javax.inject.Named;
import javax.mail.BodyPart;
import javax.mail.Multipart;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMultipart;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.contract.api.common.service.ContractService;
import ru.bitel.bgbilling.modules.license.common.bean.Component;
import ru.bitel.bgbilling.modules.license.common.bean.ConfirmedOrder;
import ru.bitel.bgbilling.modules.license.common.bean.Copy;
import ru.bitel.bgbilling.modules.license.common.bean.License;
import ru.bitel.bgbilling.modules.license.common.service.LicenseService;
import ru.bitel.bgbilling.modules.license.common.service.OrderService;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Lazy;
import ru.bitel.common.model.MapHolder;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.Configuration;
import ru.bitel.mybgbilling.kernel.common.MailMsg;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.scope.SessionModuleScoped;
import ru.bitel.oss.kernel.entity.common.bean.EntityAttr;
import ru.bitel.oss.kernel.entity.common.bean.EntityAttrEmail;
import ru.bitel.oss.kernel.entity.common.bean.EntityAttrText;

@Named
@SessionModuleScoped
@BGInjection
public class LicenseBean
	extends AbstractBean
	implements Converter
{
	private static final long serialVersionUID = -1680286207783924596L;

	private static final Logger logger = LogManager.getLogger();

	@Override
	public Object getAsObject( FacesContext arg0, UIComponent arg1, String arg2 )
	{
		final int id = Utils.parseInt( arg2 );
		return getComponentList().stream().filter( ( a ) -> a.getId() == id ).findFirst().orElse( null );
	}

	@Override
	public String getAsString( FacesContext arg0, UIComponent arg1, Object arg2 )
	{
		if( arg2 == null )
		{
			return "0";
		}

		return String.valueOf( ((Component)arg2).getId() );
	}

	@Inject
	private Configuration configuration;

	@BGInject(module = false)
	private ContractService contractService;

	@BGInject
	private LicenseService licenseService;

	@BGInject
	private OrderService orderService;

	private Supplier<List<Copy>> productList;

	private int productId;
	private Copy product;

	@Min(value = 0)
	@Max(value = 2)
	private int componentFilter = 0;

	/**
	 * Компоненты для каждой копии продукта
	 */
	private Map<Integer, Supplier<List<Component>>> componentListMap = new HashMap<>();

	/**
	 * Компоненты в наличии (подтвержденные заказы)
	 */
	private Async<List<ConfirmedOrder>> confirmedOrderList;

	/**
	 * Выбор компонентов (галочки)
	 */
	private Map<String, Boolean> selectedComponentMap = new HashMap<>();

	/**
	 * Текущая лицензия
	 */
	private Async<License> license;

	@Override
	protected void init()
		throws BGException
	{
		super.init();

		componentListMap.put( 0, Lazy.of( () -> Collections.emptyList() ) );

		populateProductList();
	}

	public int getProductId()
	{
		return productId;
	}

	public void setProductId( int productId )
	{
		this.productId = productId;

		this.product = null;

		for( Copy product : getProductList() )
		{
			if( product.getId() == productId )
			{
				this.product = product;
				break;
			}
		}

		if( product == null )
		{
			this.productId = 0;
		}

		componentListMap.computeIfAbsent( this.productId, ( k ) -> Async.of( () -> orderService.getComponentItems( k, false, true ) ) );

		selectedComponentMap.clear();
	}

	public List<Component> getComponentList()
	{
		return componentListMap.get( this.productId ).get();
	}

	public int getComponentFilter()
	{
		return componentFilter;
	}

	public void setComponentFilter( int componentFilter )
	{
		this.componentFilter = componentFilter;
	}

	public Map<String, Boolean> getSelectedComponentMap()
	{
		return selectedComponentMap;
	}

	public void setSelectedComponentMap( Map<String, Boolean> selectedOrderMap )
	{
		this.selectedComponentMap = selectedOrderMap;
	}

	public List<Copy> getProductList()
	{
		return productList.get();
	}

	public List<ConfirmedOrder> getConfirmedOrderList()
	{
		return confirmedOrderList != null ? confirmedOrderList.get() : null;
	}

	public License getLicense()
	{
		return license.get();
	}

	public void populateProductList()
	{
		productList = Async.of( () -> Utils.maskNull( licenseService.copyList( getContractId() ) ) );
	}

	public void populateConfirmedOrderList()
	{
		final int copyId = this.productId;

		if( copyId > 0 )
		{
			confirmedOrderList = Async.of( () -> Utils.maskNull( licenseService.confirmedOrderList( getContractId(), copyId, componentFilter ) ) );
		}
		else
		{
			confirmedOrderList = Async.of( () -> Collections.emptyList() );
		}

		selectedComponentMap.clear();
	}

	public void populate()
		throws BGException
	{
		populateConfirmedOrderList();

		populateLicense();
	}

	public void createLicense()
		throws BGException
	{
		Set<Set<Integer>> selectedOrderIds =
			this.selectedComponentMap.entrySet().stream().filter( ( e ) -> Boolean.TRUE.equals( e.getValue() ) ).map( ( e ) -> e.getKey() ).map( ( a ) -> Utils.toIntegerSet( a ) )
									 .collect( Collectors.toSet() );

		if( selectedOrderIds.size() == 0 )
		{
			addErrorMessage( null, "Выберите хотя бы один компонент." );
			return;
		}

		Set<Set<Integer>> testSelectedOrderIds = new HashSet<>( selectedOrderIds );

		for( ConfirmedOrder confirmedOrder : Utils.maskNull( licenseService.confirmedOrderList( getContractId(), productId, 2 ) ) )
		{
			testSelectedOrderIds.remove( Utils.toIntegerSet( confirmedOrder.getIds() ) );
		}

		if( testSelectedOrderIds.size() > 0 )
		{
			addErrorMessage( null, "Выбраны не верные компоненты." );
			return;
		}

		licenseService.licenseCreate( getContractId(), productId, selectedOrderIds.stream().map( a -> Utils.toString( a ) ).collect( Collectors.toSet() ), true );
		
		populateLicense();

		navigationBean.setSubPage( "license" );
	}

	public void populateLicense()
		throws BGException
	{
		license = Async.of( () -> {

			List<License> licenseList = licenseService.licenseList( getContractId(), productId );

			if( licenseList.size() == 0 )
			{
				return null;
			}

			License license = licenseList.get( 0 );

			license.setCopyItems( licenseService.licenseComponentList( getContractId(), license.getId() ) );

			return license;

		} );
	}

	public void mailLicense()
		throws BGException
	{
		Integer parameterId = (Integer)configuration.get( "license.mail.parameterId", 0 );

		if( parameterId == null || parameterId <= 0 )
		{
			addErrorMessage( null, "Отправка лицензии по email отключена" );
			return;
		}

		MapHolder<Integer, EntityAttr> contractParameterMap = contractService.contractParameterMap( getContractId(), customerBean.getContract().getParamGroupId() );

		EntityAttr param = contractParameterMap.getMap().get( parameterId );
		if( param == null )
		{
			addErrorMessage( null, "Не удалось отправить текст лицензии. В параметрах договора не задан электронный адрес для отправки" );
			return;
		}

		String recipientList = null;

		if( param instanceof EntityAttrText )
		{
			recipientList = ((EntityAttrText)param).getValue();
		}
		else if( param instanceof EntityAttrEmail )
		{
			recipientList = ((EntityAttrEmail)param).getData();
		}

		if( Utils.isBlankString( recipientList ) )
		{
			addErrorMessage( null, "Не удалось отправить текст лицензии. В параметрах договора не задан электронный адрес для отправки" );
			return;
		}

		StringBuilder mails = new StringBuilder();

		for( String recipient : recipientList.split( "\n" ) )
		{
			if( Utils.notBlankString( recipient ) )
			{
				mails.append( recipient );
				mails.append( ";" );
			}
		}

		try
		{
			String licenseText = license.get().getData();

			String content = "Для версии 4.6 и старше: файл необходимо положить в " +
							 "BGBillingServer/data/.\n" +
							 "Для версии 4.5 и младше: необходимо заменить все ключи " +
							 "lic.* и licence.* расположенные после lic.key " +
							 "в файле BGBillingServer/data/data.properties на " +
							 "содержимое полученного файла lic.properties.";

			Multipart multipart = new MimeMultipart();

			BodyPart part1 = new MimeBodyPart();
			part1.setText( content );
			multipart.addBodyPart( part1 );
			//
			BodyPart part2 = new MimeBodyPart();
			part2.setFileName( "lic.properties" );//кодирование миме-заголовка не делаем — латиница
			part2.setDisposition( "attachment" );
			part2.setText( licenseText );
			multipart.addBodyPart( part2 );

			new MailMsg( configuration.getParameterMap() ).sendMessage( mails.toString(), "Текст лицензии", multipart );

			addInfoMessage( null, "Текст лицензии отправлен на адрес(а): " + mails );
		}
		catch( Exception e )
		{
			logger.error( e.getMessage(), e );

			addErrorMessage( null, "Ошибка при отправке сообщения: " + e.getMessage() );
		}
	}

	public void downloadLicense()
		throws BGException
	{
		String licenseText = license.get().getData();

		try
		{
			super.writeToOutput( new ByteArrayInputStream( licenseText.getBytes( "UTF-8" ) ), "lic.properties", "text/plain", false );
		}
		catch( UnsupportedEncodingException e )
		{
			throw new BGException( e );
		}
	}
}
