/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.modules.license;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

import javax.inject.Inject;
import javax.inject.Named;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.common.BGIllegalArgumentException;
import ru.bitel.bgbilling.kernel.module.common.bean.User;
import ru.bitel.bgbilling.modules.license.common.bean.Component;
import ru.bitel.bgbilling.modules.license.common.bean.Order;
import ru.bitel.bgbilling.modules.license.common.bean.OrderBillData;
import ru.bitel.bgbilling.modules.license.common.bean.OrderType;
import ru.bitel.bgbilling.modules.license.common.service.OrderService;
import ru.bitel.common.ParameterMap;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.Configuration;
import ru.bitel.mybgbilling.kernel.common.MailMsg;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.scope.SessionModuleScoped;

@Named
@SessionModuleScoped
@BGInjection
public class LicenseOrderBean
	extends AbstractBean
{
	@Inject
	private Configuration configuration;

	@Inject
	private LicenseBean licenseBean;

	@BGInject
	private OrderService orderService;

	@Min(value = -1)
	@Max(value = 2)
	private int orderStatusFilter = 0;

	@Min(value = -1)
	@Max(value = 1)
	private int orderTypeFilter = -1;

	private int orderComponentIdFilter = -1;

	private Async<List<Order>> orderList;

	private Component createOrderComponent;

	private int createOrderType = -1;

	private int createOrderCount;

	private int createOrderTerm = 30;

	private Map<Integer, Boolean> selectedOrderMap = new HashMap<>();

	private int copyOrdersTerm;

	private Async<List<OrderBillData>> orderBillDataList;

	private String requestBillComment;

	private String requestBillEmail;

	public int getOrderStatusFilter()
	{
		return orderStatusFilter;
	}

	public void setOrderStatusFilter( int orderStatusFilter )
	{
		this.orderStatusFilter = orderStatusFilter;
	}

	public int getOrderTypeFilter()
	{
		return orderTypeFilter;
	}

	public void setOrderTypeFilter( int orderTypeFilter )
	{
		this.orderTypeFilter = orderTypeFilter;
	}

	public int getOrderComponentIdFilter()
	{
		return orderComponentIdFilter;
	}

	public void setOrderComponentIdFilter( int orderComponentIdFilter )
	{
		this.orderComponentIdFilter = orderComponentIdFilter;
	}

	public List<OrderBillData> getOrderBillDataList()
	{
		return Optional.ofNullable( orderBillDataList ).map( Async::get ).orElse( null );
	}

	public void setOrderBillDataList( Async<List<OrderBillData>> orderBillDataList )
	{
		this.orderBillDataList = orderBillDataList;
	}

	public String getRequestBillComment()
	{
		return requestBillComment;
	}

	public void setRequestBillComment( String requestBillComment )
	{
		this.requestBillComment = requestBillComment;
	}

	public String getRequestBillEmail()
	{
		return requestBillEmail;
	}

	public void setRequestBillEmail( String requestBillEmail )
	{
		this.requestBillEmail = requestBillEmail;
	}

	@Override
	protected void init()
		throws BGException
	{
		super.init();

		populate();
	}

	public void populate()
	{
		final int copyId = licenseBean.getProductId();

		if( copyId > 0 )
		{
			orderList = Async.of( () -> Utils.maskNull( orderService.getOrdersList( copyId, orderTypeFilter, orderStatusFilter, orderComponentIdFilter ) ) );
		}
		else
		{
			orderList = Async.of( () -> Collections.emptyList() );
		}
	}

	public List<Order> getOrderList()
	{
		return orderList != null ? orderList.get() : null;
	}

	public Component getCreateOrderComponent()
	{
		return createOrderComponent;
	}

	public void setCreateOrderComponent( Component createOrderComponent )
	{
		this.createOrderComponent = createOrderComponent;
	}

	public int getCreateOrderType()
	{
		return createOrderType;
	}

	public void setCreateOrderType( int createOrderType )
	{
		this.createOrderType = createOrderType;
	}

	public int getCreateOrderCount()
	{
		return createOrderCount;
	}

	public void setCreateOrderCount( int createOrderCount )
	{
		this.createOrderCount = createOrderCount;
	}

	public int getCreateOrderTerm()
	{
		return createOrderTerm;
	}

	public void setCreateOrderTerm( int createOrderTerm )
	{
		this.createOrderTerm = createOrderTerm;
	}

	public Map<Integer, Boolean> getSelectedOrderMap()
	{
		return selectedOrderMap;
	}

	public void setSelectedOrderMap( Map<Integer, Boolean> selectedOrderMap )
	{
		this.selectedOrderMap = selectedOrderMap;
	}

	public void createOrder()
		throws BGException
	{
		if( createOrderComponent == null || !licenseBean.getComponentList().contains( createOrderComponent ) )
		{
			throw new BGIllegalArgumentException( "createOrderComponent" );
		}

		if( createOrderType != 0 && createOrderType != 1 )
		{
			throw new BGIllegalArgumentException( "createOrderType" );
		}

		if( createOrderComponent.getUnlimited() == 1 )
		{
			createOrderCount = 0;
		}
		else if( createOrderCount < 0 )
		{
			createOrderCount = 0;
		}

		if( createOrderType == 0 )
		{
			if( createOrderTerm <= 0 )
			{
				addErrorMessage( "createOrderForm:term", "Укажите правильную длительность тестовой лицензии" );
				return;
			}
		}

		Order order = new Order();
		order.setCopyId( licenseBean.getProductId() );
		order.setCount( createOrderCount );
		order.setType( OrderType.getOrderTypeByCode( createOrderType ) );
		order.setComponentId( createOrderComponent.getId() );
		order.setStatus( Order.STATUS_NEW );
		order.setUserId( User.USER_CONTRACT );
		order.setTerm( createOrderTerm );

		orderService.orderCreate( getContractId(), order, true );

		populate();

		navigationBean.setSubPage( "orders" );
	}

	public int getCopyOrdersTerm()
	{
		return copyOrdersTerm;
	}

	public void setCopyOrdersTerm( int copyOrdersTerm )
	{
		this.copyOrdersTerm = copyOrdersTerm;
	}

	public void copyOrders()
		throws BGException
	{
		if( this.copyOrdersTerm <= 0 )
		{
			addErrorMessage( null, "Укажите количество дней." );
			return;
		}

		Set<Integer> selectedOrderIds =
			this.selectedOrderMap.entrySet().stream().filter( ( e ) -> Boolean.TRUE.equals( e.getValue() ) ).map( ( e ) -> e.getKey() ).collect( Collectors.toSet() );

		if( selectedOrderIds.size() == 0 )
		{
			addErrorMessage( null, "Выберите хотя бы один заказ." );
			return;
		}

		Set<Integer> testSelectedOrderIds = new HashSet<>( selectedOrderIds );

		for( Order order : getOrderList() )
		{
			testSelectedOrderIds.remove( order.getId() );
		}

		if( testSelectedOrderIds.size() > 0 )
		{
			addErrorMessage( null, "Выбраны не верные заказы." );
			return;
		}

		orderService.ordersCopy( getContractId(), licenseBean.getProductId(), selectedOrderIds, copyOrdersTerm );

		populate();
	}

	public void deleteOrders()
		throws BGException
	{
		Set<Integer> selectedOrderIds =
			this.selectedOrderMap.entrySet().stream().filter( ( e ) -> Boolean.TRUE.equals( e.getValue() ) ).map( ( e ) -> e.getKey() ).collect( Collectors.toSet() );

		if( selectedOrderIds.size() == 0 )
		{
			addErrorMessage( null, "Выберите хотя бы один заказ." );
			return;
		}

		Set<Integer> testSelectedOrderIds = new HashSet<>( selectedOrderIds );

		for( Order order : getOrderList() )
		{
			testSelectedOrderIds.remove( order.getId() );
		}

		if( testSelectedOrderIds.size() > 0 )
		{
			addErrorMessage( null, "Выбраны не верные заказы." );
			return;
		}

		orderService.ordersDelete( getContractId(), licenseBean.getProductId(), selectedOrderIds );

		this.selectedOrderMap.clear();

		populate();
	}

	public void orderBillDataList()
		throws BGException
	{
		Set<Integer> selectedOrderIds =
			this.selectedOrderMap.entrySet().stream().filter( ( e ) -> Boolean.TRUE.equals( e.getValue() ) ).map( ( e ) -> e.getKey() ).collect( Collectors.toSet() );

		if( selectedOrderIds.size() == 0 )
		{
			addErrorMessage( null, "Выберите хотя бы один заказ." );
			return;
		}

		Set<Integer> testSelectedOrderIds = new HashSet<>( selectedOrderIds );

		for( Order order : getOrderList() )
		{
			testSelectedOrderIds.remove( order.getId() );
		}

		if( testSelectedOrderIds.size() > 0 )
		{
			addErrorMessage( null, "Выбраны не верные заказы." );
			return;
		}

		final int productId = licenseBean.getProductId();

		orderBillDataList = Async.of( () -> orderService.ordersBillDataList( getContractId(), productId, selectedOrderIds ) );

		orderBillDataList.get();

		navigationBean.setSubPage( "orderBillDataList" );
	}

	public void requestBill()
		throws BGException
	{
		String senderEmail = requestBillEmail; //email для отправки счета

		if( Utils.isBlankString( senderEmail ) || !Utils.checkEmail( senderEmail ) )
		{
			addErrorMessage( null, "Укажите правильный адрес электроннной почты." );
			return;
		}

		StringBuilder emailMessage = new StringBuilder();
		emailMessage.append( "Договор " + customerBean.getContract().getTitle() + "\n" );

		emailMessage.append( getBillsInfo() );

		String comment = Utils.maskBlank( requestBillComment, "" ); //комментарий пользователя

		String receiverEmail = String.valueOf( configuration.get( "license.bills.mailTo", "" ) ); //email получателя

		if( Utils.isBlankString( receiverEmail ) )
		{
			addErrorMessage( null, "Ошибка отправки." );
			return;
		}

		if( Utils.notBlankString( comment ) )
		{
			emailMessage.append( "==========\nКомментарий:\n" );
			emailMessage.append( comment );
		}
		emailMessage.append( "\nE-mail отправителя:" + senderEmail );

		ParameterMap setup = configuration.getParameterMap();

		MailMsg message = new MailMsg( setup );
		if( Utils.notBlankString( receiverEmail ) )
		{
			message.sendMessage( receiverEmail, "BGBilling", emailMessage.toString() );
		}

		navigationBean.setSubPage( "orders" );
	}

	private String getBillsInfo()
	{
		final String[] columnTitles = { "Наименование",
										"Количество в наличии", "На сумму",
										"Количество заказано", "На сумму",
										"Общее количество", "На сумму",
										"К оплате" };

		StringBuilder bills = new StringBuilder();

		bills.append( columnTitles[0] + " || " +
					  columnTitles[1] + " | " + columnTitles[2] + " || " +
					  columnTitles[3] + " | " + columnTitles[4] + " || " +
					  columnTitles[5] + " | " + columnTitles[6] + " || " +
					  columnTitles[7] + "\n" );

		BigDecimal total = BigDecimal.ZERO;

		for( OrderBillData data : getOrderBillDataList() )
		{
			bills.append( data.getComponentTitle() + " || " );
			bills.append( data.getConfirmedCount() + " | " );
			bills.append( Utils.formatCost( data.getConfirmedCost() ) + " || " );
			bills.append( data.getCountDelta() + " | " );
			bills.append( Utils.formatCost( data.getCostDelta() ) + " || " );
			bills.append( data.getTotalCount() + " | " );
			bills.append( Utils.formatCost( data.getTotalCost() ) + " || " );
			bills.append( Utils.formatCost( data.getCostDelta() ) + "\n" );

			total = total.add( data.getCostDelta() );
		}

		bills.append( "Итого к оплате: " + Utils.formatCost( total ) + "\n" );

		return bills.toString();
	}
}
