package ru.bitel.mybgbilling.modules.megogo;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.enterprise.context.SessionScoped;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.inject.Named;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.directory.api.common.bean.Directory;
import ru.bitel.bgbilling.kernel.module.common.bean.Service;
import ru.bitel.bgbilling.modules.megogo.common.bean.MegogoContractCredentials;
import ru.bitel.bgbilling.modules.megogo.common.bean.MegogoContractSubscription;
import ru.bitel.bgbilling.modules.megogo.common.bean.MegogoSubscription;
import ru.bitel.bgbilling.modules.megogo.common.bean.MegogoTryAndBuy;
import ru.bitel.bgbilling.modules.megogo.common.service.MegogoService;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.common.model.Period;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;

@Named
@SessionScoped
@BGInjection
public class MegogoBean extends AbstractBean
{
    @BGInject
    private MegogoService megogoService;
    @BGInject(type = Service.class)
    private Directory<Service> serviceDir;
    
    private List<MegogoContractSubscription> contractSubscriptions;
    private List<MegogoContractSubscription> contractSubscriptionsAll;
    private List<MegogoSubscription> subscriptions;
    private Map<Integer, String> connectButtonMap;
    
    private Async<MegogoContractCredentials> credential;
    
    @Override
    protected void init()       throws BGException
    {
        super.init();
        populate();
    }
    
    public void populate()      throws BGException
    {
        this.credential = Async.of( () -> megogoService.getContractCredentials( getContractId() ) );
        
        this.contractSubscriptions = Utils.maskNull( megogoService.getContractSubscriptionListByType( getContractId(), -1, new Period( new Date(), null ) ) );
        this.contractSubscriptionsAll = Utils.maskNull( megogoService.contractSubscriptionList( getContractId(), -1, null ) );
        
        for( MegogoContractSubscription subscription : contractSubscriptionsAll )
        {
            initTitle( subscription );
        }
        
        Set<Integer> serviceIdConnectList = new HashSet<>();
        boolean haveMainSub = false;
        boolean haveMainInDisconnect = false;
        boolean firstConnection = contractSubscriptionsAll.size() == 0;// если 1 подлючение.
        
        for( MegogoContractSubscription subscription : contractSubscriptions )
        {
            initTitle( subscription );
            serviceIdConnectList.add( subscription.getServiceId() );
            // помечаем, что есть основная подписка.
            MegogoSubscription sub = megogoService.getSubscriptionByServiceId( subscription.getServiceId() );
            if( sub.getType() == MegogoSubscription.SUBSCRIPTION_TYPE_MAIN )
            {
                haveMainSub = true;
                if( subscription.getStatus() == MegogoContractSubscription.SUBSCRIPTION_STATUS_IN_DISCONNECT )
                {
                    haveMainInDisconnect = true;
                }
            }
        }
        
        connectButtonMap = new HashMap<>();
        Map<Integer, MegogoTryAndBuy> tryAndBuyMap = megogoService.getTryAndBuyMap( true ).getMap();
        
        // даем подключить(или перейти) только те, что еще не подключены
        subscriptions = new ArrayList<>();
        
        tryAndBuyCase:
        for( MegogoSubscription subscription : megogoService.subscriptionList( true, -1 ) )
        {
            if( !serviceIdConnectList.contains( subscription.getSid() ) )
            {
                if( haveMainInDisconnect && subscription.getType() == MegogoSubscription.SUBSCRIPTION_TYPE_MAIN )
                {// не даем выбрать переход на основную, если текущая основная находиться на этапе отключения 
                    continue;
                }
                Service service = serviceDir.get( subscription.getSid() );
                subscription.setTitle( service != null ? service.getTitle() : "Неизвестная услуга" );
                subscriptions.add( subscription );
                
                // текст который будет отображаться на кнопке подключения: Подключить( первый раз, либо дополнительный тип), Перейти(если есть основной и это основной тип) или try&Buy
                String buttonConText = ( subscription.getType() == MegogoSubscription.SUBSCRIPTION_TYPE_MAIN && haveMainSub ) ? "Перейти" : "Подключить" ;
                connectButtonMap.put( subscription.getSid(), buttonConText );
                
                MegogoTryAndBuy tAb = tryAndBuyMap.get( subscription.getSid() );
                if( tAb != null )
                {
                    // если для первых, а это уже не первое подлючение подписки, то идем дальше
                    if( tAb.getType() == MegogoTryAndBuy.TYPE_ONLY_FOR_NEW && !firstConnection )
                        continue;
                    // если только для первой подписки, то если среди всех подключенных была такая, то идем дальше
                    if( tAb.getType() == MegogoTryAndBuy.TYPE_FIRST_SERVICE )
                    {
                        for( MegogoContractSubscription contractSubscription : contractSubscriptionsAll )
                        {
                            if( contractSubscription.getServiceId() == tAb.getSid() )
                                continue tryAndBuyCase;
                        }
                    }
                    // раз дошли до сюда, предлагаем.
                    connectButtonMap.put( subscription.getSid() , tAb.getTitle() );
                }
            }
        }
    }
    
    public MegogoContractCredentials getCredential( )
    {
        return credential.get();
    }
    
    public List<MegogoContractSubscription> getContractSubscriptions()
    {
        return contractSubscriptions;
    }
    
    public List<MegogoContractSubscription> getContractSubscriptionsAll()
    {
        return contractSubscriptionsAll;
    }
    
    public List<MegogoSubscription> getSubscriptions() 
    {
        return subscriptions;
    }
    
    public Map<Integer, String> getConnectButtonMap() throws BGException
    {
        return connectButtonMap;
    }

//    public void moveToMegogo() throws IOException, BGException  {
//        ExternalContext ec = FacesContext.getCurrentInstance().getExternalContext();
//        ec.redirect( megogoService.getRedirectUrl( getContractId() ) );
//    }
    
    public String urlToMegogo() throws IOException, BGException
    {
        return megogoService.getRedirectUrl( getContractId() );
    }
    
    // подключение/переход подписки
    public void subscribe( int serviceId ) throws BGException
    {
        try
        {
            if( serviceId > 0 )
            {
                List<MegogoContractSubscription> contractlistAll = Utils.maskNull( megogoService.contractSubscriptionList( getContractId(), -1, null ) );
                boolean firstConnection = contractlistAll.size() == 0;// если 1 подлючение.
                
                megogoService.subscribe( getContractId(), serviceId );
                
                if( firstConnection )
                {
                    addInfoMessage( null, "Ваша подписка успешно активирована, закончите регистрацию по ссылке и кнопке \"Перейти на Megogo для просмотра\"" );                    
                }
            }
        }
        finally
        {
            populate();
        }
    }
    
    /** Отказ от подписки */
    public void unsubscribe( int contractSubscribeId ) throws BGException
    {
        try
        {
            if( contractSubscribeId > 0 )
            {
                megogoService.unsubscribe( contractSubscribeId );
            }
        }
        finally
        {
            populate();
        }
    }
    
    /** Отмена от перехода/отказа    */
    public void abortUnsubscribe( int contractSubscribeId ) throws BGException
    {
        try
        {
            if( contractSubscribeId > 0 )
            {
                megogoService.abortUnsubscribe( contractSubscribeId );
            }
        }
        finally
        {
            populate();
        }
    }
    
    private void initTitle( MegogoContractSubscription subscription ) throws BGException
    {
        if( subscription != null )
        {
            Service service = serviceDir.get( subscription.getServiceId() );
            subscription.setTitle( service == null ? "Неизвестная услуга" : service.getTitle() );
        }
    }

}
