package ru.bitel.mybgbilling.modules.payment;

import java.io.Serializable;
import java.math.BigDecimal;
import java.time.ZonedDateTime;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

import javax.inject.Named;
import javax.xml.ws.Holder;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.common.BGIllegalAccessException;
import ru.bitel.bgbilling.common.BGIllegalArgumentException;
import ru.bitel.bgbilling.common.BGMessageException;
import ru.bitel.bgbilling.common.bean.BGAbstractTransaction;
import ru.bitel.bgbilling.kernel.contract.autopayment.common.bean.Autopayment;
import ru.bitel.bgbilling.kernel.contract.autopayment.common.bean.ContractAutopaymentMode;
import ru.bitel.bgbilling.modules.alfabank.common.bean.Transaction;
import ru.bitel.bgbilling.modules.alfabank.common.service.AlfabankService;
import ru.bitel.common.function.Expirable;
import ru.bitel.common.model.MapHolder;
import ru.bitel.common.model.Page;
import ru.bitel.common.model.Period;
import ru.bitel.common.model.SearchResult;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.scope.SessionModuleScoped;

@Named
@SessionModuleScoped
@BGInjection
public class AlfabankPaymentBean
    extends AbstractPaymentBean
    implements PaymentServiceProviderBean, Serializable
{
    private static final Logger logger = LogManager.getLogger();
    
    @BGInject
    private AlfabankService alfabankService;
    
    private Expirable<SearchResult<Transaction>> recentTransactionList;

    private Period getPeriod()
    {
        ZonedDateTime now = ZonedDateTime.now();
        return new Period( Date.from( now.minusDays( 30 ).toInstant() ),
            Date.from( now.plusMinutes( 10 ).toInstant() ) );
    }

    public void populate()
    {
        recentTransactionList = Expirable.asyncOf(
            () -> alfabankService.searchTransactionList( getContractId(), null, null, getPeriod(), null, new Page( 1, 10 ) ),
            1, TimeUnit.MINUTES );
    }

    @Override
    protected void init()
    {
        populate();
    }

    @Override
    public Autopayment autopaymentGet( int contractId )
        throws BGException, BGMessageException, BGIllegalArgumentException, BGIllegalAccessException
    {
        return alfabankService.autopaymentGet( contractId );
    }

    @Override
    public void autopaymentDeactivate( int contractId )
        throws BGException, BGMessageException, BGIllegalArgumentException, BGIllegalAccessException
    {
        alfabankService.autopaymentDeactivate( contractId );
    }

    @Override
    public void beginTransaction( int contractId, BigDecimal amount, Holder<String> url, Holder<String> mode, Holder<MapHolder<String, String>> params )
        throws BGException
    {
        alfabankService.beginTransaction( contractId, amount, url, mode, params );
    }

    @Override
    public void beginAutopaymentTransaction( int contractId, BigDecimal amount, Holder<String> url, Holder<String> mode,
                                             Holder<MapHolder<String, String>> params,
                                             ContractAutopaymentMode autopaymentMode )
        throws BGException, BGMessageException, BGIllegalArgumentException, BGIllegalAccessException
    {
        alfabankService.beginAutopaymentTransaction( contractId, amount, url, mode, params, autopaymentMode );
    }

    @Override
    public void endTransaction( int contractId, BigDecimal amount, Holder<String> url, Holder<String> mode, Holder<MapHolder<String, String>> params )
        throws BGException, BGMessageException, BGIllegalArgumentException, BGIllegalAccessException
    {
        alfabankService.endTransaction( contractId, amount, url, mode, params );
        
        populate();
    }

    @Override
    public List<? extends BGAbstractTransaction> getRecentTransactions()
    {
        try
        {
            SearchResult<Transaction> result = recentTransactionList.get();
            if( result != null )
            {
                return result.getList();
            }
        }
        catch( Exception ex )
        {
            logger.error( ex.getMessage(), ex );
        }

        return null;
    }
}