/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
 */
package ru.bitel.mybgbilling.modules.payment;

import java.io.Serializable;
import java.math.BigDecimal;

import javax.inject.Named;
import javax.xml.ws.Holder;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.common.BGIllegalAccessException;
import ru.bitel.bgbilling.common.BGIllegalArgumentException;
import ru.bitel.bgbilling.common.BGMessageException;
import ru.bitel.bgbilling.kernel.contract.autopayment.common.bean.Autopayment;
import ru.bitel.bgbilling.kernel.contract.autopayment.common.bean.ContractAutopaymentMode;
import ru.bitel.bgbilling.modules.paymaster.common.service.PayMasterService;
import ru.bitel.common.model.MapHolder;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.scope.SessionModuleScoped;

@Named
@SessionModuleScoped
@BGInjection
public class PaymasterPaymentBean
    extends AbstractPaymentBean
    implements PaymentServiceProviderBean, Serializable
{
    @BGInject
    private PayMasterService payMasterService;

    @Override
    public void beginTransaction( int contractId, BigDecimal amount, Holder<String> url, Holder<String> mode, Holder<MapHolder<String, String>> params )
        throws BGException
    {
        payMasterService.beginTransaction( contractId, amount, url, mode, params );
    }

    @Override
    public void endTransaction( int contractId, BigDecimal amount, Holder<String> url, Holder<String> mode, Holder<MapHolder<String, String>> params )
        throws BGException, BGMessageException, BGIllegalArgumentException, BGIllegalAccessException
    {
        payMasterService.endTransaction( contractId, amount, url, mode, params );
    }

    @Override
    public Autopayment autopaymentGet( int contractId )
        throws BGException, BGMessageException, BGIllegalArgumentException, BGIllegalAccessException
    {
        return payMasterService.autopaymentGet( contractId );
    }

    @Override
    public void autopaymentDeactivate( int contractId )
        throws BGException, BGMessageException, BGIllegalArgumentException, BGIllegalAccessException
    {
        payMasterService.autopaymentDeactivate( contractId );
    }

    @Override
    public void beginAutopaymentTransaction( int contractId, BigDecimal amount, Holder<String> url, Holder<String> mode,
                                             Holder<MapHolder<String, String>> params,
                                             ContractAutopaymentMode autopaymentMode )
        throws BGException, BGMessageException, BGIllegalArgumentException, BGIllegalAccessException
    {
        payMasterService.beginAutopaymentTransaction( contractId, amount, url, mode, params, autopaymentMode );
    }

}
