/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.modules.payment;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.inject.Inject;
import javax.inject.Named;
import javax.xml.ws.Holder;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import groovy.util.ConfigObject;
import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.common.BGIllegalAccessException;
import ru.bitel.bgbilling.common.BGIllegalArgumentException;
import ru.bitel.bgbilling.common.BGMessageException;
import ru.bitel.bgbilling.modules.yoomoney.common.service.YoomoneyService;
import ru.bitel.common.Utils;
import ru.bitel.common.model.KeyValue;
import ru.bitel.common.model.MapHolder;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.scope.SessionModuleScoped;
import ru.bitel.mybgbilling.kernel.payment.PaymentBean;

@Named
@SessionModuleScoped
@BGInjection
public class YoomoneyPaymentBean
	extends AbstractPaymentBean
	implements PaymentServiceProviderBean, Serializable
{
	private static final Logger logger = LogManager.getLogger();

	@Inject
	private PaymentBean paymentBean;

	@BGInject
	private YoomoneyService yoomoneyService;

	private Map<String, List<KeyValue<String, String>>> paymentTypesMap = new HashMap<>();

	@Override
	public void beginTransaction( int contractId, BigDecimal amount, Holder<String> url, Holder<String> mode, Holder<MapHolder<String, String>> params )
		throws BGException
	{
		yoomoneyService.beginTransaction( contractId, amount, url, mode, params );
	}

	@Override
	public void endTransaction( int contractId, BigDecimal amount, Holder<String> url, Holder<String> mode, Holder<MapHolder<String, String>> params )
		throws BGException, BGMessageException, BGIllegalArgumentException, BGIllegalAccessException
	{
		yoomoneyService.endTransaction( contractId, amount, url, mode, params );
	}

	@Override
	public List<KeyValue<String, String>> getPaymentTypes()
	{
		return paymentTypesMap.computeIfAbsent( paymentBean.getPaymentProviderId(), k -> {

			String paymentTypes = "";

            ConfigObject config = paymentBean.getPaymentItem() != null ? paymentBean.getPaymentItem().getConfig() : null;
            if( config != null )
            {
                paymentTypes = (String)config.get( "paymentTypes" );
            }

            if( Utils.isBlankString( paymentTypes ) && paymentBean.getPaymentProvider() != null )
            {
                config = paymentBean.getPaymentProvider().getConfig();
                if( config != null )
                {
                    paymentTypes = (String)config.get( "paymentTypes" );
                }
            }

			List<KeyValue<String, String>> result = null;

			if( Utils.notBlankString( paymentTypes ) )
			{
				try
				{
					result = Arrays.asList( paymentTypes.split( "\\s*;\\s*" ) ).stream().map( a -> a.split( "\\s*:\\s*" ) ).map( a -> new KeyValue<>( a[0], a[1] ) )
								   .collect( Collectors.toList() );
				}
				catch( Exception ex )
				{
					logger.error( "Error parse paymentTypes: " + ex.getMessage() );
				}
			}

			if( result == null || result.isEmpty() )
			{
				return Arrays.asList( new KeyValue<>( "PC", "Оплата со счета Яндекс.Денег" ) );
			}

			return result;
		} );
	}
}
