/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.modules.phone;

import java.io.Serializable;
import java.time.LocalDate;
import java.util.Set;
import java.util.function.Supplier;

import javax.inject.Inject;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.w3c.dom.Document;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Lazy;
import ru.bitel.common.model.Page;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.MyApplicationBean.HttpRequest;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.scope.SessionModuleScoped;
import ru.bitel.mybgbilling.kernel.common.utils.Utils;

@Named
@SessionModuleScoped
@BGInjection
public class PhoneAccountBean
	extends AbstractBean
	implements Serializable
{
	private static final long serialVersionUID = 1540623437095557110L;

	private static final Logger logger = LogManager.getLogger();

	@Inject
	private PhoneBean phoneBean;

	@Inject
	private transient HttpRequest httpRequest;

	private Supplier<Document> total;
	private Supplier<Page> totalPage = Lazy.of( () -> new Page( 1, 15 ) );

	private Supplier<Document> destinations;
	private Supplier<Page> destinationsPage = Lazy.of( () -> new Page( 1, 15 ) );

	private Supplier<Document> services;
	private Supplier<Page> servicesPage = Lazy.of( () -> new Page( 1, 15 ) );

	@Override
	protected void init()
		throws BGException
	{
		super.init();
	}

	public void populate()
		throws BGException
	{
		switch( Utils.maskNull( navigationBean.getSubPage() ) )
		{
			case "destination":
				populateDestinations();
				break;

			case "service":
				populateServices();
				break;

			default:
				populateTotal();
				break;
		}
	}

	public Document getTotal()
	{
		return total != null ? total.get() : null;
	}

	public Page getTotalPage()
	{
		return totalPage.get();
	}

	public void populateTotal()
		throws BGException
	{
		Set<Integer> clientItemIds = phoneBean.getAccountIds();

		Page page = getTotalPage();
		LocalDate dateFrom = phoneBean.getDateFrom();
		LocalDate dateTo = phoneBean.getDateTo();

		total = Async.of( () -> httpRequest.requestXml( "phone", getModuleId(), "ReportAmount", getContractId(),
														"items", Utils.toString( clientItemIds ),
														"date1", dateFrom,
														"date2", dateTo,
														"pageIndex", page.getPageIndex(),
														"pageSize", page.getPageSize() ) );

		this.totalPage = phoneBean.getPage( total, page );
	}

	public Document getDestinations()
	{
		return destinations != null ? destinations.get() : null;
	}

	public Page getDestinationsPage()
	{
		return destinationsPage.get();
	}

	public void populateDestinations()
		throws BGException
	{
		Set<Integer> clientItemIds = phoneBean.getAccountIds();
		LocalDate dateFrom = phoneBean.getDateFrom();
		LocalDate dateTo = phoneBean.getDateTo();
		Page page = getDestinationsPage();

		destinations = Async.of( () -> httpRequest.requestXml( "phone", getModuleId(), "ReportDirect", getContractId(),
															   "items", Utils.toString( clientItemIds ),
															   "date1", dateFrom,
															   "date2", dateTo,
															   "pageIndex", page.getPageIndex(),
															   "pageSize", page.getPageSize() ) );

		this.destinationsPage = phoneBean.getPage( destinations, page );
	}

	public Document getServices()
	{
		return services != null ? services.get() : null;
	}

	public Page getServicesPage()
	{
		return servicesPage.get();
	}

	public void populateServices()
		throws BGException
	{
		Set<Integer> clientItemIds = phoneBean.getAccountIds();
		LocalDate dateFrom = phoneBean.getDateFrom();
		LocalDate dateTo = phoneBean.getDateTo();
		Page page = getServicesPage();

		services = Async.of( () -> httpRequest.requestXml( "phone", getModuleId(), "ReportService", getContractId(),
														   "items", Utils.toString( clientItemIds ),
														   "date1", dateFrom,
														   "date2", dateTo,
														   "pageIndex", page.getPageIndex(),
														   "pageSize", page.getPageSize() ) );

		this.servicesPage = phoneBean.getPage( services, page );
	}
}
