package ru.bitel.mybgbilling.modules.softline;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.modules.softline.common.SoftLineUtils;
import ru.bitel.bgbilling.modules.softline.common.bean.Product;
import ru.bitel.bgbilling.modules.softline.common.bean.ProductCategory;
import ru.bitel.bgbilling.modules.softline.common.bean.SoftLineSubscription;
import ru.bitel.bgbilling.modules.softline.common.bean.SubscriptionStatus;
import ru.bitel.bgbilling.modules.softline.common.bean.SubscriptionSuspendReason;
import ru.bitel.bgbilling.modules.softline.common.service.ProductService;
import ru.bitel.bgbilling.modules.softline.common.service.SoftLineSubscriptionService;
import ru.bitel.common.Utils;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;

import javax.enterprise.context.SessionScoped;
import javax.faces.application.FacesMessage;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.inject.Named;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Named
@SessionScoped
@BGInjection
public class SoftLineBean
    extends AbstractBean
    implements Serializable
{
    @BGInject
    private SoftLineSubscriptionService subscriptionService;

    @BGInject
    private ProductService productService;

    private List<ProductCategory> productCategories;
    //заведённые продукты в модуле, для установки в комбобокс
    private List<Product> softLineProducts;

    //активные подписки пользователя для таблицы
    private UserSubscription[] contractSubscriptions;

    //сюда сохраняется id выбранного в комбобоксе для подписки продукта
    private Product selectedProduct;
    private int selectedProductId;

    //вынес этот контекст сюда, чтобы можно было в ModuleExecutor получить данные для подключения к серверу
    private static ExternalContext externalContext;

    private String userEmail;

    private boolean showConfirmPanel = false;

    @Override
    protected void init()
        throws BGException
    {
        super.init();

        productCategories = productService.getProductCategories( null ).getList();
        //добавляем категорию для продуктов, у которых категория не установлена
        //productCategories.add( new ProductCategory(0, "Другое") );

        softLineProducts = productService.getProducts( null ).getList();

        loadActiveSubscriptions();

        FacesContext facesContext = FacesContext.getCurrentInstance();
        externalContext = facesContext.getExternalContext();
    }

    private void loadActiveSubscriptions()
        throws BGException
    {
        List<SoftLineSubscription> activeSubscriptions = subscriptionService.searchSubscriptions( null, null, customerBean.getContractId(), -1, SubscriptionStatus.ACTIVE, -1 ).getList();

        List<UserSubscription> userSubscriptions = new ArrayList<>();

        subs:
        for( SoftLineSubscription sub : activeSubscriptions )
        {
            if( sub.getEndDate() != null && sub.getEndDate().before( new Date() ) )
                continue;

            for( Product product : softLineProducts )
            {
                if( sub.getProductId() == product.getId() )
                {
                    userSubscriptions.add( new UserSubscription( sub.getId(), product.getTitle(), product.getPrice().toString(), sub.getStartDate(), sub.getEndDate() ) );
                    continue subs;
                }
            }
        }

        contractSubscriptions = new UserSubscription[userSubscriptions.size()];
        userSubscriptions.toArray(contractSubscriptions);
    }

    /**
     * Запрос на активацию подписки.
     * @throws BGException - Если активация недопустима по балансу или иной причине
     */
    public void activate()
        throws BGException
    {
        if( selectedProduct == null )
        {
            throw new BGException("Произошла ошибка. Подписка не активирована.");
        }

        if( Utils.isBlankString( userEmail) )
        {
            throw new BGException( "Укажите email для отправки данных!" );
        }
        //попытка активации подписки
        SoftLineSubscription subscription = new SoftLineSubscription();
        subscription.setContractId( getContractId() );
        subscription.setProductId( selectedProduct.getId() );
        subscription.setStartDate( new Date() );
        subscription.setLastModifyDate( new Date() );
        subscription.setEmail( userEmail );

        SoftLineUtils.setEndDate( subscription, getProductOfSubscription( subscription ) );

        subscriptionService.activateSubscription( subscription );
        //Отправляем всё на почту пользователю
        subscriptionService.sendDownloadDataOnUserEmail( subscription, userEmail );

        //обновляем страницу после активации подписки
        loadActiveSubscriptions();

        doHideConfirmPanel();

        FacesContext.getCurrentInstance().addMessage( null, new FacesMessage( "Подписка успешно оформлена" ) );
    }

    /**
     * Устанавливаем дату приостановки
     * @param subscriptionId - id приостанавливаемой подписки
     * @throws BGException - в случае, если не удалось приостановить подписку, например, из-за недоступности систем SoftLine
     */
    public void suspend( int subscriptionId )
        throws BGException
    {
        List<SoftLineSubscription> subscriptions = subscriptionService.searchSubscriptions( null, null, getContractId(), subscriptionId, SubscriptionStatus.ACTIVE, -1 ).getList();
        if( subscriptions != null && subscriptions.size() > 0 )
        {
            SoftLineSubscription subscription = subscriptions.get( 0 );
            if( subscription.getEndDate() == null )
            {
                SoftLineUtils.setEndDate( subscription, getProductOfSubscription( subscription ) );
            }
            subscription.setStatus( SubscriptionStatus.SUSPEND.getCode() );
            subscription.setSuspendReason( SubscriptionSuspendReason.user_refuse );
            subscriptionService.updateSubscription( subscription );
        }

        //обновляем страницу после приостановки подписки
        loadActiveSubscriptions();

        FacesContext.getCurrentInstance().addMessage(null, new FacesMessage( "Подписка успешно приостановлена" ) );
    }

    private Product getProductOfSubscription( SoftLineSubscription subscription )
    {
        for( Product product : softLineProducts )
        {
            if( product.getId() == subscription.getProductId() )
            {
                return product;
            }
        }

        return null;
    }

    public List<Product> productsOfCategory( int productCategoryId )
        throws BGException
    {
        return productService.getProductsOfCategory( productCategoryId );
    }

    // одной строкой возвращает цену и периода. Например, 7 дней - 150р или 1 месяц - 470 р
    public String getPriceOfProductPeriod( Product product )
    {
        String result = "";

        BigDecimal price = product.getPrice();
        switch( product.getPeriodType() )
        {
            case DAY:
            {
                int daysCount = product.getAmountOfDays();
                String prefix;
                if( daysCount == 1 )
                {
                    prefix = " день - ";
                }
                else if( daysCount > 4 )
                {
                    prefix = " дней - ";
                }
                else
                    prefix = " дня - ";

                result += daysCount + prefix + price + "р";
                break;
            }
            case MONTH:
            {
                result += "1 месяц - " + price + "р";
                break;
            }
            case YEAR:
            {
                result += "1 год - " + price + "р";
                break;
            }
        }

        return result;
    }

    //============= геттеры и сеттеры для доступа к данным бина ==============
    //просто возвращает строку взависимости от типа периода продукта в подписке для корректного отображения в ЛК
    public String getPriceTypeDescription( Product product )
    {
        switch( product.getPeriodType() )
        {
            case MONTH: return "1 месяц";
            case YEAR: return "1 год";
            case DAY: return product.getAmountOfDays() + " дней";
            default: return "";
        }
    }

    public String getPriceOfProduct( Product product )
    {
        String postFix = "";
        switch( product.getPeriodType() )
        {
            case DAY: postFix = "/день"; break;
            case MONTH: postFix = "/месяц"; break;
            case YEAR: postFix = "/год"; break;
        }
        return "Стоимость: " + product.getPrice() + postFix;
    }

    public int getSelectedProductId()
    {
        return selectedProductId;
    }

    public void setSelectedProductId( int selectedProductId )
    {
        this.selectedProductId = selectedProductId;
    }

    //возввращает линк для получения изображения выбранного для подписки продукта
    public String getURLSelectedProductPicture()
    {
        return "/softlineexecutor/?moduleId=" + getModuleId() + "&productId=" + selectedProductId + "&typeProduct=0";
    }

    public String getURLProductPicture( int productId )
    {
        return  "/softlineexecutor/?moduleId=" + getModuleId() + "&productId=" + productId + "&typeProduct=0";
    }

    public String getURLProductCategoryPicture( int productCategoryId )
    {
        return "/softlineexecutor/?moduleId=" + getModuleId() + "&productId=" + productCategoryId + "&typeProduct=1";
    }

    public UserSubscription[] getContractSubscriptions()
    {
        return contractSubscriptions;
    }

    public void setContractSubscriptions( UserSubscription[] contractSubscriptions )
    {
        this.contractSubscriptions = contractSubscriptions;
    }

    public Product getSelectedProduct()
    {
        return selectedProduct;
    }

    public void setSelectedProduct( Product selectedProduct )
    {
        this.selectedProduct = selectedProduct;
    }

    public List<Product> getSoftLineProducts()
    {
        return softLineProducts;
    }

    public void setSoftLineProducts( List<Product> softLineProducts )
    {
        this.softLineProducts = softLineProducts;
    }

    public List<ProductCategory> getProductCategories()
    {
        return productCategories;
    }

    public void setProductCategories( List<ProductCategory> productCategories )
    {
        this.productCategories = productCategories;
    }

    public String getUserEmail()
    {
        return userEmail;
    }

    public void setUserEmail( String userEmail )
    {
        this.userEmail = userEmail;
    }

    public static ExternalContext getExternalContext()
    {
        return externalContext;
    }

    //========  Всё, что ниже, для скрытия и показа панелей ========
    public void doShowConfirmPanel( int productId )
    {
        selectedProduct = softLineProducts.stream().filter( p -> p.getId() == productId ).findFirst().get();

        this.showConfirmPanel = true;
    }

    public void doHideConfirmPanel()
    {
        this.showConfirmPanel = false;
    }

    public boolean isShowConfirmPanel()
    {
        return showConfirmPanel;
    }
}
