/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.modules.tv;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;

import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.common.BGMessageException;
import ru.bitel.bgbilling.kernel.directory.api.common.bean.Directory;
import ru.bitel.bgbilling.modules.tv.common.bean.TvAccount;
import ru.bitel.bgbilling.modules.tv.common.bean.TvAccountSpec;
import ru.bitel.bgbilling.modules.tv.common.service.TvAccountService;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Lazy;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.scope.SessionModuleScoped;
import ru.bitel.oss.systems.inventory.product.common.bean.Product;
import ru.bitel.oss.systems.inventory.product.common.bean.ProductOffering;
import ru.bitel.oss.systems.inventory.product.common.bean.ProductSpec;
import ru.bitel.oss.systems.inventory.product.common.service.ProductService;
import ru.bitel.oss.systems.order.product.common.service.ProductOrderService;

@Named
@SessionModuleScoped
@BGInjection
public class TvBean
	extends AbstractBean
	implements Serializable
{
	private static final long serialVersionUID = -4594660879612619488L;

	private static final Logger logger = LogManager.getLogger();

	@BGInject
	private TvAccountService tvAccountService;

	@BGInject(type = TvAccountSpec.class)
	private Directory<TvAccountSpec> tvAccountSpecDirectory;

	@BGInject(module = false)
	private ProductService productService;

	@BGInject(module = false)
	private ProductOrderService productOrderService;

	@BGInject(type = ProductSpec.class, module = false)
	private Directory<ProductSpec> productSpecDirectory;

	int moduleId;

	private Async<List<TvAccount>> tvAccountList;

	private int tvAccountId;
	private TvAccount tvAccount;

	private Async<List<Product>> productList;
	private Lazy<List<Product>> historyProductList;

	private Lazy<List<ProductOffering>> productOfferingList;
	private Async<List<ProductOffering>> availableProductOfferingList;
	private String regData;

	@Override
	protected void init()
		throws BGException
	{
		super.init();

		this.moduleId = navigationBean.getModuleId();

		tvAccountList = Async.of( () -> Optional.ofNullable( tvAccountService.tvAccountTree( getContractId() ) ).map( TvAccount::getChildren ).orElse( null ) );
	}

	public int getTvAccountId()
	{
		return tvAccountId;
	}

	public void setTvAccountId( int tvAccountId )
		throws BGException
	{
		this.tvAccountId = tvAccountId;

		this.tvAccount = null;

		for( TvAccount tvAccount : getTvAccountList() )
		{
			if( tvAccount.getId() == tvAccountId )
			{
				this.tvAccount = tvAccount;
				break;
			}
		}

		if( tvAccount == null )
		{
			this.tvAccountId = 0;
		}
	}

	public TvAccount getTvAccount()
	{
		return tvAccount;
	}

	public List<TvAccount> getTvAccountList()
		throws BGException
	{
		return Optional.ofNullable( this.tvAccountList ).map( Async::get ).orElse( Collections.emptyList() );
	}

	public TvAccountSpec getTvAccountSpec( int id )
		throws BGException
	{
		logger.info( "getTvAccountSpec" );
		return tvAccountSpecDirectory.get( id );
	}

	public List<Product> getProductList()
	{
		return productList != null ? productList.get() : null;
	}

	public List<Product> getHistoryProductList()
	{
		return historyProductList != null ? historyProductList.get() : null;
	}

	public List<ProductOffering> getProductOfferingList()
	{
		return productOfferingList != null ? productOfferingList.get() : null;
	}

	public List<ProductOffering> getAvailableProductOfferingList()
	{
		return availableProductOfferingList != null ? availableProductOfferingList.get() : null;
	}

	public void populate()
		throws BGException
	{
		final Calendar calendar = new GregorianCalendar();
		calendar.add( Calendar.MINUTE, -10 );

		final int tvAccountId = this.tvAccountId;

		if( tvAccountId > 0 )
		{
			productList = Async.of( () -> Utils.maskNull( fillProductSpecName( productService.productList( moduleId, getContractId(), tvAccountId, false, null, null, calendar.getTime(), null, false, true ) ) ) );
			historyProductList = Lazy.of( () -> Utils.maskNull( fillProductSpecName( productService.productList( moduleId, getContractId(), tvAccountId, false, null, new Date(), null, null, false, false ) ) ) );
		}
		else
		{
			productList = Async.of( () -> Collections.emptyList() );
			historyProductList = Lazy.of( () -> Collections.emptyList() );
		}

        productOfferingList = Lazy.of( () -> Utils.maskNull( productOrderService.productOfferingList( moduleId, getContractId(), tvAccountId, 0, new Date(), false, true ) ) );

		availableProductOfferingList = Async.of( () -> {
			List<ProductOffering> availableTariffOptionList = new ArrayList<ProductOffering>();

			for( ProductOffering o : this.productOfferingList.get() )
			{
				if( o.isEnable() )
				{
					availableTariffOptionList.add( o );
				}
			}

			return availableTariffOptionList;
		} );
		regData="";
	}

	private List<Product> fillProductSpecName( List<Product> productList )
		throws BGException
	{
		if( productList == null || productList.size() == 0 )
		{
			return productList;
		}

		for( Product p : productList )
		{
			ProductSpec productSpec = productSpecDirectory.get( p.getProductSpecId() );
			if( productSpec != null )
			{
				p.setProductSpecTitle( productSpec.getTitle() );
			}
		}

		return productList;
	}

	public void activate( int productSpecId, int productSpecActivationModeId )
		throws BGException
	{
		if( productSpecId <= 0 )
		{
			throw new BGMessageException( "Не выбран продукт" );
		}

		if( productSpecActivationModeId <= 0 )
		{
			throw new BGMessageException( "Не выбран режим активации" );
        }

        if( tvAccountId <= 0
            || Optional.ofNullable( this.tvAccountList.get() )
                .map( list -> list.stream() )
                .orElseGet( Stream::empty )
                .noneMatch( a -> a.getId() == tvAccountId ) )
        {
            throw new BGMessageException( "Не выбран аккаунт" );
        }

		try
		{
			Product product = new Product();
			product.setContractId( getContractId() );
			product.setAccountId( tvAccountId );
			product.setProductSpecId( productSpecId );
			product.setActivationModeId( productSpecActivationModeId );
			//product.setTimeFrom( new Date() );
			product.setComment( "Активировано клиентом" );

			productOrderService.productActivate( product, null, true, true );

			addInfoMessage( null, "Подписка успешно активирована" );
		}
		finally
		{
			populate();
		}
	}

	public void deactivate( int productId )
		throws BGException
	{
		try
		{
			productOrderService.productDeactivate( getContractId(), productId, new Date(), true, true, false );

			addInfoMessage( null, "Подписка успешно деактивирована" );
		}
		finally
		{
			populate();
		}
	}

	public void reactivate( int productId )
		throws BGException
	{
		try
		{
			productOrderService.productReactivate( getContractId(), productId, true );

			addInfoMessage( null, "Подписка успешно реактивирована" );
		}
		finally
		{
			populate();
		}
	}
	
    public void registration()
        throws BGException
    {
        TvAccount account = tvAccountService.tvAccountGet( getContractId(), tvAccountId );
        if ( account == null )
        {
            return;
        }
        regData = "Логин: " + account.getLogin() + " [пароль: " + account.getPassword() + "]";
    }
	
	public String getRegData()
	{
	    return regData;
	}
}
