package ru.bitel.mybgbilling.modules.voice;

import java.io.IOException;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import javax.activation.DataHandler;
import javax.enterprise.context.SessionScoped;
import javax.inject.Inject;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.modules.voice.common.bean.VoiceDestination;
import ru.bitel.bgbilling.modules.voice.common.bean.VoiceSession;
import ru.bitel.bgbilling.modules.voice.common.bean.VoiceSessionFilter;
import ru.bitel.bgbilling.modules.voice.common.bean.report.SessionDetail;
import ru.bitel.bgbilling.modules.voice.common.bean.report.SessionDetailReportItem;
import ru.bitel.bgbilling.modules.voice.common.service.VoiceDestinationService;
import ru.bitel.bgbilling.modules.voice.common.service.VoiceSessionService;
import ru.bitel.common.TimeUtils;
import ru.bitel.common.model.Page;
import ru.bitel.common.model.Result;
import ru.bitel.mybgbilling.kernel.common.AbstractConversationBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.utils.Log;
import ru.bitel.mybgbilling.kernel.common.utils.Utils;

@Named
@SessionScoped
@BGInjection
public class VoiceSessionBean
    extends AbstractConversationBean
{
    private static final Logger logger = Log.getLog();
    private Date dateFrom;
    private Date dateTo;
    private Page page;
    
    @Inject
    private VoiceAccountBean accountBean;
    @BGInject
    private VoiceSessionService sessionService;
    @BGInject
    private VoiceDestinationService destService;
    
    private Result<VoiceSession> sessionLogList;
    private Result<VoiceSession> sessionList;
    private Result<SessionDetail<SessionDetail<SessionDetailReportItem>>> detail;
    
    private boolean onlyCharged;
    
	@Override
    protected void init()
        throws BGException
    {
        super.init();

        Calendar calendar = new GregorianCalendar();
        TimeUtils.clear_HOUR_MIN_MIL_SEC( calendar );
        calendar.set( Calendar.DATE, 1 );
        dateFrom = calendar.getTime();
        calendar.add( Calendar.MONTH, 1 );
        calendar.add( Calendar.DATE, -1 );
        dateTo = calendar.getTime();

        logger.info( "init" );
    }
    
    public void populate()
        throws BGException
    {
        
        if( page == null )
        {
            page = new Page( 1, 25 );
        }
        VoiceSessionFilter filter = getFilter();
        
        
        Result<VoiceSession> result;

        result = sessionLogList = sessionService.voiceSessionLogContractList( filter, page );

        if( result != null && result.getPage() != null )
        {
            page = result.getPage();
        }
    }

    private VoiceSessionFilter getFilter()
        throws BGException
    {
        VoiceSessionFilter filter = new VoiceSessionFilter();
        filter.setContractIds( Collections.singleton( customerBean.getContractId() ) );        
        filter.setAccountIds( accountBean.getAccountIds() );
        filter.setServiceIds( Collections.emptySet() );
        filter.setHourFrom( dateFrom );                
        filter.setHourTo( TimeUtils.getNextDay(  dateTo ) );
        filter.setOnlyCharged( onlyCharged );
        
        return filter;
    }
    
    public Result<SessionDetail<SessionDetail<SessionDetailReportItem>>> getDetail()
    {
        return detail;
    }
    
    public void populateDetail()
        throws BGException
    {
        
        if( page == null )
        {
            page = new Page( 1, 25 );
        }
        VoiceSessionFilter filter = getFilter();
        
        
        
        detail = sessionService.voiceSessionDetailReport( filter, page );

        if( detail != null && detail.getPage() != null )
        {
            page = detail.getPage();
        }
        
    }
    
    public List<VoiceSession> getVoiceSessionLogList()
    {
        logger.info( "getVoiceSessionLogList" );
        return sessionLogList != null ? sessionLogList.getList() : null;
    }

    public List<VoiceSession> getVoiceSessionList()
    {
        logger.info( "getVoiceSessionList" );
        return sessionList != null ? sessionList.getList() : null;
    }

    public Date getDateFrom()
    {
        return dateFrom;
    }

    public void setDateFrom( Date dateFrom )
    {
        this.dateFrom = dateFrom;
    }

    public Date getDateTo()
    {
        return dateTo;
    }

    public void setDateTo( Date dateTo )
    {
        this.dateTo = dateTo;
    }

    public Page getPage()
    {
        return page;
    }

    public void setPage( Page page )
    {
        this.page = page;
    }
    
    public String getDestination( int destId ) throws BGException 
    {
        if ( destId > 0 )
        {
            //TODO подумать как сделать опимально - кеш и т.п
            VoiceDestination dest = destService.voiceDestinationGet( destId );
            if ( dest != null )
            {
                return dest.getTitle();
            }
        }
        
        return null;
    }
    public void export( String type )
        throws BGException, IOException
    {
        VoiceSessionFilter filter = getFilter();
        DataHandler handler = sessionService.voiceSessionDetailExport( filter, type );
        String fileName = Utils.maskBlank( handler.getName(), "report_session_" +TimeUtils.format( dateFrom, "dd.MM.yyyy" ) + "." + type );        
        writeToOutput( handler.getInputStream(), fileName, handler.getContentType(), true );
    }
    
    public boolean isOnlyCharged()
    {
        return onlyCharged;
    }

    public void setOnlyCharged( boolean onlyCharged )
    {
        this.onlyCharged = onlyCharged;
    }
    
}
