/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.modules.voiceip;

import java.io.IOException;
import java.io.Serializable;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeFormatterBuilder;
import java.time.format.FormatStyle;
import java.util.Set;
import java.util.function.Supplier;

import javax.inject.Inject;
import javax.inject.Named;

import org.w3c.dom.Document;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Lazy;
import ru.bitel.common.model.Page;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.MyApplicationBean.HttpRequest;
import ru.bitel.mybgbilling.kernel.common.TransferData.HTTPDataSource;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.scope.SessionModuleScoped;
import ru.bitel.mybgbilling.kernel.common.utils.Utils;

@Named
@SessionModuleScoped
@BGInjection
public class VoiceIpAccountBean
	extends AbstractBean
	implements Serializable
{
	private static final long serialVersionUID = 1540623437095557110L;

	@Inject
	private VoiceIpBean voiceIpBean;

	@Inject
	private transient HttpRequest httpRequest;

	private Supplier<Document> total;
	private Supplier<Page> totalPage = Lazy.of( () -> new Page( 1, 15 ) );

	private Supplier<Document> destinations;
	private Supplier<Page> destinationsPage = Lazy.of( () -> new Page( 1, 15 ) );

	@Override
	protected void init()
		throws BGException
	{
		super.init();
	}

	public void populate()
		throws BGException
	{
		switch( Utils.maskNull( navigationBean.getSubPage() ) )
		{
			case "destination":
				populateDestinations();
				break;

			default:
				populateTotal();
				break;
		}
	}

	public Document getTotal()
	{
		return total != null ? total.get() : null;
	}

	public Page getTotalPage()
	{
		return totalPage.get();
	}

	public void populateTotal()
		throws BGException
	{
		int callType = voiceIpBean.getCallType();
		LocalDate dateFrom = voiceIpBean.getDateFrom();
		LocalDate dateTo = voiceIpBean.getDateTo();
		Page page = getTotalPage();

		total = Async.of( () -> httpRequest.requestXml( "voiceip", getModuleId(), "LoginsAmount", getContractId(),
														"date1", dateFrom,
														"date2", dateTo,
														"direct", callType,
														"pageIndex", page.getPageIndex(),
														"pageSize", page.getPageSize() ) );

		this.totalPage = voiceIpBean.getPage( total, page );
	}

	public Document getDestinations()
	{
		return destinations != null ? destinations.get() : null;
	}

	public Page getDestinationsPage()
	{
		return destinationsPage.get();
	}

	public void populateDestinations()
		throws BGException
	{
		Set<Integer> clientItemIds = voiceIpBean.getAccountIds();
		int callType = voiceIpBean.getCallType();
		LocalDate dateFrom = voiceIpBean.getDateFrom();
		LocalDate dateTo = voiceIpBean.getDateTo();
		Page page = getDestinationsPage();

		destinations = Async.of( () -> httpRequest.requestXml( "voiceip", getModuleId(), "LoginDirect", getContractId(),
															   "id", Utils.toString( clientItemIds ),
															   "date1", dateFrom,
															   "date2", dateTo,
															   "direct", callType,
															   "pageIndex", page.getPageIndex(),
															   "pageSize", page.getPageSize() ) );

		this.destinationsPage = voiceIpBean.getPage( destinations, page );
	}

	public void exportDestinations( String type )
		throws BGException, IOException
	{
		if( !"csv".equals( type ) && !"html".equals( type ) )
		{
			type = "csv";
		}

		Set<Integer> clientItemIds = voiceIpBean.getAccountIds();
		int callType = voiceIpBean.getCallType();
		LocalDate dateFrom = voiceIpBean.getDateFrom();
		LocalDate dateTo = voiceIpBean.getDateTo();
		Page page = getDestinationsPage();

		HTTPDataSource dataSource = httpRequest.request( "voiceip", getModuleId(), "LoginDirect", getContractId(),
														 "id", Utils.toString( clientItemIds ),
														 "date1", dateFrom,
														 "date2", dateTo,
														 "direct", callType,
														 "pageIndex", page.getPageIndex(),
														 "pageSize", page.getPageSize(),
														 "show_all", 1,
														 "output", type,
														 "contentType", "html" );

		DateTimeFormatter formatter = new DateTimeFormatterBuilder().appendLocalized( FormatStyle.MEDIUM, null ).toFormatter( localeBean.getLocale() );
		String fileName = Utils.maskBlank( dataSource.getName(), "report_destination_" + formatter.format( dateFrom ) + "." + type );

		writeToOutput( dataSource.getInputStream(), fileName, dataSource.getContentType(), true );
	}

	public void exportTotal( String type )
		throws BGException, IOException
	{
		if( !"csv".equals( type ) && !"html".equals( type ) )
		{
			type = "csv";
		}

		int callType = voiceIpBean.getCallType();
		LocalDate dateFrom = voiceIpBean.getDateFrom();
		LocalDate dateTo = voiceIpBean.getDateTo();
		Page page = getTotalPage();

		HTTPDataSource dataSource = httpRequest.request( "voiceip", getModuleId(), "LoginsAmount", getContractId(),
														 "date1", dateFrom,
														 "date2", dateTo,
														 "direct", callType,
														 "pageIndex", page.getPageIndex(),
														 "pageSize", page.getPageSize(),
														 "show_all", 1,
														 "output", type,
														 "contentType", "html" );

		DateTimeFormatter formatter = new DateTimeFormatterBuilder().appendLocalized( FormatStyle.MEDIUM, null ).toFormatter( localeBean.getLocale() );
		String fileName = Utils.maskBlank( dataSource.getName(), "report_total_" + formatter.format( dateFrom ) + "." + type );

		writeToOutput( dataSource.getInputStream(), fileName, dataSource.getContentType(), true );
	}
}
