/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.modules.voiceip;

import java.io.IOException;
import java.io.Serializable;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeFormatterBuilder;
import java.time.format.FormatStyle;
import java.util.Set;
import java.util.function.Supplier;

import javax.inject.Inject;
import javax.inject.Named;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Lazy;
import ru.bitel.common.model.Page;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.MyApplicationBean.HttpRequest;
import ru.bitel.mybgbilling.kernel.common.TransferData.HTTPDataSource;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.common.scope.SessionModuleScoped;
import ru.bitel.mybgbilling.kernel.common.utils.Utils;
import ru.bitel.mybgbilling.kernel.common.utils.XmlUtils;

@Named
@SessionModuleScoped
@BGInjection
public class VoiceIpSessionBean
	extends AbstractBean
	implements Serializable
{
	private static final long serialVersionUID = 1540623437095557110L;

	@Inject
	private VoiceIpBean voiceIpBean;

	@Inject
	private transient HttpRequest httpRequest;

	private Supplier<Page> page = Lazy.of( () -> new Page( 1, 15 ) );

	private Supplier<Document> sessions;

	@Override
	protected void init()
		throws BGException
	{
		super.init();
	}

	public Page getPage()
	{
		return page.get();
	}

	public void populate()
		throws BGException
	{
		Set<Integer> loginIds = voiceIpBean.getAccountIds();
		int callType = voiceIpBean.getCallType();
		LocalDate dateFrom = voiceIpBean.getDateFrom();
		LocalDate dateTo = voiceIpBean.getDateTo();
		Page page = getPage();

		sessions = Async.of( () -> {

			Document doc = httpRequest.requestXml( "voiceip", getModuleId(), "LoginSessions", getContractId(),
												   "id", Utils.toString( loginIds ),
												   "date1", dateFrom,
												   "date2", dateTo,
												   "direct", callType,
												   "pageIndex", page.getPageIndex(),
												   "pageSize", page.getPageSize() );

			for( Element e : XmlUtils.selectElements( doc, "/data/table/data/row" ) )
			{
				String a = e.getAttribute( "from_to" );
				String[] aa = a.split( "\\s*/\\s*" );
				if( aa.length > 1 )
				{
					e.setAttribute( "from", aa[0] );
					e.setAttribute( "to", aa[1] );
				}
				else
				{
					e.setAttribute( "from", a );
				}
			}

			return doc;
		} );

		this.page = voiceIpBean.getPage( sessions, page );
	}

	public Document getSessions()
	{
		return sessions != null ? sessions.get() : null;
	}

	public void export( String type )
		throws BGException, IOException
	{
		if( !"csv".equals( type ) && !"html".equals( type ) )
		{
			type = "csv";
		}

		Set<Integer> loginIds = voiceIpBean.getAccountIds();
		int callType = voiceIpBean.getCallType();
		LocalDate dateFrom = voiceIpBean.getDateFrom();
		LocalDate dateTo = voiceIpBean.getDateTo();
		Page page = getPage();

		HTTPDataSource dataSource = httpRequest.request( "voiceip", getModuleId(), "LoginSessions", getContractId(),
														 "id", Utils.toString( loginIds ),
														 "date1", dateFrom,
														 "date2", dateTo,
														 "direct", callType,
														 "pageIndex", page.getPageIndex(),
														 "pageSize", page.getPageSize(),
														 "show_all", 1,
														 "output", type,
														 "contentType", "html" );

		DateTimeFormatter formatter = new DateTimeFormatterBuilder().appendLocalized( FormatStyle.MEDIUM, null ).toFormatter( localeBean.getLocale() );
		String fileName = Utils.maskBlank( dataSource.getName(), "report_session_" + formatter.format( dateFrom ) + "." + type );

		writeToOutput( dataSource.getInputStream(), fileName, dataSource.getContentType(), true );
	}
}
