/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.plugins.bonus;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.enterprise.context.SessionScoped;
import javax.inject.Inject;
import javax.inject.Named;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.kernel.admin.plugincfg.common.PluginItem;
import ru.bitel.bgbilling.kernel.admin.plugincfg.common.PlugincfgService;
import ru.bitel.bgbilling.kernel.contract.balance.common.bean.ChargeType;
import ru.bitel.bgbilling.kernel.directory.api.common.bean.Directory;
import ru.bitel.bgbilling.kernel.tariff.option.common.bean.TariffOption;
import ru.bitel.bgbilling.kernel.tariff.option.common.bean.TariffOptionActivateMode;
import ru.bitel.bgbilling.plugins.bonus.common.bean.BonusBalance;
import ru.bitel.bgbilling.plugins.bonus.common.bean.BonusPayment;
import ru.bitel.bgbilling.plugins.bonus.common.service.BonusService;
import ru.bitel.common.Preferences;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Lazy;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.Configuration;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;

@Named
@SessionScoped
@BGInjection
public class BonusBean	extends AbstractBean
{
//	private static final Logger logger = LogManager.getLogger();
	@BGInject
	private BonusService bonusService;
	@BGInject
	private PlugincfgService plugincfgService;
	@Inject
    private Configuration configuration;
	
	@BGInject(type = ChargeType.class, module = false)
    private Directory<ChargeType> chargeDir;
	
	private Async<Boolean> isBonusPluginEnebledOfContract; // включен ли плагин на данном договоре
	private Async<BigDecimal> balance;
	private Async<BigDecimal> balanceNotActive;
	private Async<List<BonusBalance>> bonusBalanceList;
	private Lazy<List<BonusPayment>> bonusPaymentList;
	private Lazy<List<BonusCharge>> bonusChargeList;
	
	private Async<Set<Integer>> optionIdPayOnlyBonuses;

	@Override
	protected void init()		throws BGException
	{
		super.init();
		populate();
	}

	public void populate()		throws BGException
	{
	    this.isBonusPluginEnebledOfContract = Async.of( ( ) -> bonusService.pluginInclude( getContractId() ) );
	    
		Date now = new Date();
		this.balance = Async.of( ( ) -> bonusService.balance( getContractId(), now ) );
		this.balanceNotActive = Async.of( () -> Utils.maskNull( bonusService.balanceNotActive( getContractId(), now ) ) );
		this.bonusBalanceList = Async.of( () -> Utils.maskNull( bonusService.balanceList( getContractId(), now ) ) );
		this.bonusPaymentList = Lazy.of( () -> Utils.maskNull( bonusService.paymentList( getContractId(), null, null ) ) );
		
		this.bonusChargeList = Lazy.of( ( ) -> 
		{ 
			List<BonusCharge> result = new ArrayList<>();
			List<ru.bitel.bgbilling.plugins.bonus.common.bean.BonusCharge> list = Utils.maskNull( bonusService.chargeList( getContractId(), null, null ) );
			
			for( ru.bitel.bgbilling.plugins.bonus.common.bean.BonusCharge charge : list )
			{
			    ChargeType contractChargeType = chargeDir.get( charge.getTypeId() );
			    String typeTitle = contractChargeType != null ? contractChargeType.getTitle() : "Неизвестный тип!"; 
				result.add( new BonusCharge( typeTitle, charge.getSum(), charge.getDate() ) );
			}
			return result;
		} );
		
		final PluginItem bonusPlugin = configuration.getPluginMap().get().get( "ru.bitel.bgbilling.plugins.bonus" );
		this.optionIdPayOnlyBonuses = Async.of( () -> 
		{
            if( bonusPlugin != null )
            {
                PluginItem bonusPluginN = plugincfgService.getPlugin( bonusPlugin.getId() );// получаем с сервера снова, но уже с заполненным конфигом. Может есть более православный метод получения конфига плагина?
                if( bonusPluginN.getConfig() != null )
                {
                    Preferences bonusSetup = new Preferences( bonusPluginN.getConfig(), "\r\n" );
                    return Utils.toIntegerSet( bonusSetup.get( "tariffOptionsPaymentOnlyWithBonuses" ) );
                }
            }
            
            return new HashSet<>();
		} );
		
	}
	
	public boolean isBonusPluginEnebledOfContract() throws BGException
	{
	    return isBonusPluginEnebledOfContract.get();
	}
	
	public void populateBonusBalance()
    {
	    this.balance = Async.of( ( ) -> bonusService.balance( getContractId(), new Date() ) );
	    this.bonusBalanceList = Async.of( () -> Utils.maskNull( bonusService.balanceList( getContractId(), new Date() ) ) );
    }
	
	public BigDecimal getBonusBalance() throws BGException
	{
		return balance.get();
	}
	
	public BigDecimal getBonusBalanceNotActive()	throws BGException
	{
		return balanceNotActive.get();
	}
	
	public List<BonusBalance> getBonusBalanceList()	throws BGException
	{
		return bonusBalanceList.get();
	}
	
	public List<BonusPayment> getBonusPaymentList()	throws BGException
	{
		return bonusPaymentList.get();
	}
	
	public List<BonusCharge> getBonusChargeList()	throws BGException
	{
		return bonusChargeList.get();
	}
	
	public Set<Integer> getOptionIdPayOnlyBonuses()  throws BGException
	{
	    return optionIdPayOnlyBonuses.get();
	}
	
	public Map<Integer, String> getModeBonusMap( List<TariffOption> availableTariffOptionList ) throws BGException
	{
	    Map<Integer, String> modeBonusMap = new HashMap<>();
	    for( TariffOption o : availableTariffOptionList )
        {
            for( TariffOptionActivateMode m : o.getActivateModeList() )
            {
                if( m.getChargeSumma().compareTo( BigDecimal.ZERO ) != 1 ) // если <= 0, то идем дальше
                    continue;
                if( bonusService.percentOfBonus( m.getChargeTypeId() ) <= 0 ) // если процент не установлен, то идем дальше
                    continue;
                boolean payOnlyBonus = getOptionIdPayOnlyBonuses().contains( o.getId() );
                BigDecimal maxBonusRealSumMayBePay = payOnlyBonus ? m.getChargeSumma() : bonusService.bonusSummaMax( getContractId(), m.getChargeTypeId(), m.getChargeSumma(), false );
                if( maxBonusRealSumMayBePay.compareTo( BigDecimal.ZERO ) == 1 )
                {
                    modeBonusMap.put( m.getId(), Utils.formatCost( maxBonusRealSumMayBePay ) + " " + configuration.getCurrency().getSymbol() + " бонусами" );
                }
            }
        }        
	    return modeBonusMap;
	}
	
	public void activateTO( int contractId, int tariffOptionId, int modeId ) throws BGException
	{
	    bonusService.activateTO( contractId, tariffOptionId, modeId );
	    populateBonusBalance();
	}
	
}
