/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.plugins.helpdesk;

import static ru.bitel.bgbilling.plugins.helpdesk.common.bean.HelpDeskConstants.*;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import javax.enterprise.context.SessionScoped;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.plugins.helpdesk.common.bean.Topic;
import ru.bitel.bgbilling.plugins.helpdesk.common.service.HelpdeskParamService;
import ru.bitel.bgbilling.plugins.helpdesk.common.service.HelpdeskService;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Lazy;
import ru.bitel.common.model.IdTitle;
import ru.bitel.common.model.MapHolder;
import ru.bitel.common.model.Page;
import ru.bitel.common.model.Result;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;

@Named
@SessionScoped
@BGInjection
public class HelpdeskBean
	extends AbstractBean
	implements Serializable
{
	private static final long serialVersionUID = 7407972534197614292L;

	private static final Logger logger = LogManager.getLogger();

	@BGInject
	private HelpdeskService helpdeskService;

	@BGInject
	private HelpdeskParamService helpdeskParamService;

	private Async<MapHolder<IdTitle, List<IdTitle>>> categoryMap;
	private Async<List<IdTitle>> categoryList;
	private Async<Map<Integer, List<IdTitle>>> subCategoryMap;

	private Lazy<List<Topic>> topicList;
	private Lazy<Page> topicListPage = defaultPage();
	
	private Supplier<Integer> unreadTopicCount;

	private Async<Result<Topic>> topicHistoryList;
	private Lazy<Page> topicHistoryListPage = defaultPage();

	@javax.validation.constraints.Min(value = 0)
	private int id;

	private String title;

	private String message;

	private Date dateFrom;
	private Date dateTo;

	private static Lazy<Page> defaultPage()
	{
		return Lazy.of( () -> new Page( 1, 15 ) );
	}

	private int paramNotificationMode;
	private String paramNotificationParamEmail;
	private String paramNotificationParamPhone;
    private String paramNotificationParamSms;
    private String paramNotificationParamTelegram;

	public int getId()
	{
		return id;
	}

	public void setId( int id )
	{
		this.id = id;
	}

	public String getTitle()
	{
		return title;
	}

	public void setTitle( String title )
	{
		this.title = title;
	}

	public String getMessage()
	{
		return message;
	}

	public void setMessage( String message )
	{
		this.message = message;
	}

	public Date getDateFrom()
	{
		return dateFrom;
	}

	public void setDateFrom( Date dateFrom )
	{
		this.dateFrom = dateFrom;
	}

	public Date getDateTo()
	{
		return dateTo;
	}

	public void setDateTo( Date dateTo )
	{
		this.dateTo = dateTo;
	}

	public Page getTopicListPage()
	{
		return topicListPage.get();
	}

	public void setTopicListPage( Page topicListPage )
	{
		this.topicListPage = Lazy.of( () -> topicListPage != null ? topicListPage : new Page( 1, 15 ) );
	}

	public Page getTopicHistoryListPage()
	{
		return topicHistoryListPage.get();
	}

	public void setTopicHistoryListPage( Page topicHistoryListPage )
	{
		this.topicHistoryListPage = Lazy.of( () -> topicHistoryListPage != null ? topicHistoryListPage : new Page( 1, 15 ) );
	}

	public int getParamNotificationMode()
	{
		return paramNotificationMode;
	}

	public void setParamNotificationMode( int paramNotificationMode )
	{
		this.paramNotificationMode = paramNotificationMode;
	}

	public String getParamNotificationParamEmail()
	{
		return paramNotificationParamEmail;
	}

	public void setParamNotificationParamEmail( String paramNotificationParamEmail )
	{
		this.paramNotificationParamEmail = paramNotificationParamEmail;
	}

	public String getParamNotificationParamPhone()
	{
		return paramNotificationParamPhone;
	}

	public void setParamNotificationParamPhone( String paramNotificationParamPhone )
	{
		this.paramNotificationParamPhone = paramNotificationParamPhone;
	}

	public String getParamNotificationParamSms()
    {
        return paramNotificationParamSms;
    }

    public void setParamNotificationParamSms( String paramNotificationParamSms )
    {
        this.paramNotificationParamSms = paramNotificationParamSms;
    }

    public String getParamNotificationParamTelegram()
    {
        return paramNotificationParamTelegram;
    }

    public void setParamNotificationParamTelegram( String paramNotificationParamTelegram )
    {
        this.paramNotificationParamTelegram = paramNotificationParamTelegram;
    }

    public List<IdTitle> getCategoryList()
	{
		return categoryList.get();
	}

	public List<IdTitle> getSubCategoryList( int categoryId )
	{
		return subCategoryMap.get().get( categoryId );
	}

	public List<Topic> getTopicList()
	{
	    logger.info( "getTopicList" );

		return Optional.ofNullable( this.topicList ).map( Lazy::get ).orElse( null );
	}

	public List<Topic> getTopicHistoryList()
	{
		return Optional.ofNullable( this.topicHistoryList ).map( Async::get ).map( Result::getList ).orElse( null );
	}

	@Override
	protected void init()
		throws BGException
	{
		super.init();

		categoryMap = Async.of( () -> helpdeskService.categoryMapForContract( getContractId() ) );
		categoryList = Async.of( () -> new ArrayList<IdTitle>( categoryMap.get().getMap().keySet() ) );
		subCategoryMap = Async.of( () -> (Map<Integer, List<IdTitle>>)categoryMap.get().getMap().entrySet().stream()
																				 .collect( Collectors.toMap( ( e ) -> e.getKey().getId(), Map.Entry::getValue ) ) );

		populateParams();
	}

	public void populate()
		throws BGException
	{
		switch( navigationBean.getSubPage() )
		{
			case "history":
				populateHistory();
				break;

			default:
				populateTopics();
				break;
		}
	}

	public void populateTopics()
		throws BGException
	{
		final Page topicListPage = this.topicListPage.get();
		
		if( dateFrom != null || dateTo != null || !Utils.isBlankString( title ) || !Utils.isBlankString( message ) )
		{
			final Async<Result<Topic>> result = Async.of( () -> helpdeskService.topicList( getContractId(), id, false, dateFrom, dateTo, title, message, topicListPage ) );
			this.topicList = Lazy.of( () -> result.get().getList() );

			this.topicListPage = Lazy.of( () -> Optional.ofNullable( result ).map( Async::get ).map( Result::getPage ).orElseGet( () -> new Page( 1, 15 ) ) );

			this.unreadTopicCount =
				Async.of( () -> (int)Utils.maskNull( helpdeskService.topicList( getContractId(), id, false, null, null, null, null, new Page( 0, 0 ) ).getList() ).stream()
										  .filter( ( a ) -> a.getMessageNewManagerCount() > 0 ).count() )
					 .handle( ( a, e ) -> e != null ? a : 0 );
		}
		else
		{
			//XXX может убрать эту оптимизацию? а может и не надо. если потом усложниться что
			// и сделать отдельный метод веб-сервиса в биллинге, которые вернет нам unreadCount для абонента
			final Async<Result<Topic>> result = Async.of( () -> helpdeskService.topicList( getContractId(), id, false, dateFrom, dateTo, title, message, new Page( 0, 0 ) ) );

			//final Lazy<Page> topicListPageLazy = this.topicListPage = Lazy.of( () -> Optional.ofNullable( result ).map( Async::get ).map( Result::getPage ).orElseGet( () -> new Page( 1, 15 ) ) );
			final Lazy<Page> topicListPageLazy = this.topicListPage = Lazy.of( () -> {

				int count = result.get().getPage().getRecordCount();
				topicListPage.setRecordCount( count );
				if( topicListPage.getPageIndex() > topicListPage.getPageCount() )
				{
					topicListPage.setPageIndex( topicListPage.getPageCount() );
				}

				return topicListPage;
			} );

			this.topicList = Lazy.of( () -> result.get().getList().subList( topicListPageLazy.get().getPageFirstRecordNumber(),
																			Math.min( topicListPageLazy.get().getPageFirstRecordNumber() + topicListPageLazy.get().getPageSize(),
																					  topicListPageLazy.get().getRecordCount() ) ) );

			this.unreadTopicCount = Lazy.of( () -> (int)result.get().getList().stream().filter( ( a ) -> a.getMessageNewManagerCount() > 0 ).count() );

		}
	}

	public void populateHistory()
		throws BGException
	{
		final Page topicHistoryListPage = this.topicHistoryListPage.get();

		final Async<Result<Topic>> result =
			this.topicHistoryList = Async.of( () -> helpdeskService.topicList( getContractId(), id, true, dateFrom, dateTo, title, message, topicHistoryListPage ) );

		this.topicHistoryListPage = Lazy.of( () -> Optional.ofNullable( result ).map( Async::get ).map( Result::getPage ).orElseGet( () -> new Page( 1, 15 ) ) );
	}

	public void populateParams()
		throws BGException
	{
		MapHolder<String, String> paramsHolder = helpdeskParamService.getParams( getContractId() );
		Map<String, String> params = paramsHolder.getMap();

		paramNotificationMode = Utils.parseInt( params.get( COMM_MODE ) );
		paramNotificationParamPhone = Utils.maskNull( params.get( COMM_PHONE_VALUE ) );
		paramNotificationParamEmail = Utils.maskNull( params.get( COMM_EMAIL_VALUE ) );
        paramNotificationParamSms = Utils.maskNull( params.get( COMM_SMS_VALUE ) );
        paramNotificationParamTelegram = Utils.maskNull( params.get( COMM_TELEGRAM_VALUE ) );
	}

	public void saveParams()
		throws BGException
	{
		MapHolder<String, String> paramsHolder = helpdeskParamService.getParams( getContractId() );
		Map<String, String> params = paramsHolder.getMap();

		params.put( COMM_MODE, String.valueOf( paramNotificationMode ) );
		params.put( COMM_PHONE_VALUE, Utils.maskNull( paramNotificationParamPhone ) );
		params.put( COMM_EMAIL_VALUE, Utils.maskNull( paramNotificationParamEmail ) );
        params.put( COMM_SMS_VALUE, Utils.maskNull( paramNotificationParamSms ) );
        params.put( COMM_TELEGRAM_VALUE, Utils.maskNull( paramNotificationParamTelegram ) );

		helpdeskParamService.setParams( getContractId(), paramsHolder );
	}
	
	public int getUnreadTopicCount()
	{
		// для оптимизации просто возвращаем 0, если еще не вызывали populate
		if( this.unreadTopicCount == null )
		{
			return 0;
		}
		
		try
		{
			return this.unreadTopicCount.get();
		}
		catch( Exception ex )
		{
			logger.error( ex.getMessage(), ex );
		}
		
		return 0;
	}
}
