/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.plugins.helpdesk;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.Date;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;
import java.util.function.Supplier;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.enterprise.context.SessionScoped;
import javax.inject.Inject;
import javax.inject.Named;
import javax.servlet.http.Part;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import javax.xml.ws.Holder;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.common.BGIllegalAccessException;
import ru.bitel.bgbilling.common.BGIllegalArgumentException;
import ru.bitel.bgbilling.kernel.filestorage.common.bean.BGServerFile;
import ru.bitel.bgbilling.plugins.helpdesk.common.bean.Message;
import ru.bitel.bgbilling.plugins.helpdesk.common.bean.Topic;
import ru.bitel.bgbilling.plugins.helpdesk.common.service.HelpdeskService;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;

@Named
@SessionScoped
@BGInjection
public class HelpdeskTopicBean
	extends AbstractBean
	implements Serializable
{
	private static final long serialVersionUID = 9019360331010440881L;

	private static final Logger logger = LogManager.getLogger();

	@Inject
	private HelpdeskBean helpdeskBean;

	@BGInject
	private HelpdeskService helpdeskService;

	private volatile int topicId;

    private Supplier<Topic> topic = () -> null;

	private String newTopicTitlePrefix;

	@NotNull
	@Size(min = 1)
	private String newTopicTitle;

	private int newTopicNotificationMode;
	private String newTopicNotificationParam;

	private int newTopicCategoryId;
	private int newTopicSubCategoryId;

	@NotNull
	@Size(min = 1)
	private String newMessage;

	private Part newFile;
	private Part newFile2;
	private Part newFile3;
	private Part newFile4;
	private Part newFile5;

	private boolean onlyLast;

	public String getNewMessage()
	{
		return newMessage;
	}

	public void setNewMessage( String newMessage )
	{
		this.newMessage = newMessage;
	}

	public Part getNewFile()
	{
		return newFile;
	}

	public void setNewFile( Part newFile )
	{
		this.newFile = newFile;
	}

	public Part getNewFile2()
	{
		return newFile2;
	}

	public void setNewFile2( Part newFile2 )
	{
		this.newFile2 = newFile2;
	}

	public Part getNewFile3()
	{
		return newFile3;
	}

	public void setNewFile3( Part newFile3 )
	{
		this.newFile3 = newFile3;
	}

	public Part getNewFile4()
	{
		return newFile4;
	}

	public void setNewFile4( Part newFile4 )
	{
		this.newFile4 = newFile4;
	}

	public Part getNewFile5()
	{
		return newFile5;
	}

	public void setNewFile5( Part newFile5 )
	{
		this.newFile5 = newFile5;
	}

	public String getNewTopicTitlePrefix()
	{
		return newTopicTitlePrefix;
	}

	public void setNewTopicTitlePrefix( String newTopicTitlePrefix )
	{
		this.newTopicTitlePrefix = newTopicTitlePrefix;
	}

	public String getNewTopicTitle()
	{
		return newTopicTitle;
	}

	public void setNewTopicTitle( String newTopicTitle )
	{
		this.newTopicTitle = newTopicTitle;
	}

	public int getNewTopicNotificationMode()
	{
		return newTopicNotificationMode;
	}

	public void setNewTopicNotificationMode( int newTopicNotificationMode )
	{
		this.newTopicNotificationMode = newTopicNotificationMode;
	}

	public String getNewTopicNotificationParam()
	{
		return newTopicNotificationParam;
	}

	public void setNewTopicNotificationParam( String newTopicNotificationParam )
	{
		this.newTopicNotificationParam = newTopicNotificationParam;
	}

	public int getNewTopicCategoryId()
	{
		return newTopicCategoryId;
	}

	public void setNewTopicCategoryId( int newTopicCategoryId )
	{
		this.newTopicCategoryId = newTopicCategoryId;
	}

	public int getNewTopicSubCategoryId()
	{
		return newTopicSubCategoryId;
	}

	public void setNewTopicSubCategoryId( int newTopicSubCategoryId )
	{
		this.newTopicSubCategoryId = newTopicSubCategoryId;
	}

	public boolean isOnlyLast()
	{
		return onlyLast;
	}

	public void setOnlyLast( boolean onlyLast )
	{
		this.onlyLast = onlyLast;
	}

	public void populate()
	{
		topic = Async.of( () -> {

			Topic topic = helpdeskService.topicGet( topicId, true );

			helpdeskService.topicMarkReadByCustomer( getContractId(), topicId );

			return topic;
		} );
	}

	public void topic( final Topic topic )
		throws BGException
	{
		logger.info( "topic" );

		if( topic == null )
		{
			throw new BGIllegalArgumentException();
		}

		if( topic.getContractId() != getContractId() )
		{
			throw new BGIllegalArgumentException();
		}

		logger.info( "topicId: " + topic.getId() );

		this.topicId = topic.getId();

		//		navigationBean.setPage( "plugins/helpdesk/helpdeskTopic" );
		navigationBean.setSubPage( "topic" );

		onlyLast = true;

		populate();
	}

	public Topic getTopic()
	{
		return topic.get();
	}

	public List<Message> getMessageList()
	{
		return Utils.maskNull( topic.get().getMessages() );
	}

	public void addMessage()
		throws BGException
	{
		if( Utils.isBlankString( newMessage ) )
		{
			//FacesContext.getCurrentInstance().getClientIdsWithMessages()
			//	addErrorMessage( "newMessageForm:oldPassword", "AAAAAAAAAAAA" );
			return;
		}

		Message message = new Message();
		message.setTopicId( topicId );
		message.setUserIdFrom( 0 );
		message.setDateFrom( new Date() );
		message.setUuid( UUID.randomUUID().toString() );
		message.setBody( ru.bitel.mybgbilling.kernel.common.utils.Utils.escapeXmlNotValid( new StringBuilder(), newMessage ) );

		try
		{
			CompletableFuture.allOf( tryUpload( message, newFile ), tryUpload( message, newFile2 ), tryUpload( message, newFile3 ), tryUpload( message, newFile4 ),
									 tryUpload( message, newFile5 ) )
							 .join();
		}
		catch( Exception ex )
		{
			logger.error( ex.getMessage(), ex );
			addErrorMessage( null, ex.getCause().getMessage() );
		}

		int messageId = helpdeskService.messageUpdate( message );
		message.setId( messageId );

		newMessage = "";
		newFile = null;
		newFile2 = null;
		newFile3 = null;
		newFile4 = null;
		newFile5 = null;

		populate();
	}

	private CompletableFuture<Void> tryUpload( final Message message, final Part part )
		throws BGException
	{
		if( part == null )
		{
			return CompletableFuture.completedFuture( null );
		}

		final String fileName = getSubmittedFileName( part );

		logger.info( "Uploading " + fileName );

		return Async.of( () -> {

			final long size = part.getSize();

			DataHandler dh = new DataHandler( new DataSource()
			{
				@Override
				public String getContentType()
				{
					return "application/octet-stream";
				}

				@Override
				public InputStream getInputStream()
					throws IOException
				{
					return part.getInputStream();
				}

				@Override
				public String getName()
				{
					return fileName;
				}

				@Override
				public OutputStream getOutputStream()
					throws IOException
				{
					return null;
				}
			} );

			Holder<DataHandler> holder = new Holder<>( dh );

			helpdeskService.fileUpload( String.valueOf( message.getId() ), message.getUuid(), fileName, size, holder );
		} ).getFuture();
	}

	public void downloadFile( int fileId )
		throws BGException
	{
		if( fileId <= 0 )
		{
			throw new BGIllegalAccessException();
		}

		Holder<DataHandler> fileDataHandler = new Holder<DataHandler>();
		Holder<BGServerFile> serverFileHolder = new Holder<BGServerFile>();

		helpdeskService.fileDownload( fileId, getContractId(), serverFileHolder, fileDataHandler );

		//String titleCard = Base64.decode( Base64.decodeWeb( ge ), "utf-8" );

		String title = serverFileHolder.value.getTitle();

		writeToOutput( fileDataHandler.value, title, false );
	}

	public void newTopic()
		throws BGException
	{
		helpdeskBean.populateParams();

		this.newTopicNotificationMode = helpdeskBean.getParamNotificationMode();

		switch( this.newTopicNotificationMode )
		{
            case 4:
                this.newTopicNotificationParam = helpdeskBean.getParamNotificationParamTelegram();
                break;

            case 3:
                this.newTopicNotificationParam = helpdeskBean.getParamNotificationParamSms();
                break;
    
            case 2:
				this.newTopicNotificationParam = helpdeskBean.getParamNotificationParamEmail();
				break;

			case 1:
				this.newTopicNotificationParam = helpdeskBean.getParamNotificationParamPhone();
				break;

			default:
				this.newTopicNotificationParam = "";
				break;
		}
	}

	public void createTopic()
		throws BGException
	{
		if( Utils.isBlankString( newTopicTitle ) || Utils.isBlankString( newMessage ) )
		{
			return;
		}

		Topic topic = new Topic();
		topic.setContractId( getContractId() );

		if( Utils.isBlankString( newTopicTitlePrefix ) )
		{
			topic.setTitle( ru.bitel.mybgbilling.kernel.common.utils.Utils.escapeXmlNotValid( new StringBuilder(), newTopicTitle ) );
		}
		else
		{
			topic.setTitle( ru.bitel.mybgbilling.kernel.common.utils.Utils.escapeXmlNotValid( new StringBuilder(), newTopicTitlePrefix + ' ' + newTopicTitle ) );
		}

		topic.setCategoryId( newTopicCategoryId );
		topic.setSubcategoryId( newTopicSubCategoryId );
		topic.setComm( newTopicNotificationMode );
		topic.setCommValue( newTopicNotificationParam );

		Message message = new Message();
		message.setTopicId( topicId );
		message.setUserIdFrom( 0 );
		message.setDateFrom( new Date() );
		message.setBody( ru.bitel.mybgbilling.kernel.common.utils.Utils.escapeXmlNotValid( new StringBuilder(), newMessage ) );

		int[] topicMessageId = helpdeskService.topicCreate( topic, message );

		topic.setId( topicMessageId[0] );
		message.setId( topicMessageId[1] );

		this.topicId = topic.getId();

		try
		{
			CompletableFuture.allOf( tryUpload( message, newFile ), tryUpload( message, newFile2 ), tryUpload( message, newFile3 ), tryUpload( message, newFile4 ),
									 tryUpload( message, newFile5 ) )
							 .join();
		}
		catch( Exception ex )
		{
			logger.error( ex.getMessage(), ex );
			addErrorMessage( null, ex.getCause().getMessage() );
		}

		newMessage = "";
		newFile = null;
		newFile2 = null;
		newFile3 = null;
		newFile4 = null;
		newFile5 = null;

		topic( topic );

		helpdeskBean.populate();
	}

	public void closeTopic()
		throws BGException
	{
		helpdeskService.topicUpdateState( getContractId(), topicId, true );

		navigationBean.setSubPage( "" );

		helpdeskBean.populate();
	}

	public int getTopicId()
	{
		return topicId;
	}

	public void setTopicId( int topicId )
		throws BGException
	{
		if( topicId == this.topicId )
		{
			return;
		}

		Topic topic = helpdeskService.topicGet( topicId, true );

		if( topic == null )
		{
			throw new BGIllegalArgumentException();
		}

		if( topic.getContractId() != getContractId() )
		{
			throw new BGIllegalArgumentException();
		}
		
		topic( topic );
	}
	
    public Topic getTopic( int topicId )
        throws BGException
    {
        Topic result = helpdeskService.topicGet( topicId, true );
        if( result != null && result.getContractId() != getContractId() )
        {
            return null;
        }

        return result;
    }
}
