/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
 */
package ru.bitel.mybgbilling.plugins.helpdesk;

import javax.enterprise.context.RequestScoped;
import javax.enterprise.event.Observes;
import javax.enterprise.inject.Instance;
import javax.inject.Inject;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.plugins.helpdesk.common.bean.Topic;
import ru.bitel.mybgbilling.kernel.common.Configuration;
import ru.bitel.mybgbilling.kernel.common.UrlHandler.UrlEvent;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;
import ru.bitel.mybgbilling.kernel.menu.MenuBean;
import ru.bitel.mybgbilling.kernel.navigation.NavigationBean;
import ru.bitel.mybgbilling.kernel.ws.CustomerBean;

@Named
@RequestScoped
@BGInjection
public class HelpdeskUrlHandler
{
    private static final Logger logger = LogManager.getLogger();

    @Inject
    private Configuration configuration;

    @Inject
    private NavigationBean navigationBean;

    @Inject
    protected CustomerBean customerBean;

    @Inject
    protected MenuBean menuBean;

    @Inject
    private Instance<HelpdeskTopicBean> helpdeskTopicBean;

    public void handle( @Observes UrlEvent e )
        throws BGException
    {
        try
        {
            if( e.isStop() )
            {
                return;
            }
            
            if( helpdeskTopicBean.isUnsatisfied() )
            {
                return;
            }

            if( !configuration.isPluginEnabled( "helpdesk" ) )
            {
                return;
            }

            String module = e.getRequest().getParameter( "module" );
            String page = e.getRequest().getParameter( "page" );
            if( "ru.bitel.bgbilling.plugins.helpdesk".equals( module ) || "plugins/helpdesk/helpdesk".equals( page ) )
            {
                e.setStop( true );

                if( !menuBean.checkContainsPage( "plugins/helpdesk/helpdesk" ) )
                {
                    return;
                }

                int topicId = ru.bitel.common.Utils.parseInt( e.getRequest().getParameter( "topicId" ) );
                if( topicId <= 0 )
                {
                    navigationBean.setPage( "plugins/helpdesk/helpdesk" );
                    return;
                }

                Topic topic = helpdeskTopicBean.get().getTopic( topicId );
                if( topic == null || topic.getContractId() != customerBean.getContractId() )
                {
                    navigationBean.setPage( "plugins/helpdesk/helpdesk" );
                    return;
                }

                navigationBean.setPage( "plugins/helpdesk/helpdesk" );
                helpdeskTopicBean.get().topic( topic );
                return;
            }
        }
        catch( Exception ex )
        {
            logger.error( ex.getMessage(), ex );
        }
    }
}
