/*
 * Copyright (c) 2016 BiTel Inc. All rights reserved.
*/
package ru.bitel.mybgbilling.plugins.notification;

import java.util.List;
import java.util.function.Supplier;

import javax.enterprise.context.SessionScoped;
import javax.inject.Named;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.plugins.notification.common.bean.ContractNotification;
import ru.bitel.bgbilling.plugins.notification.common.service.ContractNotificationService;
import ru.bitel.common.Utils;
import ru.bitel.common.function.Async;
import ru.bitel.common.function.Lazy;
import ru.bitel.mybgbilling.kernel.common.AbstractBean;
import ru.bitel.mybgbilling.kernel.common.inject.BGInject;
import ru.bitel.mybgbilling.kernel.common.inject.BGInjection;

@Named
@SessionScoped
@BGInjection
public class NotificationBean
	extends AbstractBean
{
	private static final long serialVersionUID = -893744590855227814L;

	private static final Logger logger = LogManager.getLogger();

	@BGInject(module = false)
	private ContractNotificationService contractNotificationService;

	private Supplier<List<ContractNotification>> contractNotificationList;
	
	// для оптимизации, если еще не вызывали populate - возвращаем 0
	private Supplier<Integer> unreadCount = () -> 0;

	@Override
	protected void init()
		throws BGException
	{
		super.init();
	}

	public void populate()
		throws BGException
	{
		this.contractNotificationList = Async.of( () -> Utils.maskNull( contractNotificationService.contractNotificationList( getContractId() ) ) );

		this.unreadCount = Lazy.of( () -> this.contractNotificationList.get().stream().filter( ( a ) -> !a.isRead() ).toArray().length );
	}

	public List<ContractNotification> getContractNotificationList()
		throws BGException
	{
		if( contractNotificationList == null )
		{
			populate();
		}

		return contractNotificationList.get();
	}

	public void markRead( int id )
		throws BGException
	{
		contractNotificationService.contractNotificationMarkRead( getContractId(), id );

		for( ContractNotification n : contractNotificationList.get() )
		{
			if( n.getId() == id && n.isRead() )
			{
				return;
			}
		}

		populate();
	}

	public void deleteContractNotification( int id )
		throws BGException
	{
		contractNotificationService.contractNotificationDelete( getContractId(), id );

		populate();
	}

	public int getUnreadCount()
		throws BGException
	{
		try
		{
			return this.unreadCount.get();
		}
		catch( Exception ex )
		{
			logger.error( ex.getMessage(), ex );
		}
		
		return 0;
	}
}
